/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.firma.klient.webservice;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.*;

import org.apache.axiom.om.OMElement;
import org.apache.commons.lang.NotImplementedException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.communication.*;
import dk.gov.oiosi.communication.client.TransportController;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.configuration.ProfileMappingCollectionConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.raspprofile.communication.RaspRequest;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.RootCertificateConfig;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.security.ldap.LdapLookupFactory;
import dk.gov.oiosi.security.lookup.CertificateLoader;
import dk.gov.oiosi.security.lookup.ICertificateLookup;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationLookupFactory;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.uddi.*;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

/**
 * Invoke RASP call against ITST published RASP test service.
 */
public class OiosiRaspClient {

    private static Log log = LogFactory.getLog(OiosiRaspClient.class);
    public static final String PATH_INVOICE_XML = "./Resources/xml/OIOXML_invoice.xml";

    public static final String RaspConfigurationLive = "./Resources/RaspConfiguration.Live.xml";
    public static final String RaspConfigurationTest = "./Resources/RaspConfiguration.Test.xml";

    public enum UddiType {
    	
        Production (RaspConfigurationLive),
        NewProduction (RaspConfigurationLive),
        Test (RaspConfigurationTest),
        LocalJava_v1_2_1 (RaspConfigurationTest),
        LocalJava_v1_2_3 (RaspConfigurationTest),
        LocalJava_v1_2_4 (RaspConfigurationTest),
        LocalNet_v1_2_1 (RaspConfigurationTest),
        LocalNet_v1_2_3 (RaspConfigurationTest),
        LocalNet_v1_2_4 (RaspConfigurationTest),

        FOCES2Test("cfg/RaspConfiguration.Oces2.xml"),
        FOCES2_99018008("cfg/RaspConfiguration.99018008.xml"),
        FOCES2_99018080("cfg/RaspConfiguration.99018080.xml");
        
        private final String raspConfigurationFile;
        
        private UddiType(String configFile) {
			raspConfigurationFile = configFile;
        }
    }


    public UddiType uddiType;

    public String xmlDocumentUrl;

    public OiosiRaspClient() {
        this.uddiType = UddiType.Test;
        this.xmlDocumentUrl = PATH_INVOICE_XML;
    }

    public OiosiRaspClient(UddiType uddiType, String xmlDocumentUrl) {
        this.uddiType = uddiType;
        this.xmlDocumentUrl = xmlDocumentUrl;
    }


    /**
     * RASP Test application
     * <p/>
     * This test client goes through the following steps:
     * <p/>
     * 1. Load a document
     * 2. Validate the document
     * 3. Make an UDDI lookup to find the service address
     * 4. Use information from the UDDI response to download the server certificate via LDAP
     * 5. Check whether the downloaded certificate is revoked
     * 6. Send it to a service deployed on localhost (by default: the service_skeleton service)
     *
     * @param args Args.
     */
    public static void main(final String[] args)
    {
        new OiosiRaspClient().sendDocument();
    }

    public boolean sendDocument()
    {
        boolean result = false;

        log.info("Sending the document using config "+this.uddiType.name()+", config file: "+this.uddiType.raspConfigurationFile);
        ConfigurationHandler.setConfigurationFile(this.uddiType.raspConfigurationFile);
        
        try {
            try {
            	ConfigurationHandler.getInstance().executeStartupTasks();
            } catch (Throwable e) {
				/*
				 * Wrap it with Throwable, so both Exceptions and Errors are
				 * caught. If we switch back to old version of OIORASP, which
				 * does not have method executeStartupTasks, it will continue
				 * working.
				 */
            	log.warn("Failed to execute startup tasks", e);
            }

            log.info("Try to send message to service");

            // Set the path to the custom RASP configuration file.
            // This file contains setup for things like:
            //   - Addresses to OCSP, LDAP and UDDI servers
            //   - The path to the Axis2 modules needed (RASP, WS-Security and WS-ReliableMessaging)
            //   - Configuration for the different OIO UBL document types
            // ConfigurationHandler.setConfigurationFile("cfg/RaspConfiguration.xml");


            // Step 1: Load a document

            // Load the document from disc
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.load(xmlDocumentUrl);
            log.info("Document created from file - " + xmlDocumentUrl);

            // Search our configuration file for the document type we are currently about to send.
            // The UBL type can be inferred from the XML root element name of the file, and the configuration we get
            // contains paths to XML schemas, schematron stylesheets and XPath expressions used to find the EAN number for example
            DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
            DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);
            log.info("Document is known in configuration");


            // Step 2: Validate the document

            // Schema validate document before sending...
            SchemaValidator schemaValidator = new SchemaValidator();
            XmlSchema schema = new XmlSchema(documentTypeConfig.getSchemaPath());
            schemaValidator.schemaValidateXmlDocument(xmlDocument, schema);
            log.info("Document is schema valid");

            // Schematron validate Document before sending...
            SchematronValidator schematronValidator = new SchematronValidator(documentTypeConfig.getSchematronValidationConfig());
            schematronValidator.schematronValidateXmlDocument(xmlDocument);
            log.info("Document is schematron/logically valid");

            Uri endpointAddress = null;

            OcesX509Certificate ocesCertificateClient = null;
            OcesX509Certificate ocesCertificateServer = null;

            // Step 3: Make UDDI Lookup

            // Info about the service url (endpointAddress)
            // If the url ends with '/nemhandel' as in http://localhost:8080/Rasp-1.2.3/services/TestService/nemhandel
            // the endpoint accept all kinds of documents.
            // If the endpoint does not end with '/nemhandel' only the documents defined in the file
            // ~/dk.gov.oiosi.examples.service\WebContent\WEB-INF\services\TestService\META-INF\service.xml
            // in an element serviceGroup/service/operation
            // can be received.
            // At pressent only Invoice (OIOXML and OIOUBL2) is defined.
            //
            // The url for the service can be changed after deployment. The default is ~/services/TestService/nemhandel, but the TestService
            // can be changed to whatever suit you business.
            //  1. Deploy the war file.
            //  2. Manual rename the folder 'TestService' to the desired name (keep in mind, that the url should end on '/nemhandel'
            //     as described above.
            //  3. Manualy change the services.xml file located in
            //    ~/Tomcat 6.0\webapps\Rasp-1.2.3\WEB-INF\services\TestService\META-INF/services.xml
            //     Change the attribute serviceGroup/service@name='???' to the desired name

            String localWebserverTomcat = "http://localhost";

            switch (uddiType)
            {
                case Production:
                case NewProduction:
                case Test:
                case FOCES2Test:
                case FOCES2_99018008:
                case FOCES2_99018080:
                {
                    log.info("Sending the document though uddi lookup.");

                    Identifier identifier = Utilities.getReceiverEndpointKey(xmlDocument);

                    // Find profile definition id (based on profile name in xml document)
                    ProfileMappingCollectionConfig profileMappingConfig = ConfigurationHandler.getInstance().getProfileMappingCollectionConfig();

                    List<EndpointAddressTypeCode> endpointTypes = new LinkedList<EndpointAddressTypeCode>();
                    endpointTypes.add(EndpointAddressTypeCode.http);

                    LookupParameters parameters;
                    if (documentTypeConfig.getProfileIdXPath().isEmpty()) {
                        parameters = new LookupParameters(
                                identifier,
                                IdentifierUtility.getUddiIDFromString(documentTypeConfig.getServiceContractTModel()),
                                endpointTypes
                        );
                    } else {
                        UddiId profileUddiId = Utilities.getProfileId(xmlDocument, documentTypeConfig, profileMappingConfig);
                        List<UddiId> profileDefinitionIds = new ArrayList<UddiId>();
                        profileDefinitionIds.add(profileUddiId);


                        // Setup the UDDI lookup parameters
                        parameters = new LookupParameters(
                                identifier,
                                IdentifierUtility.getUddiIDFromString(documentTypeConfig.getServiceContractTModel()),
                                profileDefinitionIds,
                                endpointTypes
                        );
                    }

                    // Look it up...
                    log.info("Ask UDDI for: " + identifier);
                    IUddiLookupClient uddiClient = RegistryLookupClientFactory.getRegistryLookupClient();
                    List<UddiLookupResponse> uddiResultList = uddiClient.lookup(parameters);
                    log.info("Found # in uddi : " + uddiResultList.size());
                    UddiLookupResponse uddiResponse = null;
                    if (uddiResultList.size() == 0) {
                        log.error("No Information found in UDDI");
                        result = false;
                        return result;
                    }
                    else if (uddiResultList.size() == 1)
                    {
                        // Normal uddi lookup result
                        uddiResponse = (UddiLookupResponse) uddiResultList.get(0);
                    }
                    else
                    {
                        // Normally one one uddiResult is desired. However, the uddiResultList can contain multi endpoint,
                        // each endpoint match one acception protocol, multi endpoint can be return - and this is not an error

                        Map<EndpointAddressTypeCode, UddiLookupResponse>  map = new HashMap<EndpointAddressTypeCode, UddiLookupResponse>();
                        boolean validEndpointCollection = true;
                        int index = 0;

                        while(validEndpointCollection && index < uddiResultList.size())
                        {
                            uddiResponse = uddiResultList.get(index);
                            if(map.containsKey(uddiResponse.getEndpointAddress().getEndpointAddressTypeCode()))
                            {
                                // The result list contain two endpoins, using the same EndpointAddressTypeCode
                                // We do not know which one of theses endpoins that should be used.
                                validEndpointCollection = false;

                                log.error("Multi endpoint found in UDDI");
                                result = false;
                                return result;
                            }
                            else
                            {
                                map.put(uddiResponse.getEndpointAddress().getEndpointAddressTypeCode(), uddiResponse);
                            }

                            index++;
                        }

                        // even though there was multi endpoint, they were just different AddressTypeCode
                        // okay, in this example, we only accept EndpointAddressTypeCode.http, so this is only for showing, how
                        // is should be done

                        // we need to decide which endpoint we will use.
                        if(map.containsKey(EndpointAddressTypeCode.http))
                        {
                            // 1. choose is http
                            uddiResponse = map.get(EndpointAddressTypeCode.http);
                        }
                        else if(map.containsKey(EndpointAddressTypeCode.ftp))
                        {
                            // 2. is ftp - not used in rasp 1.2.4
                            uddiResponse = map.get(EndpointAddressTypeCode.ftp);
                        }
                        else if(map.containsKey(EndpointAddressTypeCode.https))
                        {
                            // 3. is https    - not used in rasp 1.2.4
                            uddiResponse = map.get(EndpointAddressTypeCode.https);
                        }
                        else if(map.containsKey(EndpointAddressTypeCode.email))
                        {
                            // 4. is email
                            uddiResponse = map.get(EndpointAddressTypeCode.email);
                        }
                        else if(map.containsKey(EndpointAddressTypeCode.other))
                        {
                            // 5. is other  - not used in rasp 1.2.4
                            uddiResponse = map.get(EndpointAddressTypeCode.other);
                        }
                        else
                        {
                            throw new NotImplementedException("Endpoint EndpointAddressTypeCode not implemented");

                        }
                    }

                    log.info("Found endpoint         : " + uddiResponse.getEndpointAddress().getKeyAsString());
                    log.info("Found server certificate Info : " + uddiResponse.getCertificateSubjectSerialNumber().getSerialNumberValue());

                    // Step 4: LDAP Lookup
                    // Get the certificate identifier from the UDDI results
                    CertificateSubject certificateSubject = uddiResponse.getCertificateSubjectSerialNumber();

                    // Use a factory to create an LDAP client, and download the certificate
                    ICertificateLookup lookup = LdapLookupFactory.createLdapLookupClient();
                    X509Certificate server = lookup.getCertificate(certificateSubject);
                    log.info("Found server certificate in LDAP : " + server.getSubjectDN());


                    // Step 5: Revocation Check

                    // Use a factory to create a revocation client, and check the certificate status
                    IRevocationLookup revocationLookup = RevocationLookupFactory.createRevocationLookupClient();
                    RevocationResponse revocationResponse = revocationLookup.checkCertificate(server);
                    log.info("Server certificate is valid : " + revocationResponse.isValid());

                    endpointAddress = uddiResponse.getEndpointAddress().getAsUri();

                    // Use the downloaded certificate as the server side credential
                    ocesCertificateServer = new OcesX509Certificate(server);

                    break;
                }

                case LocalJava_v1_2_1:
                {
                    log.info("Sending the document to the local Java 1.2.1 service.");

                    // if endpoint end withs /nemhandel, the service accept all documents
                    // else it only accept the document that is linked in the services.xml file
                    endpointAddress = new Uri(localWebserverTomcat + "/Rasp-1.2.1/services/TestService/nemhandel");
                    break;
                }
                case LocalJava_v1_2_3:
                {
                    log.info("Sending the document to the local Java 1.2.3 service.");
                    endpointAddress = new Uri(localWebserverTomcat + "/Rasp-1.2.3/services/TestService/nemhandel");
                    break;
                }
                case LocalJava_v1_2_4:
                {
                    log.info("Sending the document to the local Java 1.2.4 service.");
                    endpointAddress = new Uri(localWebserverTomcat + "/Rasp-1.2.4/services/TestService/nemhandel");
                    break;
                }
                case LocalNet_v1_2_1:
                {
                    log.info("Sending the document to the local .Net 1.2.1 service.");
                    // endpointAddress = new Uri(localWebserverIIS + "/Rasp-1.2.4/services/TestService/nemhandel");
                    throw new NotImplementedException("The uddi type '" + uddiType.toString() + "' is not supported.");
                }
                case LocalNet_v1_2_3:
                {
                    log.info("Sending the document to the local .Net 1.2.3 service.");
                    // endpointAddress = new Uri(localWebserverIIS + "/Rasp-1.2.4/services/TestService/nemhandel");
                    throw new NotImplementedException("The uddi type '" + uddiType.toString() + "' is not supported.");
                }
                case LocalNet_v1_2_4:
                {
                    log.info("Sending the document to the local .Net 1.2.4 service.");
                    // endpointAddress = new Uri(localWebserverIIS + "/Rasp-1.2.4/services/TestService/nemhandel");
                    throw new NotImplementedException("The uddi type '" + uddiType.toString() + "' is not supported.");
                }
                default:
                {
                    throw new NotImplementedException("The uddi type '" + uddiType.toString() + "' is unknown, not supported.");
                }
            }

			if ((uddiType == UddiType.LocalJava_v1_2_1 || uddiType == UddiType.LocalJava_v1_2_3 || uddiType == UddiType.LocalJava_v1_2_4)) {
                // Sending though local service, so we must use a keystore
                if(ocesCertificateServer == null)
                {
                    // just insured that one of the local service did not already defined a ocesCertificateServer
                    // Load the server certificate from keystore

                    ////////////////////////////////
                    // Note - This certificate must match the recieving certificate
                    //        else you will get the error.
                    //        General security error (Unexpected number of X509Data: for decryption (KeyId))
                    //        Meaning - The receiver can not decrypt the message
                    ////////////////////////////////

                    KeyStore ks = KeyStore.getInstance("JKS");
                    ks.load(new FileInputStream("./keys/OCES1_Test.jks"), "Test1234".toCharArray());
                    X509Certificate server = (X509Certificate) ks.getCertificate("function");
                    ocesCertificateServer = new OcesX509Certificate(server);
                }
            }

            // Endpoint identified
            log.info("Endpoint identified    : " + endpointAddress);

            if (endpointAddress.getAbsoluteUri().contains("/nemhandel")) {
                log.info("Destination service should accept all document types.");
            } else {
                log.info("Destination service only accept the document types defined in the ~/META-INF/service.xml.");
                // ~/dk.gov.oiosi.examples.service\WebContent\WEB-INF\services\TestService\META-INF\service.xml
            }

            /*
             * Load own certificate. This is used for retrieving sender id, when a confidential sender id is present
             * in the document to be sent. Note that this certificate is not the one used for signing and encryption.
             *
             * The certificate used for signing and encrypting is specified in the file policy.xml.
             */
            // Load the client certificate from keystore
            X509Certificate client;

            if (uddiType == UddiType.Production
                    || uddiType == UddiType.NewProduction
                    || uddiType == UddiType.Test
                    || uddiType == UddiType.FOCES2Test
                    || uddiType == UddiType.FOCES2_99018008
                    || uddiType == UddiType.FOCES2_99018080) {
                // load the certificate from windows key store

                // ToDo: Implement a way to retrieve the function certificate though the RaspConfiguration.xml file
                // Present we can not retrieve the function certificate, based on the settings in the
                // RaspConfiguration.xml file. The only certificate that is defined is the root certificate,
                // which can be retrieved as showed below.
                // So a small hack is used to get the correct certificate.
                // HACK: Retrieving the function certificate, based on the settings for the root certificate, by changing the aliase

                //RootCertificateConfig config = new RootCertificateConfig();
                List<RootCertificateConfig> rootCertificateConfigList = ConfigurationHandler.getInstance().getRootCertificateLocationCollection();

                if(rootCertificateConfigList.size()<1)
                {
                    throw new MissingCredentialsException("There is not defined any root certificate in the configuration file.");
                }

                // Note: this is just a exampel implementation
                // We always just use the fist certificate store, to find the desired functions cetificate,
                // If more then one rootCertificateConfig was defined, meaning more the one .jsp file
                // we could also has decided to use one of those.
                // Real production implementation must identify a way, to decide which certificate store to use
                int certificateIndex = 0;
                RootCertificateConfig rootCertificateConfig = rootCertificateConfigList.get(certificateIndex);
                rootCertificateConfig.setKeyLabel("function");
                // The 'rootCertificateConfig' is used to get the root certificate
                // However, we need the function certificate
                client = CertificateLoader.GetCertificateFromCertificateStoreInformation(rootCertificateConfig);

            }
            else if (uddiType == UddiType.LocalJava_v1_2_1
                    || uddiType == UddiType.LocalJava_v1_2_3
                    || uddiType == UddiType.LocalJava_v1_2_4
                    || uddiType == UddiType.LocalNet_v1_2_1
                    || uddiType == UddiType.LocalNet_v1_2_3) {

                KeyStore ks2 = KeyStore.getInstance("JKS");
                ks2.load(new FileInputStream("./keys/OCES_Test.jks"), "Test1234".toCharArray());

                //client = (X509Certificate) ks2.getCertificate("employee");
                client = (X509Certificate) ks2.getCertificate("function");

            }
            else {
                throw new Exception("The uddi type '" + uddiType.toString() + "' is unknown, not supported.");
            }

            ocesCertificateClient = new OcesX509Certificate(client);

            Date now = new Date();
            log.info("My certificate : " + ocesCertificateClient.getSubject().getSerialNumberValue() + " " +ocesCertificateClient.getSubject().getCn());
            if(now.before(ocesCertificateClient.getCertificate().getNotBefore())) {
                log.warn("My certificate is not yet active...! Activation date : "+ ocesCertificateClient.getCertificate().getNotBefore().toString());
            } else if (now.after(ocesCertificateClient.getCertificate().getNotAfter())) {
                log.warn("My certificate has expired...! Expire date : "+ ocesCertificateClient.getCertificate().getNotAfter().toString());
            } else {
                log.info("My certificate is valid, expire: "+ ocesCertificateClient.getCertificate().getNotAfter().toString());
            }

            log.info("Destination service certificate : " + ocesCertificateServer.getSubject().getSerialNumberValue() + " " +ocesCertificateServer.getSubject().getCn());
            if(now.before(ocesCertificateServer.getCertificate().getNotBefore())) {
                log.warn("Destination service certificate is not yet active...! Activation date : "+ ocesCertificateServer.getCertificate().getNotBefore().toString());
            } else if (now.after(ocesCertificateServer.getCertificate().getNotAfter())) {
                log.warn("Destination service certificate has expired...! Expire date : "+ ocesCertificateServer.getCertificate().getNotAfter().toString());
            } else {
                log.info("Destination service certificate is valid, expire: "+ ocesCertificateServer.getCertificate().getNotAfter().toString());
            }

            //Step 6: Send it
            // Create the credentials
            Credentials credentials = new Credentials(ocesCertificateClient, ocesCertificateServer);

            // Use policy defined in RaspConfiguration.xml
            SendPolicy sendPolicy = ConfigurationHandler.getInstance().getSendPolicy();
            log.info("SendPolicy loaded from configuration");

            // Create a request
            RaspRequest request = new RaspRequest(new Request(endpointAddress, credentials, sendPolicy));

            // Send request to service
            Response response = request.getResponse(new OiosiMessage(xmlDocument), "urn:uuid:" + System.nanoTime());
            /* NOTE    "Problem with sending soap message. Error during encryption" */
            /* If you get the error "Problem with sending soap message. Error during encryption" */
            /* you are property missing the JCE addon */

            /* From the installation guide :
                The java runtime environment must have Java Cryptography Extension (JCE) Unlimited Strength Jurisdiction Policy Files installed

                JDKs and JCE package can be found here – installation instructions are included in the downloaded packages.
                http://java.sun.com/javase/downloads/index.jsp */

            OMElement body = response.getResponseMessage().getMessageXml().getOMDocument();

            SignatureValidationProof signatureProof = response.getResponseMessage().getSignatureProof();

            log.info("Received response from service: " + (body == null ? "empty message" : body));
            log.info("SignatureValidationProof: " + signatureProof);

            // All okay, the document has been send and received okay
            result = true;
        } catch (Exception e) {
            log.error("Failed to send message to service : " + e.getMessage(), e);
        }

        try {
            // Let the ReliableMessaging session finish...
            long sleep = 10000;
            log.info("Now sleep for " + sleep + " milisec - Let the ReliableMessaging session finish...");
            Thread.sleep(sleep);
            TransportController.shutdown();
            log.info("ReliableMessaging session finish and transportController.shutdown.");
        }
        catch (Exception ignore)
        { }

        log.info("Done...");

        return result;
    }
}