/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Sogaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.firma.webservice;

import java.util.HashSet;
import java.util.Set;

import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.service.IServiceContract;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;

/**
 * This is an example of a RASP web service.
 * 
 * XML Schema validation, Schematron validation and security is performed in the
 * background by Axis handlers. This behavior can be configured in the file services.xml:
 * {@code
 * <oiosi:XMLTransformation>false</oiosi:XMLTransformation>
 * <oiosi:XMLSchematron>true</oiosi:XMLSchematron>
 * <oiosi:XMLValidation>true</oiosi:XMLValidation>
 * <oiosi:SignatureProof>true</oiosi:SignatureProof>
 * <oiosi:CustomHeaders>true</oiosi:CustomHeaders>
 * }
 * 
 * If any of these handlers are disabled, then manual checks must be performed in
 * the requestRespond method.
 * 
 * XMLTransformation is disabled by default, since it is only used when OIOUBL to OIOXML 
 * transformation is needed.
 */
public class MyWebService implements IServiceContract {

	/*
	 * Used for storing already received message ids.
	 * 
	 * In a real world scenario received message ids should be persisted,
	 * so that they are not lost if the web service is restarted.
	 */
	private static Set<String> receivedGUIDs = new HashSet<String>();
	
    public OiosiMessage requestRespond(OiosiMessage message) throws MainException {

        /*
         * Check that the document is not a duplicate
         */
    	if(receivedGUIDs.add(message.getMessageIdentifier())) {
    		
    		// Retrieve business document payload
			XmlDocument payload = message.getMessageXml();
        	
        	// Use payload here. In this example we simply print it out.
        	System.out.println(payload.getOMDocument());
    	}
    	else {
    		//The document was a duplicate. It should therefore not be used.
    	}

    	// Read the signature validation proof
    	SignatureValidationProof signatureProof = message.getSignatureProof();
    	System.out.println(signatureProof);
    	
        /*
         * Finally reply with an empty body (i.e. do not send a business document back)
         */
    	XmlDocument emptyBody = new XmlDocument();
    	OiosiMessage replyMessage = new OiosiMessage(emptyBody);
    	
    	// Set the soap action for the reply message if possible
        try {
            DocumentTypeConfigSearcher documentSearcher = new DocumentTypeConfigSearcher();
            DocumentTypeConfig documentTypeConfig = documentSearcher.findUniqueDocumentType(message.getMessageXml());
            replyMessage.setReplyAction(documentTypeConfig.getEndpointType().getReplyAction());
        } catch (Exception e) {
        	// Could not find reply action. Just don't set it then.
        }

        return replyMessage;
    }
}