/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.addressing;

import dk.gov.oiosi.exception.NullOrEmptyArgumentException;

/**
 * CVR Identifier.
 */
public class IdentifierCvr extends Identifier {

    /**
     * the CVR representation of Identifier.
     */
    private String cvrNumber;

    private static final String keyTypeValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/cvrNumber/";

    
    /**
     * CVR Identifer.
     * 
     * @param value the CVR representation of Identifier.
     */
    public IdentifierCvr(final String value) {
        set(value);
    }

    /**
     * Set and Validate CVR.
     * 
     * (non-Javadoc)
     * @see dk.gov.oiosi.addressing.Identifier#set(java.lang.String)
     */
    public void set(String value) {
        if (value == null || value.length() == 0) {
            throw new NullOrEmptyArgumentException("cvrNumber");
        }

        if (value.toLowerCase().startsWith("dk") && value.length() > 2) {
            value = value.substring(2);
        }

        
        if (value.length() != 8) {
            throw new IncorrectBusinessIdentifierException("Not a valid cvr number [" + value + "], length is not 8.");
        }
        try {
            Long.parseLong(value);
        } catch (NumberFormatException e) {
            throw new IncorrectBusinessIdentifierException("Not a valid cvr number [" + value + "], contains non digits.");
        }

        this.cvrNumber = value;
    }

    /**
     * Return the String representation of Identifier.
     * 
     * @return the cvr number as a string
     */
    public String getAsString() {
        return cvrNumber;
    }

    /**
     * Compare two Identifiers.
     * 
     * @param other
     *            Identifier to compare
     * @return true if equal
     */
    public boolean equals(IdentifierCvr other) {
        if (cvrNumber != null) {
            return cvrNumber.equals(other.getAsString());
        }
        return false;
    }
    
    @Override
    public String getType() {
        return "CVR";
    }

	@Override
	public String getKeyTypeValue() {
		return keyTypeValue;
	}

	@Override
	public boolean isAllowedInPublic() {
		return true;
	}
    
}
