/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.axis2.module.customheaders;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.addressing.IdentifierUtility;
import dk.gov.oiosi.axis2.module.RaspHandler;
import dk.gov.oiosi.axis2.module.RaspModule;
import dk.gov.oiosi.exception.AxisFaultUtil;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.policy.OiosiPolicyBean;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axiom.soap.SOAPHeaderBlock;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * Extracts custom headers for incoming messages.
 */
public class CustomHeaderInHandler extends RaspHandler {

    /**
     * local log.
     */
    private static final Log log = LogFactory.getLog(CustomHeaderInHandler.class);

    /**
     * Description.
     */
    private static HandlerDescription EMPTY_HANDLER_METADATA = new HandlerDescription("default Handler");

    /**
     * Description.
     */
    private HandlerDescription handlerDesc;

    /**
     * Constructor.
     */
    public CustomHeaderInHandler() {
        this.handlerDesc = EMPTY_HANDLER_METADATA;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#init(org.apache.axis2.description.HandlerDescription)
     */
    public void init(HandlerDescription handlerdesc) {
        this.handlerDesc = handlerdesc;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#invoke(org.apache.axis2.context.MessageContext)
     */
    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {
        if (log.isDebugEnabled()) {
            log.debug("CustomHeaderInHandler.invoke " + msgContext.getSoapAction());
        }

        // test...
        if (!msgContext.isEngaged(RaspModule.MODULE_NAME)) {
            log.debug("Rasp module not engaged - skip");
            return InvocationResponse.CONTINUE;
        }

        if (!msgContext.isServerSide()) {
            // we know in client what headers we sent...
            log.debug("client side - skip");
            return InvocationResponse.CONTINUE;
        }

        boolean rmMessage = RaspModule.getInstance().isRMNamespaceByAction(msgContext.getSoapAction());
        /*if (rmMessage) {
            // only for payload requests
            log.debug("only for payload - skip");
            return InvocationResponse.CONTINUE;
        }*/

        OiosiPolicyBean oiosiPolicyBean = getPolicyBean(msgContext);
        if (!oiosiPolicyBean.doCustomHeaders()) {
            log.debug(" - skip handling of custom headers in stack...");
            return InvocationResponse.CONTINUE;
        }

        SOAPEnvelope envelope = msgContext.getEnvelope();
        if (envelope == null) {
            return InvocationResponse.CONTINUE;
        }
        SOAPHeader header = envelope.getHeader();
        if (header == null) {
            return InvocationResponse.CONTINUE;
        }

        boolean foundIdentifier = false;
        boolean foundSenderID = false;
        boolean foundSenderType = false;
        boolean foundReceiverID = false;
        boolean foundReceiverType = false;
        boolean headerSwitched = false;


        List<?> list = header.getHeaderBlocksWithNSURI(Constants.RASP_CUSTOM_HEADER_NS);
        List<SOAPHeaderBlock> mandatoryHeaders = new ArrayList<SOAPHeaderBlock>();
        if (list != null) {
            String _RaspSenderPartyID = "";
            String _RASPSenderPartyType = "";
            String _RASPReceiverPartyID = "";
            String _RASPReceiverPartyType = "";
            for (Object object : list) {
                SOAPHeaderBlock shb = (SOAPHeaderBlock) object;
                // set processed to indicate we have handled header
                shb.setProcessed();

                if (Constants.RASP_MESSAGE_IDENTIFIER.equals(shb.getLocalName())) {
                    msgContext.setProperty(Constants.RASP_MESSAGE_IDENTIFIER, shb.getText());
                    foundIdentifier = true;
                    msgContext.setProperty(Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER, shb);
                }

                // Identify sender id
                if (Constants.RASP_SENDER_PARTY_IDENTIFIER.equals(shb.getLocalName())) {
                    _RaspSenderPartyID = shb.getText();
                    foundSenderID = true;
                }

                // Identify receiver ID
                if (Constants.RASP_RECEIVER_PARTY_IDENTIFIER.equals(shb.getLocalName())) {
                    _RASPReceiverPartyID = shb.getText();
                    foundReceiverID = true;
                }

                if (Constants.RASP_SENDER_PARTY_TYPE_HEADER.equals(shb.getLocalName())) {
                    _RASPSenderPartyType = shb.getText();
                    foundSenderType = true;
                }
                if (Constants.RASP_RECEIVER_PARTY_TYPE_HEADER.equals(shb.getLocalName())) {
                    _RASPReceiverPartyType = shb.getText();
                    foundReceiverType = true;
                }
            }
            /*if (list.size() > 0) {
                msgContext.setProperty(Constants.RASP_CUSTOM_HEADER_LIST, list);
            }*/


            if (foundReceiverID && foundSenderID && !headerSwitched && foundReceiverType && foundSenderType) {
                // Convert string id & type into identifier's
                Identifier idRaspSenderParty = IdentifierUtility.GetIdentifierFromKeyType(_RaspSenderPartyID, _RASPSenderPartyType);
                Identifier idRaspReceiverParty = IdentifierUtility.GetIdentifierFromKeyType(_RASPReceiverPartyID, _RASPReceiverPartyType);
                // Switch sender for receiver headers
                if ((null != idRaspSenderParty) & (null != idRaspReceiverParty)) {
                    msgContext.setProperty(Constants.RASP_RECEIVER_PARTY_IDENTIFIER, idRaspSenderParty);
                    mandatoryHeaders.add(createNewSoapHeader(Constants.RASP_RECEIVER_PARTY_IDENTIFIER, idRaspSenderParty.getAsString()));

                    msgContext.setProperty(Constants.RASP_SENDER_PARTY_IDENTIFIER, idRaspReceiverParty);
                    mandatoryHeaders.add(createNewSoapHeader(Constants.RASP_SENDER_PARTY_IDENTIFIER, idRaspReceiverParty.getAsString()));
                }
                headerSwitched = true;
            }

            // mandatory headers should be added to response - AxisServiceReceiver sets RASP_CUSTOM_HEADER_MANDATORY list as RASP_CUSTOM_HEADER_LIST for the response.
            if (mandatoryHeaders.size() > 0) {
                msgContext.setProperty(Constants.RASP_CUSTOM_HEADER_MANDATORY, mandatoryHeaders);
            }

        }

        // Enforce mandatory headers
        if ((!foundIdentifier && !rmMessage) || !foundReceiverID || !foundSenderID) {
            String msg = "Message receiver without mandatory Rasp Headers (identifier=" + foundIdentifier + ", sender="
                    + foundSenderID + ", receiver=" + foundReceiverID + ").";
            log.warn(msg);
            MainException e = new MainException(msg);
            e.setReceiverError(false);
            e.setSoapFaultAction("MissingHeaderFault");
            throw AxisFaultUtil.buildAxisFault(msgContext, e);
        } else {
            log.debug("All Rasp Headers found");
        }

        return InvocationResponse.CONTINUE;
    }

    private SOAPHeaderBlock createNewSoapHeader(String name, String value) {
        SOAPFactory soapFactory = OMAbstractFactory.getSOAP12Factory();
        OMNamespace ns = soapFactory.createOMNamespace(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_NS, dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_PREFIX);
        SOAPHeaderBlock newHeader = soapFactory.createSOAPHeaderBlock(name, ns);
        newHeader.setText(value);
        return newHeader;
    }


//    QName partyIdentifierType = new QName(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_NS, dk.gov.oiosi.Constants.PARTY_IDENTIFIER_TYPE);

//    private Identifier tryGetIdentifier(SOAPHeaderBlock shb, MessageContext msgContext) throws AxisFault {
//        if(shb==null) {
//            return null;
//        }
//
//        String value = shb.getText();
//
//        // Todo: fix this
//        // no type specified - try to create
//        try {
//            return new IdentifierCvr(value);
//        } catch(IllegalArgumentException ignore) {
//        }
//        try {
//            return new IdentifierEan(value);
//        } catch(IllegalArgumentException ignore) {
//        }
//        try {
//            return new IdentifierOvt(value);
//        } catch(IllegalArgumentException ignore) {
//        }
//
//        MainException error = new MainException("Could not create Identifier from transfered value (no specified type) - value : [" + value + "]");
//        error.setReceiverError(false);
//        error.setSoapFaultAction("MissingHeaderFault");
//        throw AxisFaultUtil.buildAxisFault(msgContext, error);
//    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#flowComplete(org.apache.axis2.context.MessageContext)
     */
    public void flowComplete(MessageContext ctx) {
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#getName()
     */
    public String getName() {
        return this.getClass().getSimpleName();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#getParameter(java.lang.String)
     */
    public Parameter getParameter(String name) {
        return this.handlerDesc.getParameter(name);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.Handler#getHandlerDesc()
     */
    public HandlerDescription getHandlerDesc() {
        return handlerDesc;
    }
}