/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.axis2.module.signatureproof;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.axis2.module.RaspHandler;
import dk.gov.oiosi.axis2.module.RaspModule;
import dk.gov.oiosi.communication.fault.OiosiFaultCode;
import dk.gov.oiosi.communication.fault.OiosiInnerFaultCode;
import dk.gov.oiosi.exception.AxisFaultUtil;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.policy.OiosiPolicyBean;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.security.SignatureValidationProofAllreadyCompletedException;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.RevocationLookupFactory;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;
import dk.gov.oiosi.util.AxisWSSUtil;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.rampart.handler.WSSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerConstants;

import java.security.cert.X509Certificate;

/**
 * Create signature proof in axis stack.
 */
public class SignatureProofHandler extends RaspHandler {

    private static final Log log = LogFactory.getLog(SignatureProofHandler.class);

    private static HandlerDescription EMPTY_HANDLER_METADATA = new HandlerDescription("default Handler");
    private HandlerDescription handlerDesc;

    /**
     * Constructor.
     */
    public SignatureProofHandler() {
        this.handlerDesc = EMPTY_HANDLER_METADATA;
    }

    @Override
    public void init(HandlerDescription handlerDescription) {
        this.handlerDesc = handlerDescription;
    }

    @Override
    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {
        log.debug("SignatureProofHandler: invoke");

        OiosiPolicyBean oiosiPolicyBean = getPolicyBean(msgContext);
        if (!oiosiPolicyBean.doSignatureProof()) {
            log.warn("SignatureProofHandler not activated. Check your policy configuration.");
            return InvocationResponse.CONTINUE;
        }

        if (!msgContext.isEngaged(RaspModule.MODULE_NAME)) {
            log.error("SignatureProofHandler is missing prerequisite: Rasp module not engaged");
            // return InvocationResponse.ABORT;
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Receiver, "SignatureProofHandler is missing prerequisite: Rasp module not engaged");
        }

        if (!msgContext.isEngaged(WSSHandlerConstants.SECURITY_MODULE_NAME)) {
            log.error("SignatureProofHandler is missing prerequisite: Security module not engaged");
            // return InvocationResponse.ABORT;
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Receiver, "SignatureProofHandler is missing prerequisite: Security module not engaged");
        }

        if (RaspModule.getInstance().isRMNamespaceByAction(msgContext.getSoapAction())) {
            // The message is part of the RM handshake, so do not create signatureproof
            return InvocationResponse.CONTINUE;
        }

        Object recv_results = msgContext.getProperty(WSHandlerConstants.RECV_RESULTS);
        if (recv_results == null) {
            log.error("The received result was null");
            // return InvocationResponse.ABORT;
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Receiver, "The received result was null");
        }

        X509Certificate signerCertificate = AxisWSSUtil.getCertificateFrom_RECV_RESULTS(recv_results);
        if (signerCertificate == null) {
            log.error("Could not read/find any signer certificate");
            //return InvocationResponse.ABORT;
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Receiver, "Could not read/find any signer certificate");
        }
        msgContext.setProperty(Constants.RASP_SIGNING_CERTIFICATE, signerCertificate);

        // Check if the certificate is valid.
        boolean passedRevocation = false;
        boolean passedValidation = false;

        // Check the validity of the certificate via an OCSP or CRL lookup - we do not know ~ RevocationLookupFactory.createRevocationLookupClient();
        if (log.isDebugEnabled()) {
            log.debug("Check Certificate: " + signerCertificate.getSubjectX500Principal());
        }

        RevocationResponse response;
        try {
            IRevocationLookup lookup = RevocationLookupFactory.createRevocationLookupClient();
            response = lookup.checkCertificate(signerCertificate);

            if (response.isValid()) {
                passedRevocation = true;
            } else {
                log.warn("Certificate is not valid.");
                throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Sender, "Sender certificate has been revoked.");
            }
        } catch (AxisFault exception) {
            throw exception;
        } catch (RevocationException e) {
            log.warn("Validation of certificate failed " + signerCertificate.getSubjectX500Principal() + ": " + e.getMessage(), e);
            throw AxisFaultUtil.buildAxisFault(msgContext, e);
        } catch (MainException e) {
            log.warn("Validation of certificate failed " + signerCertificate.getSubjectX500Principal() + ": " + e.getMessage(), e);
            throw AxisFaultUtil.buildAxisFault(msgContext, e);
        } catch (Exception e) {
            log.warn("Could not validate incoming signer certificate: " + e.getMessage(), e);
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Receiver, "Could not validate incoming signer certificate: " + e.getMessage());
        }

        try {
            CertificateValidator.validateCertificate(signerCertificate);
            passedValidation = true;
        } catch (CertificateNotActiveException e) {
            log.warn("Incoming certificate not valid yet: " + signerCertificate.getSubjectX500Principal(), e);
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Sender, "Sender certificate is not active");
        } catch (CertificateExpiredException e) {
            log.warn("Incoming certificate is expired: " + signerCertificate.getSubjectX500Principal(), e);
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Sender, "Sender certificate has expired");
        }

        // Calculates whether the certificate is valid
        boolean certificateValid = passedRevocation && passedValidation;

        try {
            SignatureValidationProof signatureProof = new SignatureValidationProof();
            signatureProof.setCertificateSubject(signerCertificate.getSubjectX500Principal().toString());
            signatureProof.setValidCertificate(certificateValid);

            // Rampart do not let invalid signatures through
            signatureProof.setValidSignature(true);
            signatureProof.setUnchangedMessage(true);

            // Check if message was encrypted
            signatureProof.setEncryptedMessage(AxisWSSUtil.getMessageEncryptedFrom_RECV_RESULTS(recv_results));

            // Get timestamp from SecurityHeader
            signatureProof.setTimeStamp(AxisWSSUtil.getTimestampCreatedFrom_RECV_RESULTS(recv_results));

            // set completed
            signatureProof.setCompleted();

            msgContext.setProperty(Constants.RASP_SIGNATURE_PROOF, signatureProof);
        } catch (SignatureValidationProofAllreadyCompletedException thisShouldNotHappen) {
            log.error("SignatureProof could not be created: " + thisShouldNotHappen, thisShouldNotHappen);
            throw AxisFaultUtil.buildAxisFault(msgContext, thisShouldNotHappen);
        }

        if (!certificateValid) {
            throw AxisFaultUtil.buildAxisFault(msgContext, OiosiInnerFaultCode.SignatureNotValidFault, OiosiFaultCode.Sender, "Sender cetificate validated not valid");
        }

        return InvocationResponse.CONTINUE;
    }

    @Override
    public void flowComplete(MessageContext arg0) {
        // Do nothing...
    }

    @Override
    public String getName() {
        return this.getClass().getSimpleName();
    }

    @Override
    public Parameter getParameter(String name) {
        return this.handlerDesc.getParameter(name);
    }

    @Override
    public HandlerDescription getHandlerDesc() {
        return handlerDesc;
    }
}