/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common;

import dk.gov.oiosi.addressing.*;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.uddi.*;

import java.net.URISyntaxException;

/**
 * Utilities for converting identifiers between strings and other types.
 */
public class IdentifierUtility {

    /**
     * Hide constructor.
     */
    private IdentifierUtility() {
    }

    /**
     * Returns an UDDI identifier (e.g. a tModel key) as an UddiId object (e.g.
     * UddiGuidId or UddiNonGuidId). Throws an exception if the format is not
     * right.
     *
     * @param uddiIdentifier The UDDI identifier to convert
     * @return the UddiId subclass instance. Throws an exception if the format is *         not right
     */
    public static UddiId getUddiIDFromString(final String uddiIdentifier) {
        UddiId idObject;
        if (uddiIdentifier.toLowerCase().startsWith("uddi:")) {
            idObject = new UddiGuidId(uddiIdentifier, true);
            return idObject;
        } else {
            if (UddiGuidId.isValidGuidId(uddiIdentifier, false)) {
                idObject = new UddiGuidId(uddiIdentifier, false);
                return idObject;
            } else {
                idObject = new UddiNonGuidId(uddiIdentifier);
                return idObject;
            }
        }
    }

    /**
     * From an identifier string and identifier type, returns the right
     * IIdentifier subclass.
     *
     * @param endpointKey     The endpoint key as string
     * @param endpointKeyType The type of the key
     * @return the relevant IIdentifier subclass
     * @throws UnknownEndpointTypeException unknow endpoint type
     */
    public static Identifier getIdentifierFromKeyType(final String endpointKey, final EndpointKeyTypeCode endpointKeyType)
            throws UnknownEndpointTypeException {
        Identifier id;
        switch (endpointKeyType.getCode()) {
            case EndpointKeyTypeCode.cvr:
                id = new IdentifierCvr(endpointKey);
                break;
            case EndpointKeyTypeCode.ean:
                id = new IdentifierEan(endpointKey);
                break;
            case EndpointKeyTypeCode.ovt:
                id = new IdentifierOvt(endpointKey);
                break;
            case EndpointKeyTypeCode.p:
                id = new IdentifierP(endpointKey);
                break;
            case EndpointKeyTypeCode.se:
                id = new IdentifierSe(endpointKey);
                break;
            case EndpointKeyTypeCode.duns:
                id = new IdentifierDuns(endpointKey);
                break;
            case EndpointKeyTypeCode.vans:
                id = new IdentifierVans(endpointKey);
                break;
            case EndpointKeyTypeCode.iban:
                id = new IdentifierIban(endpointKey);
                break;
            case EndpointKeyTypeCode.cpr:
                id = new IdentifierCpr(endpointKey);
                break;
            default:
                throw new UnknownEndpointTypeException("Unknown : " + endpointKeyType.getCode());
        }

        return id;
    }

    /**
     * Takes an endpoint an returns a endpoint address.
     *
     * @param endpointAddress the endpoint address
     * @return a specific endpoint address
     */
    public static EndpointAddress getEndpointAddressFromString(final String endpointAddress) {
        try {
            EndpointAddress address;
            if (endpointAddress.toLowerCase().startsWith("http://")) {
                address = new EndpointAddressHttp(new Uri(endpointAddress));
            } else if (endpointAddress.toLowerCase().startsWith("mailto:")) {
                address = new EndpointAddressSMTP(new Uri(endpointAddress));
            } else {
                address = new EndpointAddressSMTP(new Uri(endpointAddress));
            }
            return address;
        } catch (URISyntaxException e) {
            throw new IllegalArgumentException("EndpointAddress could not be created from " + endpointAddress, e);
        }
    }
}