/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.configuration.*;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UnknownEndpointTypeException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.xpath.DocumentXPathResolver;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;

/**
 * Utilities that can be used for UDDI endpoint handling.
 */
public class Utilities {

    /**
     * Hide constructor.
     */
    private Utilities() {
    }

    /**
     * From an XPath expression, gets the identifier for the endpoint key.
     *
     * @param xmlDoc             The XML-document to search
     * @param xpath              The XPath-expression to apply
     * @param keyType            The type of key to return
     * @param prefixedNamespaces The list of namespaces-prefix pairs used in the xpath
     *                           expression
     * @return Returns the endpoint key
     * @throws UnknownEndpointTypeException
     * @throws XmlException
     */
    public final static Identifier getEndpointKeyByXpath(XmlDocument xmlDoc, String xpath, PrefixedNamespace[] prefixedNamespaces,
                                                         EndpointKeyTypeCode keyType) throws UnknownEndpointTypeException, XmlException {
        // 1. Get the endpoint as string:
        String endpointKeyString = DocumentXPathResolver.getElementValueByXpath(xmlDoc, xpath, prefixedNamespaces);

        // 2. Convert to the correct IIdentifierType:
        Identifier id = IdentifierUtility.getIdentifierFromKeyType(endpointKeyString, keyType);

        return id;
    }

    /**
     * Gets the endpoint key type code from a given OiosiMessage and DocumentTypeConfig.
     *
     * @param message      The OiosiMessage
     * @param documentType The document type config
     * @return The endpoint key type code
     */
    public static EndpointKeyTypeCode getEndpointKeyTypeCode(OiosiMessage message, DocumentTypeConfig documentType)
            throws KeyTypeMappingFailedException, XmlException {
        XmlDocument xmlDocument = message.getMessageXml();
        return getEndpointKeyTypeCode(xmlDocument, documentType);
    }

    /**
     * Gets the endpoint key type code from an Xml document and a
     * DocumentTypeConfig.
     *
     * @param xmlDocument  The XML document
     * @param documentType The document type config
     * @return The endpoint key type code
     */
    public static EndpointKeyTypeCode getEndpointKeyTypeCode(XmlDocument xmlDocument, DocumentTypeConfig documentType)
            throws XmlException, KeyTypeMappingFailedException {

        DocumentEndpointInformation endpointType = documentType.getEndpointType();
        KeyTypeMappingExpression mappingExpression = endpointType.getKey().getMappingExpression("EndpointKeyType");

        return getKeyTypeCode(xmlDocument, documentType, mappingExpression);
    }

    /**
     * Gets the endpoint key type code from an XML document, a
     * DocumentTypeConfig and a given KeyTypeMappingExpression.
     *
     * @param xmlDocument       The XML document
     * @param documentType      The document type config
     * @param mappingExpression The mapping expression
     * @return The endpoint key type code
     * @throws XmlException
     * @throws KeyTypeMappingFailedException
     */
    public static EndpointKeyTypeCode getKeyTypeCode(XmlDocument xmlDocument, DocumentTypeConfig documentType, KeyTypeMappingExpression mappingExpression)
            throws XmlException, KeyTypeMappingFailedException {

        // Finds the endpoint key type value from the given XPath
        String xpathExpression = mappingExpression.getXPathExpression();
        String endPointKeyTypeValue = DocumentXPathResolver.getElementValueByXpath(xmlDocument, xpathExpression, documentType
                .getNamespaces());
        KeyTypeMapping mapping = mappingExpression.getMapping(endPointKeyTypeValue);

        String mapsTo = mapping.getMapsTo();

        // Statically maps the maps to to a code
        return new EndpointKeyTypeCode(mapsTo);
    }

    /**
     * Retrieves Receiver endpoint key from document as an identifier.
     *
     * @param xmlDocument The XML document to search
     * @return The identifier
     * @throws MainException
     */
    public static Identifier getReceiverEndpointKey(XmlDocument xmlDocument) throws MainException {
        DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);

        return getReceiverEndpointKey(xmlDocument, documentTypeConfig);
    }

    /**
     * Retrieves Receiver endpoint key from document given a specific document type configuration.
     *
     * @param xmlDocument        The XML document to search
     * @param documentTypeConfig The configuration for the document
     * @return The identifier
     * @throws MainException
     */
    public static Identifier getReceiverEndpointKey(XmlDocument xmlDocument, DocumentTypeConfig documentTypeConfig) throws MainException {
        PrefixedNamespace[] namespaces = documentTypeConfig.getNamespaces();
        DocumentEndpointInformation endpointType = documentTypeConfig.getEndpointType();
        String keyXpath = endpointType.getKey().getXPath();
        KeyTypeMappingExpression mappingExpression = endpointType.getKey().getMappingExpression("EndpointKeyType");
        EndpointKeyTypeCode code = Utilities.getKeyTypeCode(xmlDocument, documentTypeConfig, mappingExpression);
        Identifier identifier = Utilities.getEndpointKeyByXpath(xmlDocument, keyXpath, namespaces, code);
        return identifier;
    }

    /**
     * Retrieves Sender endpoint key from XML document.
     *
     * @param xmlDocument The XML document
     * @return The identifier identifying the endpoint key
     * @throws MainException
     */
    public static Identifier getSenderEndpointKey(XmlDocument xmlDocument) throws MainException {
        DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);

        return getSenderEndpointKey(xmlDocument, documentTypeConfig);
    }

    /**
     * Retrieves Sender endpoint key from document given a specific document type configuration.
     *
     * @param xmlDocument        The XML document
     * @param documentTypeConfig The configuration for the document
     * @return The identifier identifying the endpoint key
     * @throws MainException
     */
    public static Identifier getSenderEndpointKey(XmlDocument xmlDocument, DocumentTypeConfig documentTypeConfig) throws MainException {
        PrefixedNamespace[] namespaces = documentTypeConfig.getNamespaces();
        DocumentEndpointInformation endpointType = documentTypeConfig.getEndpointType();
        String senderKeyXpath = endpointType.getSenderKey().getXPath();
        KeyTypeMappingExpression senderMappingExpression = endpointType.getSenderKey().getMappingExpression("EndpointKeyType");
        EndpointKeyTypeCode code = Utilities.getKeyTypeCode(xmlDocument, documentTypeConfig, senderMappingExpression);
        Identifier identifier = Utilities.getEndpointKeyByXpath(xmlDocument, senderKeyXpath, namespaces, code);
        return identifier;
    }

    /**
     * Gets a profile ID from an XML document given a DocumentTypeConfig and a KeyTypeMappingExpression.
     *
     * @param xmlDocument          The XML document to search
     * @param documentTypeConfig   The configuration for the document
     * @param profileMappingConfig A profile mapping
     * @return The profile ID as UddiId object.
     * @throws Exception On error...
     */
    public static UddiId getProfileId(XmlDocument xmlDocument, DocumentTypeConfig documentTypeConfig, ProfileMappingCollectionConfig profileMappingConfig) throws Exception {
        PrefixedNamespace[] namespaces = documentTypeConfig.getNamespaces();
        String profileIdXPath = documentTypeConfig.getProfileIdXPath();
        String profileName = DocumentXPathResolver.getElementValueByXpath(xmlDocument, profileIdXPath, namespaces);

        if (profileMappingConfig.containsProfileMappingByName(profileName)) {
            ProfileMapping profileMapping = profileMappingConfig.getMapping(profileName);
            String profileTModelGuid = profileMapping.getTModelGuid();
            return IdentifierUtility.getUddiIDFromString(profileTModelGuid);
        }

        throw new OioublDocumentProfileMappingNotFoundException(profileName);
    }

    /**
     * Gets a profile ID from an XML document given a
     * DocumentTypeConfig and a KeyTypeMappingExpression.
     *
     * @param profileName The profile name
     * @return The profile ID
     * @throws Exception
     */
    public static UddiId getProfileId(String profileName) throws Exception {
        ProfileMappingCollectionConfig profileMappingConfig = ConfigurationHandler.getInstance().getProfileMappingCollectionConfig();
        return Utilities.getProfileId(profileName, profileMappingConfig);
    }

    /**
     * Gets a profile ID from an XML document given a
     * DocumentTypeConfig and a KeyTypeMappingExpression.
     *
     * @param profileName          The profile name
     * @param profileMappingConfig A profile mapping
     * @return The profile ID
     * @throws Exception
     */
    public static UddiId getProfileId(String profileName, ProfileMappingCollectionConfig profileMappingConfig) throws Exception {
        if (profileMappingConfig.containsProfileMappingByName(profileName)) {
            ProfileMapping profileMapping = profileMappingConfig.getMapping(profileName);
            String profileTModelGuid = profileMapping.getTModelGuid();
            return IdentifierUtility.getUddiIDFromString(profileTModelGuid);
        }

        throw new OioublDocumentProfileMappingNotFoundException(profileName);
    }
}