package dk.gov.oiosi.common.cache;

import dk.gov.oiosi.common.LogUtil;
import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.holder.IMultipleQuantityValueHolder;
import dk.gov.oiosi.common.cache.holder.LeastRecentlyUsedListValueHolder;
import dk.gov.oiosi.common.cache.stat.AbstractStatisticsCollectorCache;
import dk.gov.oiosi.exception.ArgumentNullException;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Cache algorithm: Least-Frequently used. Counts how often an item is needed.
 * Those that are used least often are discarded first. "Age bits" (see <a
 * href="http://en.wikipedia.org/wiki/Cache_algorithms">Wiki:
 * Cache_algorithms</a>) are kept by _cacheUsageIndex in parent class. Each time
 * something is returned by tryGetValue method, _cacheUsageIndex is incremented
 * and used ValueHolder is updated with result. Cache value, which has lowest
 * value, is removed when the cache is full.
 * <p/>
 * Cache allows to keep more than one value for the same key. Number of
 * instances to keep is configurable, by default it is 1.
 *
 * @param <TKey>
 * @param <TValue>
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
/*
 * dlk, 2012-03-22
 *
 * This cache is supposed to be used for Transformer cache in SchematronStore.
 * But returning of used value is missing.
 *
 * The way how client returns instances to cache should be analysed.
 *
 * Current implementation supposes that it should be done by "add" method.
 *
 * Differences to QuantityCache:
 *
 * 1. Allows to have more than one instance for each key, but by default uses
 * maximum 1.
 *
 * 2. When cache is full, least frequently used key is removed from cache. There
 * is a special counter in ValueListHolder, which increased each time this key
 * is used, when we have to remove something, we find ValueListHolder with
 * lowest usage counter.
 *
 * TODO DLK: May be counter at ValueListHolder have to be cleared after some
 * time to guarantee that they are actual. E.g. in situation when some value was
 * often used 1 day ago, and now other values are very often used, it can have
 * sense to remove those old one...
 */
public class MultipleQuantityCache<TKey, TValue> extends AbstractStatisticsCollectorCache<TKey, TValue> implements IThreadSafeCache<TKey, TValue> {

	public static final String CONFIG_MAX_SIZE = "maxSize";
	public static final String CONFIG_MAX_EQUAL_KEY_SIZE = "maxEqualKeySize";

	private Map<TKey, IMultipleQuantityValueHolder<TKey, TValue>> _keyedDictionary;
	private int _maxSize;
	private int _maxEqualKeySize;
	protected int _cacheUsageIndex;
	private final Object _lockObject = new Object();

	/**
	 * Keeps configuration so cache users can use it for additional
	 * configuration of their functionality
	 */
	private final Map<String, String> configuration;

	/**
	 * Constructor that takes the max size of the cache
	 *
	 * @param maxSize
	 *            Max size of the cache.
	 */
	public MultipleQuantityCache(int maxSize) {
		this(maxSize, 1);
	}

	/**
	 * Constructor that takes the max size of the cache and maximum number of
	 * values for the same keys
	 *
	 * @param maxSize
	 *            Max size of the cache.
	 * @param maxEqualKeySize
	 *            Max number of values to store for the same key value
	 */
	public MultipleQuantityCache(int maxSize, int maxEqualKeySize) {
		if (maxSize < 0) {
			throw new IllegalArgumentException(CONFIG_MAX_SIZE);
		}
		this.configuration = new HashMap<String, String>();
		this.configuration.put(CONFIG_MAX_SIZE, String.valueOf(maxSize));
		this.setup(maxSize, maxEqualKeySize);
	}

	public MultipleQuantityCache(Map<String, String> configuration) {
		if (configuration == null) {
			configuration = new HashMap<String, String>(2);
		}
		/*
		 * dlk, 2012-03-21
		 *
		 * Keep configuration so cache users can use it for additional
		 * configuration of their functionality
		 */
		this.configuration = configuration;
		if (configuration.containsKey(CONFIG_MAX_SIZE)) {
			String value = configuration.get(CONFIG_MAX_SIZE);
			int maxSize = Integer.valueOf(value);
			if (maxSize < 0) {
				throw new IllegalArgumentException(CONFIG_MAX_SIZE);
			}
			value = configuration.get(CONFIG_MAX_EQUAL_KEY_SIZE);
			int maxEqualKeySize = 1;
			if (value != null) {
				maxEqualKeySize = Integer.valueOf(value);
				if (maxEqualKeySize < 0) {
					throw new IllegalArgumentException(CONFIG_MAX_EQUAL_KEY_SIZE);
				}
			}
			this.setup(maxSize, maxEqualKeySize);
		} else {
			throw new IllegalArgumentException("Argument [" + CONFIG_MAX_SIZE + "] was not present in the configuration arguments");
		}
	}

	protected IMultipleQuantityValueHolder<TKey, TValue> buildValueHolder(TKey key) {
		return new LeastRecentlyUsedListValueHolder<TKey, TValue>(key, _maxEqualKeySize);
	}

	private void setup(int maxSize, int maxEqualKeySize) {
		this._maxSize = maxSize;
		this._maxEqualKeySize = maxEqualKeySize;
		this._keyedDictionary = new LinkedHashMap<TKey, IMultipleQuantityValueHolder<TKey, TValue>>(maxSize);
	}

	public void add(TKey key, TValue value) {
		if (key == null) {
			throw new ArgumentNullException("key");
		}
		this.statData.incrementAddTotalCount();
		if (_maxSize == 0) {
			return;
		}
		/*
		 * Check expired is synchronized already by _lockObject
		 */
		this.checkExpired();
		synchronized (_lockObject) {
			IMultipleQuantityValueHolder<TKey, TValue> valueHolder = _keyedDictionary.get(key);
			if (valueHolder == null) {
				valueHolder = buildValueHolder(key);
				_keyedDictionary.put(key, valueHolder);
			}
			valueHolder.add(value);
		}
	}

	public void set(TKey key, TValue value) {
		this.statData.incrementSetTotalCount();
		add(key, value);
	}

	public boolean containsKey(TKey key) {
		if (key == null) {
			throw new ArgumentNullException("key");
		}
		if (_maxSize == 0) {
			this.statData.incrementContainsMissCount();
			return false;
		}
		boolean found = true;
		synchronized (_lockObject) {
			IMultipleQuantityValueHolder<TKey, TValue> valueHolder = _keyedDictionary.get(key);
			if (valueHolder == null || valueHolder.isEmpty()) {
				found = false;
			}
		}
		/*
		 * Move statistics update outside synchronized section - statData uses
		 * concurrent-safe objects
		 */
		if (found) {
			this.statData.incrementContainsMatchCount();
		} else {
			this.statData.incrementContainsMissCount();
		}
		return found;
	}

	public void remove(TKey key) {
		if (key == null) {
			throw new ArgumentNullException("key");
		}
		this.statData.incrementRemoveTotalCount();
		if (_maxSize == 0) {
			return;
		}
		synchronized (_lockObject) {
			_keyedDictionary.remove(key);
		}
	}

	public boolean tryGetValue(TKey key, OutVariable<TValue> value) {
		if (key == null) {
			throw new ArgumentNullException("key");
		}
		if (_maxSize == 0) {
			return false;
		}
		boolean found = true;
		synchronized (_lockObject) {
			IMultipleQuantityValueHolder<TKey, TValue> valueHolder = _keyedDictionary.get(key);
			if (valueHolder == null || valueHolder.isEmpty()) {
				found = false;
			} else {
				this._cacheUsageIndex++;
				valueHolder.incrementUseCount(_cacheUsageIndex);
				value.setVariable(valueHolder.getValue());
			}
		}
		/*
		 * Move statistics update outside synchronized section - statData uses
		 * concurrent-safe objects
		 */
		if (found) {
			this.statData.incrementTryGetMatchCount();
		} else {
			this.statData.incrementTryGetMissCount();
		}
		return found;
	}

	public int checkExpired() {
		this.statData.incrementCheckExpiredTotalCount();
		/*
		 * We need to protect this method from simultanious access, otherwise
		 * ConcurrentModificationException can occur if someone calls this
		 * method outside (not via method add)
		 */
		boolean removed = false;
		synchronized (_lockObject) {
			if (_keyedDictionary.size() >= _maxSize) {
				IMultipleQuantityValueHolder<TKey, TValue> minUsedHolder = null;
				/*
				 * Let's find less used key during all cache live - not just the
				 * oldest.
				 */
				for (IMultipleQuantityValueHolder<TKey, TValue> holder : _keyedDictionary.values()) {
					if (minUsedHolder == null || minUsedHolder.getUseCount() > holder.getUseCount()) {
						minUsedHolder = holder;
					}
				}
				_keyedDictionary.remove(minUsedHolder.getKey());
				removed = true;
			}
		}
		/*
		 * Move statistics update outside synchronized section - statData uses
		 * concurrent-safe objects
		 */
		if (removed) {
			this.statData.incrementCheckExpiredTotalRemovedCount();
			return 1;
		}
		return 0;
	}

	@Override
	public String toString() {
		final int maxLen = 10;
		StringBuilder sb = new StringBuilder();
        if (this.getConfiguration() != null) {
	        sb.append("name=");
	        sb.append(this.getConfiguration().get(ICache.CONFIG_CACHE_NAME));
        }
		sb.append(", curSize=");
		int size = this._keyedDictionary.size();
		sb.append(size);
        sb.append(", cacheImpl=");
		sb.append(this.getClass().getName());
		sb.append(", maxSize=");
		sb.append(_maxSize);
		if (size > 0) {
			sb.append(", list(first ");
			sb.append(maxLen);
			sb.append(")=");
			sb.append(_keyedDictionary != null ? LogUtil.toString(_keyedDictionary.entrySet(), maxLen, 200) : null);
		}
		sb.append(", maxEqual=");
		sb.append(_maxEqualKeySize);
		return sb.toString();
	}

	public Map<String, String> getConfiguration() {
		return configuration;
	}
}