/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common.cache;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.exception.ArgumentNullException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * A cache implementation that stores a defined number of results in memory.
 * <p/>
 * Caching algorithm: Least recently used. Discards the least recently used
 * items first. "Age bits" (see <a
 * href="http://en.wikipedia.org/wiki/Cache_algorithms">Wiki:
 * Cache_algorithms</a>) are kept by modifying ArrayList with keys (when key
 * value is checked by contains() or tryGet() algorithm, returned value is moved
 * to the start of ArrayList (even if it was already the first value).
 * 
 * @deprecated Use {@link LeastRecentlyUsedQuantityCache} instead
 */
public class QuantityCache<TKey, TValue> implements IThreadSafeCache<TKey, TValue> {

    public static final String CONFIG_MAX_SIZE = "maxSize";
    
	private ArrayList<TKey> _indexSortedKeys;
    private HashMap<TKey, TValue> _keyedDictionary;
    private int _maxSize;
    private final Object _lockObject = new Object();
    private final Map<String, String> configuration;

    /**
     * Constructor that takes the max size of the cache.
     *
     * @param maxSize Max size of the cache.
     */
    public QuantityCache(int maxSize) {
        if (maxSize < 0) {
            throw new IllegalArgumentException(CONFIG_MAX_SIZE);
        }
        this.configuration = new HashMap<String, String>();
        this.configuration.put(CONFIG_MAX_SIZE, String.valueOf(maxSize));
        this.setup(maxSize);
    }

    public QuantityCache(Map<String, String> configuration) {
        this.configuration = configuration;
        if (configuration.containsKey(CONFIG_MAX_SIZE)) {
            String value = configuration.get(CONFIG_MAX_SIZE);
            int maxSize = Integer.valueOf(value);
            if (maxSize < 0) {
                throw new IllegalArgumentException(CONFIG_MAX_SIZE);
            }

            this.setup(maxSize);
        } else {
            throw new IllegalArgumentException("Argument "+CONFIG_MAX_SIZE+" was not present int the configuration arguments");
        }
    }

    private void setup(int maxSize) {
        this._maxSize = maxSize;
        this._indexSortedKeys = new ArrayList<TKey>(maxSize);
        this._keyedDictionary = new HashMap<TKey, TValue>(maxSize);
    }

    public void add(TKey key, TValue value) {
        if (key == null) {
            throw new ArgumentNullException("key");
        }
        if (_maxSize == 0) {
            return;
        }
        synchronized (_lockObject) {
            this.checkExpired();
            _indexSortedKeys.add(key);
            _keyedDictionary.put(key, value);
        }
    }

    public void set(TKey key, TValue value) {
        add(key, value);
    }

    public boolean containsKey(TKey key) {
        if (key == null) {
            throw new ArgumentNullException("key");
        }
        if (_maxSize == 0) {
            return false;
        }
        synchronized (_lockObject) {
            if (!_keyedDictionary.containsKey(key)) {
                return false;
            }
            updateIndex(key);
            return true;
        }
    }

    public void remove(TKey key) {
        if (key == null) {
            throw new ArgumentNullException("key");
        }
        if (_maxSize == 0) {
            return;
        }
        synchronized (_lockObject) {
            if (!_keyedDictionary.containsKey(key)) {
                return;
            }
            _indexSortedKeys.remove(key);
            _keyedDictionary.remove(key);
        }
    }

    public boolean tryGetValue(TKey key, OutVariable<TValue> value) {
        if (key == null) {
            throw new ArgumentNullException("key");
        }
        if (_maxSize == 0) {
            return false;
        }
        synchronized (_lockObject) {
            TValue storedValue = _keyedDictionary.get(key);
            if (storedValue == null) {
                return false;
            }
            updateIndex(key);
            value.setVariable(storedValue);
            return true;
        }
    }

    private void updateIndex(TKey key) {
        // the latest used key is at index _indexSortedKeys.size()
        // the last used is at index 0
        _indexSortedKeys.remove(key);
        _indexSortedKeys.add(key);
    }

    public int checkExpired() {
    	boolean removed = false;
		/*
		 * dlk, 2012-04-05
		 * 
		 * Protect this public method also with synchronized block
		 */
    	synchronized (_keyedDictionary) {
	        if (_keyedDictionary.size() != 0) {
	            // the cache contain some data
	            if (_keyedDictionary.size() >= _maxSize) {
	                // cache size to big
	                // Get the last used cache key
	                TKey key = _indexSortedKeys.remove(0);
	                _keyedDictionary.remove(key);
	                removed = true;
	            }
	        }
    	}
    	return removed ? 1 : 0;
    }

    @Override
    public Map<String, String> getConfiguration() {
        return this.configuration;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("QuantityCache [_maxSize=");
        sb.append(_maxSize);
        sb.append(", curSize = ");
        sb.append(_keyedDictionary.size());
        sb.append("]");
        return sb.toString();
    }
}