/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.common.cache;

import dk.gov.oiosi.common.LogUtil;
import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.TimeSpan;
import dk.gov.oiosi.common.cache.stat.AbstractStatisticsCollectorCache;
import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * Simple, generic timed and bounded cache, based on LRU (Least Recently Used)
 * cache algorithm.
 * <p/>
 * By default, it is bounded to 100 items to keep.
 * <p/>
 * By passing -1 as maxSize it is possible to convert it to unbounded cache.
 * <p/>
 * By passing 0 as maxSize - no caching is done at all.
 */
public class TimedCache<TKey, TValue> extends AbstractStatisticsCollectorCache<TKey, TValue> implements IThreadSafeCache<TKey, TValue> {

    public static final String CONFIG_VALIDITY_TIME_IN_HOURS = "validityTimeInHours";
    public static final String CONFIG_FREQUENCY_IN_HOURS = "frequencyInHours";

    public static final String CONFIG_VALIDITY_TIME_IN_MINUTES = "validityTimeInMinutes";
    public static final String CONFIG_FREQUENCY_IN_MINUTES = "frequencyInMinutes";

    private static final int DEFAULT_FREQUENCY_IN_MINUTES = 10;
    private static final int DEFAULT_VALIDITY_IN_MINUTES = 60;


    // The logger
    private static Log log = LogFactory.getLog(TimedCache.class);

    /**
     * Lifetime of cache element.
     */
    private final TimeSpan cacheTimeOut;
    /**
     * Frequences, with which a scheduled job should call checkExpired method of
     * this class
     */
    private final TimeSpan cacheCheckFrequency;

    /**
     * Internal map.
     */
    private final Map<TKey, TimedCacheValue<TValue>> cache;

    /*
      * Let's use AtomicBoolean, which guarantees that 2 threads will not change
      * it from false to true simultaniously by method
      */
    private AtomicBoolean isCheckRunningNow = new AtomicBoolean(false);

    private final Map<String, String> configuration;

    /*
     * Constructors, present before OIORASP 1.3.0
     */

    /**
     * TimedCache constructor, default check expiration frequency will be used
     *
     * @param cacheTimeOutInMillis cache timeout in milliseconds
     * @since OIORASP 1.2.3
     */
    public TimedCache(long cacheTimeOutInMillis) {
        this(new TimeSpan(TimeUnit.MILLISECONDS, cacheTimeOutInMillis));
    }

    /**
     * TimedCache constructor, default check expiration frequency will be used
     *
     * @param cacheTimeOut time span for cache
     * @since OIORASP 1.2.3
     */
    public TimedCache(TimeSpan cacheTimeOut) {
        this(cacheTimeOut, new TimeSpan(TimeUnit.MINUTES, DEFAULT_FREQUENCY_IN_MINUTES), null);
    }

    /*
     * Internal constructor, which does all the job and used by all other constructors
     */
    private TimedCache(TimeSpan cacheTimeOut, TimeSpan checkExpirationFrequency, Map<String, String> configuration) {
        if (configuration == null) {
            this.configuration = buildConfigMap(cacheTimeOut, checkExpirationFrequency);
            this.cacheTimeOut = cacheTimeOut;
            this.cacheCheckFrequency = checkExpirationFrequency;
        } else {
            this.configuration = configuration;
            this.cacheTimeOut = parseTimeOut(configuration);
            this.cacheCheckFrequency = parseFrequency(configuration);
        }
        /*
           * It is important to use LinkedHashMap - so we can be sure that
           * elements in Map are iterated in historical order.
           *
           * This is used in checkExpired() method.
           */
        this.cache = new LinkedHashMap<TKey, TimedCacheValue<TValue>>();
        this.createExpiredCheckTask();
    }

    protected TimeSpan parseTimeOut(Map<String, String> configuration) {
        return parseField(configuration, CONFIG_VALIDITY_TIME_IN_MINUTES, CONFIG_VALIDITY_TIME_IN_HOURS, DEFAULT_VALIDITY_IN_MINUTES);
    }

    protected TimeSpan parseFrequency(Map<String, String> configuration) {
        return parseField(configuration, CONFIG_FREQUENCY_IN_MINUTES, CONFIG_FREQUENCY_IN_HOURS, DEFAULT_FREQUENCY_IN_MINUTES);
    }

    private TimeSpan parseField(Map<String, String> configuration, String configMinutes, String configHours, int configDefault) {
        TimeSpan timeSpan;
        if (configuration.containsKey(configMinutes)) {
            String value = configuration.get(configMinutes);
            int minutes = Integer.valueOf(value);
            timeSpan = new TimeSpan(TimeUnit.MINUTES, minutes);
        } else if (configuration.containsKey(configHours)) {
            String value = configuration.get(configHours);
            int hours = Integer.valueOf(value);
            timeSpan = new TimeSpan(TimeUnit.HOURS, hours);
        } else {
            log.warn("The " + configMinutes + "/" + configHours + " are not defined for the TimedCache, use default '" + configDefault + "' minutes.");
            timeSpan = new TimeSpan(TimeUnit.MINUTES, configDefault);
        }
        return timeSpan;
    }

    /**
     * @param timeSpan           Cache timeout value.
     * @param frequencyInMinutes frequency in minutes for checking the time cache.
     */
    public TimedCache(TimeSpan timeSpan, int frequencyInMinutes) {
        this(timeSpan, new TimeSpan(TimeUnit.MINUTES, frequencyInMinutes), null);
    }

    /**
     * Recommended constructor to use in tests
     *
     * @param timeSpan                 Cache timeout value.
     * @param expirationCheckFrequency Expiration check frequency
     */
    public TimedCache(TimeSpan timeSpan, TimeSpan expirationCheckFrequency) {
        this(timeSpan, expirationCheckFrequency, null);
    }

    /**
     * This constructor is actually used by OIORASP to create timed cache
     *
     * @param configuration
     */
    public TimedCache(Map<String, String> configuration) {
        this(null, null, configuration);
    }

    private static Map<String, String> buildConfigMap(TimeSpan timeSpan, TimeSpan frequencyTimeSpan) {
        Map<String, String> configuration = new HashMap<String, String>();
        if (timeSpan.getTimeUnit() == TimeUnit.HOURS) {
            configuration.put(CONFIG_VALIDITY_TIME_IN_HOURS, String.valueOf(timeSpan.getTimeUnitSpan()));
        } else {
            configuration.put(CONFIG_VALIDITY_TIME_IN_MINUTES, String.valueOf(timeSpan.getSpanMinutes()));
        }
        if (frequencyTimeSpan.getTimeUnit() == TimeUnit.HOURS) {
            configuration.put(CONFIG_FREQUENCY_IN_HOURS, String.valueOf(frequencyTimeSpan.getTimeUnitSpan()));
        } else {
            configuration.put(CONFIG_FREQUENCY_IN_MINUTES, String.valueOf(frequencyTimeSpan.getSpanMinutes()));
        }
        return configuration;
    }

    private void createExpiredCheckTask() {
        // Start a scheduled task to clean the cache at a fix rate
        CacheExpiredCheckTask<TKey, TValue> task = new CacheExpiredCheckTask<TKey, TValue>(this);
        /*
           * Let's use the same scheduling infrastructure in the whole OIORASP
           * library.
           */
        ScheduledExecutor.getInstance().scheduleJob(task, cacheCheckFrequency.getTimeUnitSpan(), cacheCheckFrequency.getTimeUnit());
    }

    /**
     * @return size of cache for memory usage monitoring
     */
    public final int getSize() {
        /*
           * Field cache cannot be null - it is created in all constructors and
           * marked as final
           */
        return cache.size();
    }

    /**
     * @return size of cache before it is cleared.
     */
    public final int clearCache() {
        int size = cache.size();
        synchronized (cache) {
            cache.clear();
        }
        return size;
    }

    /**
     * Add to cache.
     *
     * @param key   search key
     * @param value object to cache
     */
    public final void add(final TKey key, final TValue value) {
        if (key != null) {
            long timeOut = System.currentTimeMillis() + cacheTimeOut.getSpanMillis();
            TimedCacheValue<TValue> cacheValue = new TimedCacheValue<TValue>(timeOut, value);
            synchronized (cache) {
                cache.put(key, cacheValue);
            }
            this.statData.incrementAddTotalCount();
        }
    }

    /**
     * Sets the value of the specified key.
     */
    public final void set(final TKey key, final TValue value) {
        if (key != null) {
            synchronized (cache) {
                remove(key);
                add(key, value);
            }
            this.statData.incrementSetTotalCount();
        }
    }

    /**
     * Get from cache. Return null if not in cache.
     *
     * @param key      search key
     * @param outValue the value found stored in an OutVariable
     * @return whether any results where found
     */
    public final boolean tryGetValue(final TKey key, OutVariable<TValue> outValue) {
        boolean found;
        synchronized (cache) {
            TimedCacheValue<TValue> cacheValue = cache.get(key);

            if (cacheValue == null) {
                found = false;
            } else {
                // check if the found object has expired
                long now = System.currentTimeMillis();
                if (cacheValue.getTimeOut() < now) {
                    // cached value has expired
                    // removing it from the cache
                    cache.remove(key);
                    found = false;
                } else {
                    // TODO: outValue is null coming from CertificateUtil line 198..!
                    outValue.setVariable(cacheValue.getValue());
                    found = true;
                }
            }
        }

        /*
           * Move statistics update outside synchronized section - statData uses
           * concurrent-safe objects
           */
        if (found) {
            this.statData.incrementTryGetMatchCount();
        } else {
            this.statData.incrementTryGetMissCount();
        }

        return found;
    }

    /**
     * Checks whether the cache contains an element with the given key.
     *
     * @param key The key to check
     * @return true if in cache
     */
    public final boolean containsKey(final TKey key) {
        boolean found;

        // using the tryGetValue, as it will remove the object if it has expired
        OutVariable<TValue> ignoreMe = new OutVariable<TValue>();
        found = tryGetValue(key, ignoreMe);

        /*
           * Move statistics update outside synchronized section - statData uses
           * concurrent-safe objects
           */
        if (found) {
            this.statData.incrementContainsMatchCount();
        } else {
            this.statData.incrementContainsMatchCount();
        }

        return found;
    }

    /**
     * Removes the given key from the cache.
     *
     * @param key The key to remove
     */
    public final void remove(final TKey key) {
        this.statData.incrementRemoveTotalCount();
        synchronized (cache) {
            cache.remove(key);
        }
    }

    /**
     * Check and remove expired entries.
     */
    public int checkExpired() {
        log.trace("checkExpired");
        this.statData.incrementCheckExpiredTotalCount();
        int removedCount = 0;
        /*
           * Atomically sets the value to true if the current value is false. If
           * current value is not false - returns false and considers that check
           * is already running
           */
        if (isCheckRunningNow.compareAndSet(false, true)) {
            /*
                * Previous check guarantees, that only one thread will come into
                * this block. Next check guarantees, that there will be no
                * add/tryGet for this cache while we clean it
                */
            synchronized (cache) {
                // The Expired check is not running
                // Now we are running
                try {
                    Iterator<TKey> iterator = cache.keySet().iterator();
                    long now = System.currentTimeMillis();
                    boolean checkAgain = true;

                    while (iterator.hasNext() && checkAgain) {
                        TKey key = iterator.next();
                        TimedCacheValue<TValue> cachedValue = cache.get(key);
                        if (cachedValue.getTimeOut() < now) {
                            removedCount++;
                            /*
                             * One of benefits using Iterators on Hash implementations - we
                             * can remove elements from collection without breaking the order.
                             */
                            iterator.remove();
                        } else {
                            /*
                             * There is no sense to iterate around the whole cache, because
                             * elements are sorted in the order they were added, and timeOut
                             * for each of them is increasing.
                             */
                            checkAgain = false;
                        }
                    }
                    if (log.isDebugEnabled()) {
                        log.debug(removedCount + " cached element were removed from the TimeCache.");
                    }
                } finally {
                    /*
                          * Mark the check as finished.
                          *
                          * At this point we can be sure that the value of
                          * isCheckRunningNow is true, because only this thread
                          * managed to set it to true.
                          */
                    this.isCheckRunningNow.set(false);
                }
            }
        } else {
            /*
                * The check is already running, skip this attempt
                */
        }
        this.statData.incrementCheckExpiredTotalRemovedCount(removedCount);
        return removedCount;
    }

    public Map<String, String> getConfiguration() {
        return configuration;
    }

    @Override
    public String toString() {
        final int maxLen = 10;
        StringBuilder sb = new StringBuilder();
        if (this.getConfiguration() != null) {
            sb.append("name=");
            sb.append(this.getConfiguration().get(ICache.CONFIG_CACHE_NAME));
        }
        sb.append(", cacheImpl=");
        sb.append(this.getClass().getName());
        sb.append(", cacheTimeOut=");
        sb.append(this.cacheTimeOut);
        sb.append(", curSize=");
        int size = cache.size();
        sb.append(size);
        if (size > 0) {
            sb.append(", list(first ");
            sb.append(maxLen);
            sb.append(")=");
            sb.append(LogUtil.toString(cache.entrySet(), maxLen, 200));
        }
        return sb.toString();
    }
}