package dk.gov.oiosi.common.cache.holder;

import java.util.ArrayList;
import java.util.List;

/**
 * Value holder, which allows to have more than one value for the same key.
 * 
 * Supposed to implement least-frequently used cache policy.
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * 
 * @param <TKey>
 * @param <TValue>
 */
public class LeastRecentlyUsedListValueHolder<TKey, TValue> implements IMultipleQuantityValueHolder<TKey, TValue> {

	private final TKey key;
	private final List<InternalValueHolder<TValue>> valueList;
	private volatile int useCount;

	public LeastRecentlyUsedListValueHolder(TKey key, int listSize) {
		this.valueList = new ArrayList<InternalValueHolder<TValue>>(listSize);
		this.key = key;
	}

	@Override
	public TValue getValue() {
		if (this.valueList.size() > 1) {
			InternalValueHolder<TValue> longestTimeBeforeUsedValue = this.valueList.get(0);
			for (int i = 1; i < this.valueList.size(); i++) {
				InternalValueHolder<TValue> vh = this.valueList.get(i);
				if (longestTimeBeforeUsedValue.useIndex > vh.useIndex) {
					longestTimeBeforeUsedValue = vh;
				}
			}
			longestTimeBeforeUsedValue.useIndex = useCount;
			return longestTimeBeforeUsedValue.value;
		}
		InternalValueHolder<TValue> vh = this.valueList.get(0);
		vh.useIndex = useCount;
		return vh.value;
	}

	@Override
	public int incrementUseCount(int cacheUsageIndex) {
		return this.useCount++;
	}

	@Override
	public int getUseCount() {
		return this.useCount;
	}

	@Override
	public TKey getKey() {
		return this.key;
	}

	@Override
	public void add(TValue value) {
		this.valueList.add(new InternalValueHolder<TValue>(value));
	}

	protected class InternalValueHolder<TValueInternal> {
		protected TValueInternal value;
		protected volatile int useIndex;

		public InternalValueHolder(TValueInternal value) {
			this.value = value;
			this.useIndex = 0;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();
			sb.append(value);
			sb.append(" (");
			sb.append(useIndex);
			sb.append(")");
			return sb.toString();
		}
	}

	@Override
	public boolean isEmpty() {
		return valueList.isEmpty();
	}

	@Override
	public String toString() {
		final int maxLen = 10;
		StringBuilder sb = new StringBuilder();
		sb.append("used=");
		sb.append(useCount);
		sb.append(", values=");
		sb.append(valueList != null ? valueList.subList(0, Math.min(valueList.size(), maxLen)) : null);
		return sb.toString();
	}

}
