package dk.gov.oiosi.common.schedule;

import org.apache.log4j.Logger;

import java.io.Serializable;

/**
 * Parent class for all scheduled/delayed jobs in OIORASP
 * <p/>
 * Encapsulates termination logic and start/end logging.
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public abstract class AbstractScheduleJob implements Runnable, Serializable {

    private static final long serialVersionUID = 6289824404339953185L;

    @Override
    public void run() {
        final Logger log = getLog();
        final boolean logEnabled = log != null;
        final boolean infoEnabled = logEnabled && log.isInfoEnabled();
        long start = 0;
        if (infoEnabled) {
            start = System.currentTimeMillis();
            log.info("Scheduler: started " + getJobDetails());
        }
        boolean continueAttempts = false;
        try {
            continueAttempts = doJob();
        } catch (RuntimeException e) {
            if (logEnabled) {
                log.error("Scheduler: " + getJobDetails() + " failed with exception", e);
            }
            throw e;
        } catch (Exception e) {
            if (logEnabled) {
                log.error("Scheduler: " + getJobDetails() + " failed with exception", e);
            }
            throw new RuntimeException("Scheduler: " + getJobDetails() + " failed", e);
        }
        if (infoEnabled) {
            long duration = System.currentTimeMillis() - start;
            StringBuilder sb = new StringBuilder();
            sb.append("Scheduler: finished ").append(getJobDetails());
            sb.append(" in ").append(duration).append(" ms");
            sb.append(", continue attempts = ").append(continueAttempts);
            log.info(sb.toString());
        }
        if (!continueAttempts) {
            if (log.isDebugEnabled()) {
                log.debug("Scheduler: job " + getJobDetails() + " requested to stop future executions");
            }
            throw new ScheduledExecutionTerminateRuntimeException("Scheduler: job " + getJobDetails() + " requested to stop future executions");
        }
    }

    /**
     * Contains actual job logic. Should not throw any exception, otherwise future executions will be ignored.
     *
     * @return true if job should continue to execute next planned time, otherwise - false to never execute this job
     */
    public abstract boolean doJob();

    /**
     * @return Short description of the job - job name, execution details, potentially important parameters
     */
    public abstract String getJobDetails();

    /**
     * @return log object to log start/end/error messages, related to this job. If is null, nothing is logged, even
     *         exceptions in a job.
     */
    public abstract Logger getLog();

    @Override
    public String toString() {
        return getJobDetails();
    }
}