package dk.gov.oiosi.common.startup;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationLookupFactory;
import dk.gov.oiosi.security.revocation.crl.CrlCacheUpdateJob;

/**
 * Loads CRL lists by given URLs in configuration
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class DownloadCRLStartupTask extends AbstractStartupTask {

	public static final String CONFIG_DOWNLOAD_CRL_URL = "downloadCrlUrl";

	public DownloadCRLStartupTask() {
		super();
	}
	
	public DownloadCRLStartupTask(List<StartupTaskConfiguration> configList) {
		super();
		init(configList);
	}
	
	@Override
	public void doTask() throws MainException {
		try {
			long startTotal = System.currentTimeMillis();
			log.info("DownloadCRLStartupTask: started");

			if (!isRevocationCrlSource()) {
				log.warn("DownloadCRLStartupTask: startup task to pre-load CRL data is configured, but RevocationLookup implementation does not use CRL as revocation source. Please either change implementation or remove this startup task as it has no sense. Task is skipped.");
				return;
			}

			List<URL> urlList = new ArrayList<URL>(configList.size());
			for (StartupTaskConfiguration c : configList) {
				if (CONFIG_DOWNLOAD_CRL_URL.equals(c.getName())) {
					URL url;
					try {
						url = new URL(c.getValue());
					} catch (Exception e) {
						log.warn("DownloadCRLStartupTask: failed parse CRL URL " + c.getValue() + ", skip this URL: " + e.getMessage());
						continue;
					}
					downloadCrlUrl(url);
					urlList.add(url);
				}
			}

			if (log.isInfoEnabled()) {
				if (urlList.isEmpty()) {
					log.info("DownloadCRLStartupTask: no valid CRL URLs are found in this task configuration");
				} else {
					log.info("DownloadCRLStartupTask: successfully started scheduled tasks to download CRL lists in " + (System.currentTimeMillis() - startTotal) + " ms for next urls: " + urlList);
				}

			}
		} catch (Exception e) {
			log.error("DownloadCRLStartupTask: failed to execute task", e);
		}
	}

	protected boolean isRevocationCrlSource() {
		IRevocationLookup lookup;
		try {
			lookup = RevocationLookupFactory.createRevocationLookupClient();
		} catch (Exception e) {
			log.warn("DownloadCRLStartupTask: failed to create revocation lookup client to check revocation source type, consider non CRL", e);
			return false;
		}
		return lookup.getRevocationSourceType() == IRevocationLookup.RevocationSourceType.CRL;
	}

	protected void downloadCrlUrl(URL url) {
		CrlCacheUpdateJob preloadJob = new CrlCacheUpdateJob(url, "preload");
		ScheduledExecutor.getInstance().runJobNow(preloadJob);
	}

}
