package dk.gov.oiosi.common.startup;

import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.documenttype.NoDocumentTypeFoundFromXmlDocumentException;
import dk.gov.oiosi.xml.schematron.SchematronStore;
import dk.gov.oiosi.xml.schematron.SchematronStoreFactory;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Startup task, which allows to initialize schematron transformers cache by
 * either giving folder (configuration field "prevalidateXmlFolder") with list
 * of files, which are most often validated by client/server or by specifying the
 * list of friendly names of such document types (configuration field
 * "initDocumentTypeFriendlyName").
 *
 * Does not throw any exception, as invalid configuration can be not critical.
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class SchematronPreloadStartupTask extends AbstractStartupTask {

	protected static final String CONFIG_INIT_DOCUMENT_TYPE_FRIENDLY_NAME = "initDocumentTypeFriendlyName";
	protected static final String CONFIG_PREVALIDATE_XML_FOLDER = "prevalidateXmlFolder";

	protected Map<String, Integer> documentTypeProcessedMap;

	public SchematronPreloadStartupTask() {
		super();
	}

	public SchematronPreloadStartupTask(List<StartupTaskConfiguration> configList) {
		super();
		init(configList);
	}


	@Override
	public void doTask() {
		try {
			log.info("SchematronPreloadStartupTask: started");
			long start = System.currentTimeMillis();
			documentTypeProcessedMap = new LinkedHashMap<String, Integer>();
			for (StartupTaskConfiguration config : this.configList) {
				if (CONFIG_PREVALIDATE_XML_FOLDER.equals(config.getName())) {
					prevalidateXmlFolder(config.getValue());
				} else if (CONFIG_INIT_DOCUMENT_TYPE_FRIENDLY_NAME.equals(config.getName())) {
					initDocumentTypeFriendlyName(config.getValue());
				}
			}
			if (log.isInfoEnabled()) {
				log.info("SchematronPreloadStartupTask: preloaded schematron for next document types: " + documentTypeProcessedMap);
				log.info("SchematronPreloadStartupTask: finished in " + (System.currentTimeMillis() - start)+" ms");
			}
		} catch (Exception e) {
			log.error("SchematronPreloadStartupTask: failed to execute task", e);
		}
	}

	protected void initDocumentTypeFriendlyName(String value) throws ConfigurationException {
		SchematronStore store = SchematronStoreFactory.getSchematronStore();
		DocumentTypeCollectionConfig documentTypeCollectionConfig = ConfigurationHandler.getInstance().getRaspDocumentTypeCollectionConfig();
		DocumentTypeConfig[] documentTypesList = documentTypeCollectionConfig.getDocumentTypes();
		for (DocumentTypeConfig documentTypeConfig : documentTypesList) {
			if (documentTypeConfig.getFriendlyName().equals(value)) {
				try {
					long start = System.currentTimeMillis();
					store.getCompiledSchematron(documentTypeConfig.getSchematronValidationConfig().getSchematronDocumentPath());
					if (log.isInfoEnabled()) {
						StringBuilder sb = new StringBuilder();
						sb.append("SchematronPreloadStartupTask: preloaded schematron for ");
						sb.append(documentTypeConfig.getFriendlyName());
						sb.append(" in ");
						sb.append((System.currentTimeMillis() - start));
						sb.append(" ms");
						log.info(sb.toString());
					}
					increaseDocumentTypeProcessedCount(documentTypeConfig);
					return;
				} catch (Exception e) {
					log.error("SchematronPreloadStartupTask: failed to preload schematron for document type '" + documentTypeConfig.getFriendlyName()+"'", e);
					return;
				}
			}
		}
		log.warn("SchematronPreloadStartupTask: failed to find matching document type config for friendly name '" + value+"'");
	}

	protected void prevalidateXmlFolder(String value) throws FileNotFoundException, MainException, IOException {
		File file = new File(value);
		if (file.exists() && file.isDirectory()) {
			File[] fileList = file.listFiles(new FilenameFilter() {
				@Override
				public boolean accept(File dir, String name) {
					return name.endsWith(".xml");
				}
			});
			if (fileList == null || fileList.length == 0) {
				log.warn("SchematronPreloadStartupTask: no *.xml files found in configured folder " + file.getCanonicalPath());
				return;
			}
			for (File xmlFile : fileList) {
				long start = System.currentTimeMillis();
				XmlDocument xmlDocument = new XmlDocument();
				try {
					xmlDocument.load(xmlFile.getCanonicalPath());
				} catch (Exception e) {
					log.warn("SchematronPreloadStartupTask: failed to load file " + xmlFile.getCanonicalPath() + " as xml", e);
					continue;
				}
				DocumentTypeConfig documentType;
				try {
					DocumentTypeConfigSearcher documentSearcher = new DocumentTypeConfigSearcher();
					documentType = documentSearcher.findUniqueDocumentType(xmlDocument);
				} catch (NoDocumentTypeFoundFromXmlDocumentException e) {
					log.warn("SchematronPreloadStartupTask: failed to find matching document type config for file " + xmlFile.getCanonicalPath());
					continue;
				} catch (Exception e) {
					log.warn("SchematronPreloadStartupTask: failed to find matching document type config for file " + xmlFile.getCanonicalPath(), e);
					continue;
				}
				SchematronValidator sv = new SchematronValidator(documentType.getSchematronValidationConfig());
				boolean success = false;
				try {
					sv.schematronValidateXmlDocument(xmlDocument);
					success = true;
				} catch (Exception e) {
				}
				increaseDocumentTypeProcessedCount(documentType);
				if (log.isInfoEnabled()) {
					StringBuilder sb = new StringBuilder();
					sb.append("SchematronPreloadStartupTask: done schematron validation of ");
					sb.append(xmlFile.getCanonicalPath());
					sb.append(" in ");
					sb.append((System.currentTimeMillis() - start));
					sb.append(" ms, result = ");
					sb.append((success ? "ok" : "failed"));
					log.info(sb.toString());
				}
			}
		} else {
			log.warn("SchematronPreloadStartupTask: folder '" + file.getCanonicalPath() + "' does not exist or is not a directory");
		}
	}

	protected void increaseDocumentTypeProcessedCount(DocumentTypeConfig documentTypeConfig) {
		String mapKey = documentTypeConfig.getFriendlyName();
		Integer count = documentTypeProcessedMap.get(mapKey);
		if (count == null) {
			documentTypeProcessedMap.put(mapKey, 1);
		} else {
			documentTypeProcessedMap.put(mapKey, 1 + count);
		}
	}
}
