package dk.gov.oiosi.common.startup;

import dk.gov.oiosi.common.cache.stat.CacheStateLogScheduledJob;
import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.exception.MainException;

import java.util.concurrent.TimeUnit;

/**
 * Allows users to configure OIORASP library to start scheduled job
 * {@link CacheStateLogScheduledJob}. Can be configured to start with required
 * intervals, e.g. each hour (default value, scheduleJobRunInterval = 1,
 * scheduleJobRunIntervalTimeUnit = HOURS), or each 10 minutes
 * (scheduleJobRunInterval = 10, scheduleJobRunIntervalTimeUnit = MINUTES).
 * <p>
 * See {@link TimeUnit} class for a list of allowed time unites.
 * <p>
 * In case if configuration is not valid, default configuration (each hour) is
 * used. Warning messages are logged with details.
 *
 * Checks either CacheStateLogScheduledJob logger is properly configured - it
 * should be set to info level. Otherwise logs warning message.
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class StartCacheStateLogScheduledJobStartupTask extends AbstractStartupTask {

	public static final String CONFIG_SCHEDULE_JOB_RUN_INTERVAL = "scheduleJobRunInterval";
	public static final String CONFIG_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT = "scheduleJobRunIntervalTimeUnit";

	protected static final int DEFAULT_SCHEDULE_JOB_RUN_INTERVAL = 1;
	protected static final TimeUnit DEFAULT_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT = TimeUnit.HOURS;

	@Override
	public void doTask() throws MainException {
		if (!CacheStateLogScheduledJob.log.isInfoEnabled()) {
			log.warn("Startup task StartCacheStateLogScheduledJobStartupTask is configured, but logging level for category '" + CacheStateLogScheduledJob.log.getName() + "' is not set to 'info', so the task is skipped.");
			return;
		}

		long scheduleJobRunInterval = DEFAULT_SCHEDULE_JOB_RUN_INTERVAL;
		TimeUnit scheduleJobRunIntervalTimeUnit = DEFAULT_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT;
		for (StartupTaskConfiguration config : this.configList) {
			if (CONFIG_SCHEDULE_JOB_RUN_INTERVAL.equals(config.getName())) {
				try {
					scheduleJobRunInterval = Long.parseLong(config.getValue());
				} catch (Exception e) {
					log.warn("StartCacheStateLogScheduledJobStartupTask: failed to parse value of config field " + CONFIG_SCHEDULE_JOB_RUN_INTERVAL + "='" + config.getValue() + "', use deafult: " + DEFAULT_SCHEDULE_JOB_RUN_INTERVAL + " " + DEFAULT_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT);
					scheduleJobRunInterval = DEFAULT_SCHEDULE_JOB_RUN_INTERVAL;
					scheduleJobRunIntervalTimeUnit = DEFAULT_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT;
				}
			}
			if (CONFIG_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT.equals(config.getName())) {
				try {
					scheduleJobRunIntervalTimeUnit = TimeUnit.valueOf(config.getValue());
				} catch (Exception e) {
					log.warn("StartCacheStateLogScheduledJobStartupTask: failed to parse value of config field " + CONFIG_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT + "='" + config.getValue() + "', use deafult: " + DEFAULT_SCHEDULE_JOB_RUN_INTERVAL + " " + DEFAULT_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT);
					scheduleJobRunInterval = DEFAULT_SCHEDULE_JOB_RUN_INTERVAL;
					scheduleJobRunIntervalTimeUnit = DEFAULT_SCHEDULE_JOB_RUN_INTERVAL_TIME_UNIT;
				}
			}
		}
		scheduleJob(scheduleJobRunInterval, scheduleJobRunIntervalTimeUnit);
	}

	/*
	 * Extracted to be testable and overwritable
	 */
	protected void scheduleJob(long scheduleJobRunInterval, TimeUnit scheduleJobRunIntervalTimeUnit) {
		ScheduledExecutor.getInstance().scheduleJob(buildJobImpl(), scheduleJobRunInterval, scheduleJobRunIntervalTimeUnit);
	}

	/*
	 * Extracted to be testable and overwritable
	 */
	protected CacheStateLogScheduledJob buildJobImpl() {
		return new CacheStateLogScheduledJob();
	}

}
