/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.common.Guid;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlDocumentException;
import org.apache.axiom.soap.SOAPHeaderBlock;

import java.security.cert.X509Certificate;
import java.util.Hashtable;
import java.util.Map;

/**
 * Represents an outgoing/incoming message (xml + metadata).
 */
public class OiosiMessage {

    /**
     * The user-defined ID of the message.
     */
    private String messageId;

    /**
     * Custom properties.
     */
    private Map<String, Object> properties = new Hashtable<String, Object>();

    /**
     * Mandatory RASP Soap header : SenderPartyIdentifier
     */
    private Identifier senderPartyIdentifier;

    /**
     * The soap request action for the message.
     */
    private String requestAction = "*";

    /**
     * Mandatory RASP Soap header : ReceiverPartyIdentifier
     */
    private Identifier receiverPartyIdentifier;

    /**
     * Actual Message Payload.
     */
    private XmlDocument messageXml;

    /**
     * Constructor.
     *
     * @param xml  the message
     * @param guid message id
     */
    public OiosiMessage(final XmlDocument xml, final String guid) {
        messageXml = xml;
        messageId = guid;
    }

    /**
     * Constructor that takes the an XML-document as parameter. The XML-Document
     * represents the message body.
     *
     * @param xml the message
     */
    public OiosiMessage(final XmlDocument xml) {
        messageXml = xml;
        messageId = Guid.newGuid().toString();
    }

    /**
     * A list of custom headers.
     */
    private Map<String, Object> messageHeaders = new Hashtable<String, Object>();

    /**
     * Returns a table of custom headers.
     *
     * @return The custom headers
     */
    public Map<String, Object> getMessageHeaders() {
        return messageHeaders;
    }

    /**
     * Add a custom header.
     *
     * @param key   key for custom header. XML element name
     * @param value custom header value
     */
    public void addMessageHeader(String key, Identifier value) {
        messageHeaders.put(key, value);
    }

    /**
     * Add a custom header.
     *
     * @param key   key for custom header. xml element name
     * @param value custom header value
     */
    public void addMessageHeader(String key, String value) {
        messageHeaders.put(key, value);
    }

    /**
     * Add a custom header.
     *
     * @param key   key for custom header
     * @param value actual soap header to add
     */
    public void addMessageHeader(String key, SOAPHeaderBlock value) {
        messageHeaders.put(key, value);
    }

    /**
     * Set mandatory RASP Soap header : MessageIdentifier
     *
     * @param value Value for Message Identifier.
     */
    public void setMessageIdentifier(String value) {
        messageHeaders.put(Constants.RASP_MESSAGE_IDENTIFIER, value);
        messageId = value;
    }

    /**
     * Set mandatory RASP Soap header : SenderPartyIdentifier
     *
     * @param value Identifier
     */
    public void setSenderPartyIdentifier(Identifier value) {
        messageHeaders.put(Constants.RASP_SENDER_PARTY_IDENTIFIER, value);
        senderPartyIdentifier = value;
    }

    /**
     * Get mandatory RASP Soap header : SenderPartyIdentifier
     *
     * @return SenderPartyIdentifier
     */
    public Identifier getSenderPartyIdentifier() {
        return senderPartyIdentifier;
    }

    /**
     * Set mandatory RASP Soap header : ReceiverPartyIdentifier
     *
     * @param value Identifier.
     */
    public void setReceiverPartyIdentifier(Identifier value) {
        messageHeaders.put(Constants.RASP_RECEIVER_PARTY_IDENTIFIER, value);
        receiverPartyIdentifier = value;
    }

    /**
     * Get mandatory RASP Soap header : SenderPartyIdentifier
     *
     * @return SenderPartyIdentifier
     */
    public Identifier getReceiverPartyIdentifier() {
        return receiverPartyIdentifier;
    }

    /**
     * Get the custom properties.
     *
     * @return The properties
     */
    public Map<String, Object> getProperties() {
        return properties;
    }

    /**
     * Get the soap request action for the message.
     *
     * @return soap request action
     */
    public String getRequestAction() {
        return requestAction;
    }

    /**
     * Sets the soap request action for the message.
     *
     * @param value soap request action
     */
    public void setRequestAction(String value) {
        requestAction = value;
    }

    /**
     * The soap reply action for the message.
     */
    private String replyAction = "*";

    /**
     * Gets the soap reply action for the message.
     *
     * @return soap reply action
     */
    public String getReplyAction() {
        return replyAction;
    }

    /**
     * Sets the soap reply action for the message.
     *
     * @param value soap reply action
     */
    public void setReplyAction(String value) {
        replyAction = value;
    }

    /**
     * Gets the actual message payload.
     *
     * @return The payload as an XML document
     */
    public XmlDocument getMessageXml() {
        return messageXml;
    }

    /**
     * Sets the actual message payload.
     *
     * @param e The message payload
     */
    public void setMessageXml(XmlDocument e) {
        messageXml = e;
    }

    /**
     * Checks whether the message has a body set
     *
     * @return True, if there is a body
     */
    public boolean hasBody() {
        try {
            if (messageXml == null || messageXml.getOMDocument() == null) {
                return false;
            }
        } catch (XmlDocumentException e) {
            return false;
        }
        return true;
    }

    /**
     * Gets the message id.
     * <p/>
     * Deprecated From OIORASP 1.3.0: Use method getMessageIdentifier()
     *
     * @return message id
     */
    @Deprecated
    public String getMessageId() {
        return messageId;
    }

    /**
     * Gets the message identifier.
     *
     * @return The message identifier
     */
    public String getMessageIdentifier() {
        return messageId;
    }

    /**
     * The Certificate which signed the message.
     */
    private X509Certificate signerCertificate;

    /**
     * Sets the certificate which signed the message.
     *
     * @param cert The signing certificate
     */
    public void setSignerCertificate(X509Certificate cert) {
        signerCertificate = cert;
    }

    /**
     * Gets the certificate which signed the message.
     *
     * @return The signing certificate
     */
    public X509Certificate getSignerCertificate() {
        return signerCertificate;
    }

    /**
     * The SignatureProof of received message.
     */
    private SignatureValidationProof signatureProof;

    /**
     * Sets the SignatureProof of received message.
     *
     * @param s The signature proof
     */
    public void setSignatureProof(SignatureValidationProof s) {
        signatureProof = s;
    }

    /**
     * Gets the SignatureProof of received message.
     *
     * @return The signature proof
     */
    public SignatureValidationProof getSignatureProof() {
        return signatureProof;
    }
}