/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.communication.configuration;

import dk.gov.oiosi.exception.ArgumentNullException;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class represents a mapping between unique profile names and the corresponding tModel GUID.
 */
public class ProfileMappingCollectionConfig {

    private List<ProfileMapping> _profileMappings = new ArrayList<ProfileMapping>();

    /**
     * Returns a list of mappings between unique profile names and their
     * corresponding tModel GUID
     *
     * @return The mapping
     */
    public Iterable<ProfileMapping> getProfileMappingIterator() {
        return _profileMappings;
    }

    /**
     * Sets the collection of profile mappings
     *
     * @param collection The collection to use in this configuration
     */
    public void setProfileMappings(ProfileMapping[] collection) {
        _profileMappings = Arrays.asList(collection);
    }

    /**
     * Add the collection of profile mappings to the current list
     *
     * @param collection The collection to add
     */
    public void addProfileMappings(ProfileMapping[] collection) {
        _profileMappings.addAll(Arrays.asList(collection));
    }

    /**
     * Adds a new profile mapping to the collection
     *
     * @throws DocumentAllreadyAddedException On error...
     */
    public void AddProfileMapping(ProfileMapping profileMapping) throws DocumentAllreadyAddedException {
        if (profileMapping == null) {
            throw new ArgumentNullException("profileMapping");
        }
        if (containsProfileMappingByName(profileMapping.getName())) {
            throw new DocumentAllreadyAddedException(profileMapping.getName());
        }
        _profileMappings.add(profileMapping);
    }

    /**
     * Clears the mapping list
     */
    public void Clear() {
        if (_profileMappings != null) {
            _profileMappings.clear();
        }
    }

    /**
     * Determines whether the mapping collection contains a mapping with the name profileMappingName
     *
     * @param profileMappingName The mapping name to check the existence of.
     * @return True, if profile mapping exists.
     */
    public boolean containsProfileMappingByName(String profileMappingName) {
        for (ProfileMapping mapping : _profileMappings) {
            String currentProfileMappingName = mapping.getName();
            if (currentProfileMappingName.equals(profileMappingName)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get the profile mapping corresponding to the given name
     *
     * @param profileMappingName The profile mapping name
     * @return The corresponding profile mapping
     */
    public ProfileMapping getMapping(String profileMappingName) {
        for (ProfileMapping mapping : _profileMappings) {
            if (mapping.getName().equals(profileMappingName)) {
                return mapping;
            }
        }
        return null;
    }
}