/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.configuration;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.common.cache.ICache;
import dk.gov.oiosi.common.cache.IThreadSafeCache;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.security.revocation.crl.CrlInstance;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupKey;
import dk.gov.oiosi.uddi.UddiService;
import dk.gov.oiosi.uddi.UddiTModel;
import org.apache.log4j.Logger;

import javax.security.auth.x500.X500Principal;
import javax.xml.transform.Transformer;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CacheFactory {

    private static final Logger log = Logger.getLogger(CacheFactory.class);

    /**
     * Cache to store the ocsp lookup - check is a certificate is valid
     */
    private IThreadSafeCache<X500Principal, RevocationResponse> ocspLookupCache = null;

    /**
     * Cache to store the result from the CRL list - the certificates that has been revoked, ala blacklisted
     */
    private IThreadSafeCache<String, CrlInstance> crlLookupCache = null;

    private IThreadSafeCache<UddiLookupKey, List<UddiService>> uddiServiceCache = null;

    private IThreadSafeCache<UddiId, UddiTModel> uddiTModelCache = null;

    /**
     * Cache to store certificated, that has been retrieved from LDAP
     */
    private IThreadSafeCache<CertificateSubject, X509Certificate> certificateCache = null;

    /**
     * Cache to store the compiled schematron
     */
    private IThreadSafeCache<String, Transformer> schematronStoreCache = null;

    /**
     *  Cache to store the compiled schema file
     */
    // private ICache<String, XmlSchemaSet> schemaStoreCache = null;

    /**
     * Cache to store the UnfinishedSignatureValidationProof
     */
    //private ICache<String, UnfinishedSignatureValidationProof> messageIdUnfinishedSignaturesCache = null;

    private static CacheFactory cacheFactory = new CacheFactory();

    @SuppressWarnings("unchecked")
    /*
     * Marked as protected to make it testable
     */
    protected CacheFactory() {
        CacheConfig cacheConfig = ConfigurationHandler.getInstance().getCacheConfig();
        try {
            this.ocspLookupCache = this.createCache(cacheConfig.getOcspLookupCache());
            this.crlLookupCache = this.createCache(cacheConfig.getCrlLookupCache());
            this.uddiServiceCache = this.createCache(cacheConfig.getUddiServiceCache());
            this.uddiTModelCache = this.createCache(cacheConfig.getUddiTModelCache());
            this.certificateCache = this.createCache(cacheConfig.getCertificateCache());
            this.schematronStoreCache = this.createCache(cacheConfig.getSchematronCache());
        } catch (Throwable e) {
            throw new RaspLibraryException(e);
        }
    }

    @SuppressWarnings({"rawtypes"})
    /**
     * Instantiates cache implementation. If given cacheConfig is null or is not valid, generates default cache implementation.
     */
    /*
      * Marked as protected to avoid disclosure of internal implementation of
      * this class for clients code, but with possibility to override
      * implementation.
      */
    protected IThreadSafeCache createCache(CacheConfigElement cacheConfig) throws Exception {
        IThreadSafeCache cache = createCacheInternal(cacheConfig);
        if (cache == null) {
            /*
                * CacheConfigElement should always contain cache code
                */
            CacheConfigElement cacheConfigInternal = null;
            String cacheCode = null;
            if (cacheConfig != null) {
                cacheCode = cacheConfig.getCacheCode();
                if (log.isInfoEnabled()) {
                    log.info("CacheFactory: not found cache configuration for " + cacheCode + ", try to find default");
                }
                cacheConfigInternal = CacheFactoryConstants.getDefaultCacheConfig(cacheCode);
            }
            if (cacheConfigInternal == null) {
                log.warn("CacheFactory: not found default configuration for cache code " + cacheCode + " in class CacheFactoryConstants, use global default configuration");
                cacheConfigInternal = CacheFactoryConstants.getDefaultCacheConfig();
            }
            cache = createCacheInternal(cacheConfigInternal);
        }
        if (log.isInfoEnabled()) {
            log.info("Initialized cache " + cacheConfig.getCacheCode() + ": " + cache);
        }
        return cache;
    }

    @SuppressWarnings({"unchecked", "rawtypes"})
    private IThreadSafeCache createCacheInternal(CacheConfigElement cacheConfig) throws Exception {
        if (cacheConfig == null) {
            log.warn("CacheFactory: cannot find cache configuration, use default");
            return null;
        }
        Class cacheClass = null;
        if (cacheConfig.getImplementationNamespaceClass() != null && cacheConfig.getImplementationNamespaceClass().length() > 0) {
            try {
                cacheClass = Class.forName(cacheConfig.getImplementationNamespaceClass());
            } catch (Throwable e) {
                log.error("Failed to find class of cache implementation, config = " + cacheConfig, e);
            }
        }
        IThreadSafeCache cache = null;
        if (cacheClass != null) {
            try {
                Map<String, String> configurationList = cacheConfig.getConfigurationList();
                if (configurationList == null) {
                    configurationList = new HashMap<String, String>(1);
                }
                /*
                     * CACHE_NAME is removed from configuration at
                     * RaspConfiguration.xml files, it can be set programmatically
                     * by the library itself.
                     *
                     * Setting it in RaspConfiguration.xml which is error-prone
                     * (client can just forget to set it!), and it is already
                     * defined by parent tag of cache configuration element.
                     */
                configurationList.put(ICache.CONFIG_CACHE_NAME, cacheConfig.getCacheCode());
                cache = (IThreadSafeCache) cacheClass.getConstructor(new Class[]{Map.class}).newInstance(cacheConfig.getConfigurationList());
            } catch (Throwable e) {
                log.error("Failed to instantiate cache implementation class, it should implement IThreadSafeCache interface and contain constructor with Map object, config = " + cacheConfig, e);
            }
        }
        return cache;
    }

    public static CacheFactory getInstance() {
        return cacheFactory;
    }

    public ICache<X500Principal, RevocationResponse> getOcspLookupCache() {
        return ocspLookupCache;
    }

    public IThreadSafeCache<String, CrlInstance> getCrlLookupCache() {
        return crlLookupCache;
    }

    public ICache<UddiLookupKey, List<UddiService>> getUddiServiceCache() {
        return uddiServiceCache;
    }

    public ICache<UddiId, UddiTModel> getUddiTModelCache() {
        return uddiTModelCache;
    }

    public ICache<CertificateSubject, X509Certificate> getCertificateCache() {
        return certificateCache;
    }

    public IThreadSafeCache<String, Transformer> getSchematronStoreCache() {
        return this.schematronStoreCache;
    }
}