/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.exception;

import dk.gov.oiosi.communication.fault.OiosiFaultCode;
import dk.gov.oiosi.communication.fault.OiosiInnerFaultCode;
import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axiom.soap.SOAP12Constants;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.i18n.Messages;
import org.apache.commons.lang.NotImplementedException;
import org.apache.commons.logging.LogFactory;

import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.List;

/**
 * Utility for creating and throwing Axis2 faults.
 */
public class AxisFaultUtil {

    /**
     * Build and throw Axis Fault.
     *
     * @param msgContext message context
     * @param e          exception
     * @throws AxisFault On error...
     */
    public static void throwAxisFault(MessageContext msgContext, MainException e) throws AxisFault {
        AxisFault axisFault = buildAxisFault(msgContext, e);
        throw axisFault;
    }

    /**
     * Build and throw Axis Fault.
     *
     * @param msgContext      message context
     * @param soapFaultAction fault action
     * @param soapFaultCode   fault code
     * @param e               exception
     * @throws AxisFault On error...
     */
    public static void throwAxisFault(MessageContext msgContext, String soapFaultAction, String soapFaultCode, Exception e)
            throws AxisFault {
        AxisFault axisFault = buildAxisFault(msgContext, soapFaultAction, soapFaultCode, e);
        throw axisFault;
    }

    /**
     * Build Axis Fault.
     *
     * @param msgContext message context
     * @param e          exception
     * @return AxisFault
     * @throws AxisFault On error...
     */
    public static AxisFault buildAxisFault(MessageContext msgContext, MainException e) throws AxisFault {
        return buildAxisFault(msgContext, e.getSoapFaultAction(), e.getSoapFaultCode(), e);
    }

    public static AxisFault buildAxisFault(MessageContext msgContext, OiosiInnerFaultCode oiosiInnerFaultCode, OiosiFaultCode oiosiFaultCode, String message) throws AxisFault {
        MainException exception = new MainException(message);
        // Java OIORASP server sends HTTP code 400 ("Bad request") if it considers Sender fault (does not work at .NET),
        // Java OIORASP server sends HTTP code 500 ("Internal server error") if it thinks it is receivers problem (works at .NET).

        switch (oiosiFaultCode) {
            case Receiver: {
                exception.setReceiverError(true);
                break;
            }
            case Sender: {
                // Hack - big ugly error
                // The .Net version of RASP can not extract the code, if the oiosiFaultCode is set to sender
                // exception.setReceiverError(false);
                // so we always blame our self - should be updated when error is handler correct on the receiver .Net side

                exception.setReceiverError(true);
                break;
            }
            default: {
                // should never happens
                throw new NotImplementedException("oiosiFaultCode case unknown '" + oiosiFaultCode + "'");
            }
        }

        switch (oiosiInnerFaultCode) {
            case SchemaValidationFault: {
                exception.setSoapFaultAction("SchemaValidationFault");
                break;
            }
            case SchematronValidationFault: {
                exception.setSoapFaultAction("SchematronValidationFault");
                break;
            }
            case SignatureNotValidFault: {
                exception.setSoapFaultAction("SignatureNotValidFault");
                break;
            }
            case UnknownDocumentTypeFault: {
                exception.setSoapFaultAction("UnknownDocumentTypeFault");
                break;
            }
            case MessagePersistencyFault: {
                exception.setSoapFaultAction("MessagePersistencyFault");
                break;
            }
            case XsltTransformationFault: {
                exception.setSoapFaultAction("XsltTransformationFault");
                break;
            }
            case InternalSystemFailureFault: {
                exception.setSoapFaultAction("InternalSystemFailureFault");
                break;
            }
            case MissingHeaderFault: {
                exception.setSoapFaultAction("MissingHeaderFault");
                break;
            }
            case DispatchingNotPossibleFault: {
                exception.setSoapFaultAction("DispatchingNotPossibleFault");
                break;
            }
            case DispatchingDestinationUnknownFault: {
                exception.setSoapFaultAction("DispatchingDestinationUnknownFault");
                break;
            }
            case DispatchingFailedFault: {
                exception.setSoapFaultAction("DispatchingFailedFault");
                break;
            }
            case DispatchingDeniedFault: {
                exception.setSoapFaultAction("DispatchingDeniedFault");
                break;
            }
            case ValidationFault: {
                exception.setSoapFaultAction("ValidationFault");
                break;
            }
            default: {
                exception.setSoapFaultAction("InternalSystemFailureFault");
            }
        }

        return buildAxisFault(msgContext, exception);
    }

    /**
     * Build Axis Fault.
     *
     * @param msgContext      message context
     * @param soapFaultAction fault action
     * @param soapFaultCode   fault code
     * @param e               exception
     * @return AxisFault
     * @throws AxisFault On error...
     */
    public static AxisFault buildAxisFault(MessageContext msgContext, String soapFaultAction, String soapFaultCode, Exception e) throws AxisFault {
        // Hack - big big hack -  Java OIORASP server sends HTTP code 400 ("Bad request") if it considers Sender fault (does not work at .NET),
        // so always default to receiver. This should be fix before next release
        soapFaultCode = "Receiver";

        org.apache.commons.logging.Log log = LogFactory.getLog(AxisFaultUtil.class);
        log.debug("buildAxisFault : " + soapFaultAction + " : " + soapFaultCode);
        String faultActionUri = dk.gov.oiosi.Constants.RASP_NAME_SPACE + "/" + soapFaultAction;
        String nsURI = msgContext.getEnvelope().getNamespace().getNamespaceURI();
        QName soapFaultSubCode = new QName(nsURI, soapFaultAction);
        if (SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI.equals(nsURI)) {
            //fac = OMAbstractFactory.getSOAP11Factory();

            AxisFault axisFault = new AxisFault(soapFaultSubCode, e.getMessage(), e);
            axisFault.setFaultAction(faultActionUri);
            throw axisFault;

        } else if (SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI.equals(nsURI)) {
            //fac =OMAbstractFactory.getSOAP12Factory();
        } else {
            throw new AxisFault(Messages.getMessage("invalidSOAPversion"));
        }

        String message = e.getMessage();

        QName faultCode = new QName(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI, soapFaultCode, SOAP12Constants.SOAP_DEFAULT_NAMESPACE_PREFIX);

        List<QName> list = new ArrayList<QName>();
        list.add(new QName(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_NS, soapFaultAction, dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_PREFIX));

        AxisFault tmp = new AxisFault(message, e);
        tmp.setStackTrace(e.getStackTrace());

        tmp.setFaultAction(faultActionUri);
        tmp.setFaultCode(faultCode);
        tmp.setFaultSubCodes(list);

        // TODO : Get this fixed in Axis
        // THIS IS TO AVOID A PROBLEM IN AXIS!!!
        // REUSE MERCURY AxisFaultUtil
        AxisFault axisFault = new AxisFault(message, org.wso2.mercury.util.AxisFaultUtil.generateAxisFaultMessageContext(msgContext, tmp));
        axisFault.setStackTrace(e.getStackTrace());

        log.debug("Set application fault type ...");

        axisFault.setFaultType(org.apache.axis2.Constants.APPLICATION_FAULT);
        return axisFault;
    }
}