/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.policy;

import org.apache.axiom.om.OMElement;
import org.apache.neethi.Assertion;
import org.apache.neethi.Constants;
import org.apache.neethi.PolicyComponent;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * Used to hold properties loaded from service.xml.
 */
public class OiosiPolicyBean implements Assertion {

    public static final String RASP_POLICY_NAMESPACE = "http://oiosi.dk/rasp/policy";
    public static final String RASP_POLICY_NAMESPACE_PREFIX = "oiosi";

    public static final String RASP_POLICY_BEAN = "OIOSIConfig";

    public static final String RASP_CONFIGURATION = "RaspConfiguration";
    public static final String CUSTOM_HEADERS = "CustomHeaders";
    public static final String SIGNATURE_PROOF = "SignatureProof";
    public static final String XML_VALIDATION = "XMLValidation";
    public static final String XML_SCHEMATRON = "XMLSchematron";
    public static final String XML_TRANSFORMATION = "XMLTransformation";


    /**
     * Configuration bean name.
     */
    private QName name = new QName(RASP_POLICY_NAMESPACE, RASP_POLICY_BEAN, RASP_POLICY_NAMESPACE_PREFIX);

    /**
     * Parent.
     */
    private OiosiPolicyBean parent = null;

    /**
     * Parent.
     *
     * @return parent
     */
    public OiosiPolicyBean getParent() {
        return parent;
    }

    public void setParent(OiosiPolicyBean p) {
        this.parent = p;
    }

    /* (non-Javadoc)
     * @see org.apache.neethi.Assertion#getName()
     */
    public QName getName() {
        return name;
    }

    /* (non-Javadoc)
     * @see org.apache.neethi.Assertion#isOptional()
     */
    public boolean isOptional() {
        return false;
    }

    /* (non-Javadoc)
     * @see org.apache.neethi.Assertion#normalize()
     */
    public PolicyComponent normalize() {
        return this;
    }

    /* (non-Javadoc)
     * @see org.apache.neethi.Assertion#serialize(javax.xml.stream.XMLStreamWriter)
     */
    public void serialize(XMLStreamWriter writer) throws XMLStreamException {
        try {

            String prefix = writer.getPrefix(RASP_POLICY_NAMESPACE);
            if (prefix == null) {
                prefix = RASP_POLICY_NAMESPACE_PREFIX;
                writer.setPrefix(prefix, RASP_POLICY_NAMESPACE);
            }

            // <oiosi:OIOSIConfig>
            writer.writeStartElement(prefix, RASP_POLICY_BEAN, RASP_POLICY_NAMESPACE);
            // xmlns:oiosi=".."
            writer.writeNamespace(prefix, RASP_POLICY_NAMESPACE);

            // <oiosi:XMLTransformation />
            writer.writeStartElement(prefix, XML_TRANSFORMATION, RASP_POLICY_NAMESPACE);
            writer.writeCharacters("" + doTransformation);
            writer.writeEndElement();

            // <oiosi:XMLSchematron />
            writer.writeStartElement(prefix, XML_SCHEMATRON, RASP_POLICY_NAMESPACE);
            writer.writeCharacters("" + doSchematron);
            writer.writeEndElement();

            // <oiosi:XMLValidation />
            writer.writeStartElement(prefix, XML_VALIDATION, RASP_POLICY_NAMESPACE);
            writer.writeCharacters("" + doValidation);
            writer.writeEndElement();

            // <oiosi:SignatureProof />
            writer.writeStartElement(prefix, SIGNATURE_PROOF, RASP_POLICY_NAMESPACE);
            writer.writeCharacters("" + doSignatureProof);
            writer.writeEndElement();

            // <oiosi:CustomHeaders />
            writer.writeStartElement(prefix, CUSTOM_HEADERS, RASP_POLICY_NAMESPACE);
            writer.writeCharacters("" + doCustomHeaders);
            writer.writeEndElement();

            // <oiosi:RaspConfiguration />
            writer.writeStartElement(prefix, RASP_CONFIGURATION, RASP_POLICY_NAMESPACE);
            writer.writeCharacters("" + raspConfigurationFile);
            writer.writeEndElement();

            // </oiosi:OIOSIConfig>
            writer.writeEndElement();
        } catch (Exception e) {
            throw new XMLStreamException(e);
        }

    }

    /* (non-Javadoc)
     * @see org.apache.neethi.PolicyComponent#equal(org.apache.neethi.PolicyComponent)
     */
    public boolean equal(PolicyComponent policy) {
        if (policy == null) {
            return false;
        }

        if (this == policy) {
            return true;
        }

        if (getClass() != policy.getClass()) {
            return false;
        }

        OiosiPolicyBean other = (OiosiPolicyBean) policy;

        if (doValidation != other.doValidation) {
            return false;
        }

        if (doSchematron != other.doSchematron) {
            return false;
        }

        if (doTransformation != other.doTransformation) {
            return false;
        }

        if (doSignatureProof != other.doSignatureProof) {
            return false;
        }

        if (doCustomHeaders != other.doCustomHeaders) {
            return false;
        }

        if (!raspConfigurationFile.equals(other.raspConfigurationFile)) {
            return false;
        }

        return true;
    }

    /* (non-Javadoc)
     * @see org.apache.neethi.PolicyComponent#getType()
     */
    public short getType() {
        return Constants.TYPE_ASSERTION;
    }

    /**
     * Get values from xml.
     *
     * @param policyElement xml policy element
     */
    public void moveValues(OMElement policyElement) {

        OMElement tmp;
        tmp = policyElement.getFirstChildWithName(new QName(RASP_POLICY_NAMESPACE, XML_TRANSFORMATION));
        if (tmp != null) {
            doTransformation = getBooleanValue(tmp);
        }

        tmp = policyElement.getFirstChildWithName(new QName(RASP_POLICY_NAMESPACE, XML_SCHEMATRON));
        if (tmp != null) {
            doSchematron = getBooleanValue(tmp);
        }

        tmp = policyElement.getFirstChildWithName(new QName(RASP_POLICY_NAMESPACE, XML_VALIDATION));
        if (tmp != null) {
            doValidation = getBooleanValue(tmp);
        }

        tmp = policyElement.getFirstChildWithName(new QName(RASP_POLICY_NAMESPACE, SIGNATURE_PROOF));
        if (tmp != null) {
            doSignatureProof = getBooleanValue(tmp);
        }

        tmp = policyElement.getFirstChildWithName(new QName(RASP_POLICY_NAMESPACE, CUSTOM_HEADERS));
        if (tmp != null) {
            doCustomHeaders = getBooleanValue(tmp);
        }

        tmp = policyElement.getFirstChildWithName(new QName(RASP_POLICY_NAMESPACE, RASP_CONFIGURATION));
        if (tmp != null) {
            raspConfigurationFile = tmp.getText();
        }

    }

    /**
     * util to get boolean.
     *
     * @param val xml element
     * @return boolean
     */
    private boolean getBooleanValue(OMElement val) {
        return "true".equalsIgnoreCase(val.getText());
    }

    // values...

    /**
     * Do schema validation in stack.
     */
    private boolean doValidation = false;

    /**
     * Do schematron valitation in stack.
     */
    private boolean doSchematron = false;

    /**
     * Do pruning in stack.
     */
    private boolean doTransformation = false;

    /**
     * Crate signature proof in stack.
     */
    private boolean doSignatureProof = false;

    /**
     * Do custom header handling/extracting in stack.
     */
    private boolean doCustomHeaders = true;

    /**
     * Name of configuration file (used on server side) in stack.
     */
    private String raspConfigurationFile = null;

    /**
     * Do schema validation in stack.
     *
     * @return do schema validation
     */
    public boolean doValidation() {
        return doValidation;
    }

    /**
     * Do schematron validation in stack.
     *
     * @return do schematron
     */
    public boolean doSchematron() {
        return doSchematron;
    }

    /**
     * Do pruning in stack.
     *
     * @return do pruning
     */
    public boolean doTransformation() {
        return doTransformation;
    }

    /**
     * Crate signature proof in stack.
     *
     * @return do signature proof
     */
    public boolean doSignatureProof() {
        return doSignatureProof;
    }

    /**
     * Do custom header handling/extracting in stack.
     *
     * @return do custom headers
     */
    public boolean doCustomHeaders() {
        return doCustomHeaders;
    }

    /**
     * Name of configuration file (used on server side) in stack.
     *
     * @return file name
     */
    public String getRaspConfiguration() {
        return raspConfigurationFile;
    }
}