/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.oces;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.X509CheckStatus;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationCheckStatus;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.security.cert.X509Certificate;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Represents an OCES X509 certificate. Encapsulates an X509Certificate object.
 */
public class OcesX509Certificate {

    private static Log log = LogFactory.getLog(OcesX509Certificate.class);

    /**
     * Certificate.
     */
    private X509Certificate x509Certificate;

    /**
     * Status of logical check.
     */
    private X509CheckStatus x509CheckStatus = new X509CheckStatus(X509CheckStatus.NotChecked);

    /**
     * OCSP status.
     */
    private RevocationCheckStatus revocationCheckStatus = new RevocationCheckStatus(RevocationCheckStatus.NotChecked);

    /**
     * Certificate type.
     */
    private OcesCertificateType ocesCertificateType = new OcesCertificateType(OcesCertificateType.NonOces);

    /**
     * Certificate subject.
     */
    private CertificateSubject subject;

    /**
     * Constructor that takes the X509Certificate wrapped. If the certificate is
     * not an OCES-certificate an exception will be thrown.
     *
     * @param certificate An OCES x509 certificate
     * @throws FailedGetOcesCertificateTypeException
     *          On error...
     * @throws InvalidOcesCertificateException
     *          On error...
     */
    public OcesX509Certificate(X509Certificate certificate) throws InvalidOcesCertificateException, FailedGetOcesCertificateTypeException {
        if (certificate == null) {
            throw new ArgumentNullException("certificate");
        }

        x509Certificate = certificate;
        subject = new CertificateSubject(x509Certificate);
        setCertificateType();

        if (ocesCertificateType.getType() == OcesCertificateType.NonOces) {
            log.error("Object not created, because type=OcesCertificateType.NonOces for [" + certificate.getSubjectDN() + "]");
            throw new InvalidOcesCertificateException(certificate);
        }
    }

    /**
     * Checks the status of the certificate to determine if it is revoked. Updates the
     * internal state with the result.
     *
     * @param revocationLookupClient The revocation client to use for the request
     * @return Returns the check status
     * @throws RevocationException On error...
     */
    public RevocationCheckStatus checkRevocationStatus(IRevocationLookup revocationLookupClient) throws RevocationException {
        RevocationResponse response;
        try {
            response = revocationLookupClient.checkCertificate(x509Certificate);
        } catch (RevocationException e) {
            revocationCheckStatus = new RevocationCheckStatus(RevocationCheckStatus.UnknownIssue);
            throw e;
        }

        if (response.isValid()) {
            revocationCheckStatus = new RevocationCheckStatus(RevocationCheckStatus.AllChecksPassed);
        } else {
            revocationCheckStatus = new RevocationCheckStatus(RevocationCheckStatus.CertificateRevoked);
        }

        return revocationCheckStatus;
    }

    /**
     * Gets the X509Certificate2 that is the basis of the OCES Certificate.
     *
     * @return The X509 certificate
     */
    public X509Certificate getCertificate() {
        return x509Certificate;
    }

    /**
     * Gets the certificate subject of the OCES certificate.
     *
     * @return Certificate subject
     */
    public CertificateSubject getSubject() {
        return subject;
    }

    /**
     * Gets the certificate check status.
     *
     * @return x509 check status
     */
    public X509CheckStatus getX509CheckStatus() {
        return x509CheckStatus;
    }

    /**
     * Gets the revocation check status
     *
     * @return Revocation status
     */
    public RevocationCheckStatus getOcspCheckStatus() {
        return revocationCheckStatus;
    }

    /**
     * Gets the certificate Oces certificate type
     *
     * @return The certificate type
     */
    public OcesCertificateType getOcesCertificateType() {
        return ocesCertificateType;
    }

    /**
     * @return subject serial number
     * @throws MainException On error...
     * @deprecated Gets certificate serial number
     */
    public CertificateSubject getSubjectSerialNumber() throws MainException {
        return subject;
    }

    /**
     * Sets the certificate type.
     *
     * @throws FailedGetOcesCertificateTypeException
     *          On error...
     */
    private void setCertificateType() throws FailedGetOcesCertificateTypeException {
        ocesCertificateType = getOcesCertificateType(x509Certificate);
    }

    /**
     * Get the OCES certificate type from a given certificate.
     *
     * @param certificate The certificate to examine
     * @return The certificate type
     * @throws FailedGetOcesCertificateTypeException
     *          On error...
     */
    public static OcesCertificateType getOcesCertificateType(X509Certificate certificate) throws FailedGetOcesCertificateTypeException {
        if (certificate == null) {
            throw new ArgumentNullException("certificate");
        }

        try {
            return getFromSubject(certificate);
        } catch (Exception e) {
            log.error("Could not get Oces certificate type from subject " + e, e);
            throw new FailedGetOcesCertificateTypeException(e);
        }
    }

    /**
     * Try to get the cvr number as a string value from the certificate, if such exists
     *
     * @param cvrNumberString The resulting cvr string
     * @return Whether a cvr number string value could be parsed.
     */
    public boolean TryGetCvrNumberString(OutVariable<String> cvrNumberString) {

        String serialNumberValue = subject.getSerialNumberValue();
        Pattern regEx = Pattern.compile("CVR:\\d+");
        Matcher match = regEx.matcher(serialNumberValue);

        if (!match.find()) {
            return false;
        }

        String cvrNumber = match.group();
        cvrNumber = cvrNumber.replace("CVR:", "");
        cvrNumber = cvrNumber.trim();

        cvrNumberString.setVariable(cvrNumber);

        return cvrNumber != null && !cvrNumber.isEmpty();
    }

    /**
     * Resolve certificate type from subject.
     *
     * @param certificate The certificate to get the type of
     * @return The certificate type
     * @throws MainException On error...
     */
    private static OcesCertificateType getFromSubject(X509Certificate certificate) throws MainException {

        OcesX509CertificateConfig _config = ConfigurationHandler.getInstance().getOcesX509CertificateConfig();

        CertificateSubject subject = new CertificateSubject(certificate.getSubjectX500Principal().toString());
        String ssn = subject.getSubjectString();

        if (ssn == null) {
            return new OcesCertificateType(OcesCertificateType.NonOces);
        }

        if (ssn.indexOf(_config.get_personalCertificateSubjectKey().getSubjectKeyString()) != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesPersonal);
        }
        if (ssn.indexOf(_config.get_functionCertificateSubjectKey().getSubjectKeyString()) != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesDevice);
        }
        if (ssn.indexOf(_config.get_organizationCertificateSubjectKey().getSubjectKeyString()) != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesOrganisation);
        }
        if (ssn.indexOf(_config.get_employeeCertificateSubjectKey().getSubjectKeyString()) != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesEmployee);
        }

        // Old OCES test certificates have DID
        //if (ssn.indexOf("DID:") != -1) {
        //    return new OcesCertificateType(OcesCertificateType.OcesDevice);
        //}

        return new OcesCertificateType(OcesCertificateType.NonOces);
    }

    @Override
    public String toString() {
        return subject.getSubjectString();
    }
}