package dk.gov.oiosi.security.revocation.crl;

import java.net.URL;
import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;

import dk.gov.oiosi.common.schedule.AbstractScheduleJob;

/**
 * Describes job for updating CRL cache. Marked as friendly to be accessible
 * only inside the same package.
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * 
 */
public class CrlCacheUpdateJob extends AbstractScheduleJob {

	private static final long serialVersionUID = 3534084002214501623L;

	private static final Logger log = Logger.getLogger(CrlCacheUpdateJob.class);

	private final URL crlUrl;
	private final String actionDetail;
	private final String jobDetails;

	public CrlCacheUpdateJob(URL crlUrl, String actionDetail) {
		if (crlUrl == null) {
			throw new IllegalArgumentException("CrlCacheUpdateJob " + actionDetail + ": null url is passed");
		}
		this.actionDetail = actionDetail;
		this.crlUrl = crlUrl;
		StringBuilder sb = new StringBuilder();
		sb.append("CrlCacheUpdateJob.");
		sb.append(actionDetail);
		sb.append(" for ");
		sb.append(crlUrl);
		this.jobDetails = sb.toString();
	}

	@Override
	public boolean doJob() {
		CrlCache cache = CrlLookup.getCache();
		CrlInstance crl = cache.getCRL(crlUrl);
		/*
		 * In case of first execution of this job to init cache, previous line
		 * of code already loaded CRL data and scheduled a job to update it
		 * depending on CRL next revocation date.
		 * 
		 * Without the check below the cache would be updated twice - by call
		 * getCRL() above and by call crl.updateData() in try/catch block.
		 */
		Calendar c = Calendar.getInstance();
		c.add(Calendar.MINUTE, CrlCache.MINIMAL_CACHE_UPDATE_INTERVAL_MINUTES * -1);
		Date thisUpdate = crl.getLastUpdate();
		boolean updateRequired = thisUpdate == null || thisUpdate.before(c.getTime());
		if (!updateRequired) {
			if (log.isInfoEnabled()) {
				log.info("CrlCacheUpdateJob: it is not required to update CRL cache now for url '" + crlUrl + "', as it was already done less than 5 minutes ago: " + thisUpdate + ", skip scheduling new update job");
			}
			return false;
		}
		long start = System.currentTimeMillis();
		boolean success = true;
		try {
			crl.updateData();
		} catch (Exception e) {
			success = false;
			StringBuilder sb = new StringBuilder();
			sb.append("CrlCacheUpdateJob: failed to ");
			sb.append(actionDetail);
			sb.append(" CRL cache for url '");
			sb.append(crlUrl);
			sb.append("' after ");
			sb.append((System.currentTimeMillis() - start));
			sb.append(" ms, continue attempts later");
			log.error(sb.toString(), e);
			return true;
		}
		/*
		 * CRL update finished successfully - no exception is thrown.
		 * 
		 * A new job should be requested to update by new update time
		 */
		cache.scheduleCacheUpdate(crlUrl, crl.getNextUpdate());
		if (log.isInfoEnabled()) {
			log.info("CrlCacheUpdateJob: done CRL update in " + (System.currentTimeMillis() - start) + " ms, result " + (success ? "OK" : "FAILURE") + ", url '" + crlUrl + "', scheduled next run at " + crl.getNextUpdate());
		}
		return false;
	}

	@Override
	public Logger getLog() {
		return log;
	}

	@Override
	public String getJobDetails() {
		return jobDetails;
	}

}
