/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation.crl;

import dk.gov.oiosi.security.revocation.CheckCertificateRevokedUnexpectedException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.cert.*;
import java.util.*;

/**
 * Class used for storing CRLs retrieved from URLs in X509 certificates.
 */
public class CrlInstance {

    private static final Log log = LogFactory.getLog(CrlInstance.class);

    /*
      * dlk, 2012-03-20
      *
      * Marked as volatile to inform all client threads about modifications of a
      * link to cached data
      */
    private volatile X509CRL data;
    private volatile Date lastUpdate;
    private final URL url;
    private final CertificateFactory cf;

    /**
     * Creates a new CRLInstance instance.
     *
     * @param cf  A factory used for generating X509CRL class's from a stream.
     * @param url The URL corresponding to the CRL.
     */
    public CrlInstance(CertificateFactory cf, URL url) {
        this.cf = cf;
        this.data = null;
        this.lastUpdate = null;
        this.url = url;
    }

    /**
     * Checks whether the given certificate is revoked.
     * <p/>
     * Thread-safe
     *
     * @param cert The certificate to check whether revoked or not
     * @return Returns true if revoked, false otherwise.
     * @throws CheckCertificateRevokedUnexpectedException
     *          On error...
     */
    /*
      * dlk, 2012-03-19
      *
      * Removed synchronized modifier from method and cache expiration check -
      * cache is updated in separate job by replacing link to data. It is marked
      * as volatile and never cleared so if at first access it was not null, it
      * cannot be null at second access, and as it is volatile, we guarantee that
      * client threads will always use latest cache.
      */
    public boolean isRevoked(Certificate cert) throws CheckCertificateRevokedUnexpectedException {
        if (data != null) {
            return data.isRevoked(cert);
        }
        /*
           * By default, if instance didn't manage to initialize, all certificates
           * are considered as revoked, to avoid security issues. Certificates can
           * be revoked because the private key was stolen by attacher. So such
           * attacker can use it to send illegal data, and if e.g. NH server is
           * restarted, we could allow such revoked certificate...
           *
           * Of cause, certificates with bigger probability are not revoked, but
           * security performance improvements has lower priority than security.
           */
        return true;
    }

    /**
     * Downloads a new CRL from the URL stored in the class.
     *
     * @throws CheckCertificateRevokedUnexpectedException
     *          On error...
     */
    protected void updateData() throws CheckCertificateRevokedUnexpectedException {
        long start = System.currentTimeMillis();
        X509CRL crlData = loadData(url, cf);
        /*
           * Method loadData always should return non-null object, but in case of
           * future modifications let's protect the code from setting null
           * variable
           */
        if (crlData != null) {
            Date localLastUpdateDate = Calendar.getInstance().getTime();
            if (log.isInfoEnabled()) {
                logCacheState(System.currentTimeMillis() - start, crlData, localLastUpdateDate);
            }
            this.data = crlData;
            this.lastUpdate = localLastUpdateDate;
        }
    }

    /**
     * Load CRL data, always non-null, if any problem occurs - throws CheckCertificateRevokedUnexpectedException
     * <p/>
     * Marked as protected to be testable
     *
     * @param crlUrl  URL to CRL server
     * @param factory factory to process response from CRL server
     * @return loaded data, always returns non-null result
     * @throws CheckCertificateRevokedUnexpectedException
     *          only this kind of exceptions is returned
     */
    protected static X509CRL loadData(URL crlUrl, CertificateFactory factory) throws CheckCertificateRevokedUnexpectedException {
        try {
            HttpURLConnection urlConnection = (HttpURLConnection) crlUrl.openConnection();

            /*
                * TODO DLK: Add connection and read timeouts
                */

            // Only download .crl file if it is present on the server.
            if (urlConnection.getResponseCode() == HttpURLConnection.HTTP_OK) {
                InputStream input = urlConnection.getInputStream();
                try {
                    // Downloads the .crl file into an X509CRL object.
                    return (X509CRL) factory.generateCRL(input);
                } finally {
                    try {
                        input.close();
                    } catch (Exception e) {
                        // Hide exception here, not relevant
                        log.warn("Error closing InputStream: " + e.getMessage(), e);
                    }
                }
                // Everything went well.
            } else {
                // url connection not valid, throwing an exception
                throw new CheckCertificateRevokedUnexpectedException("CRL could not be downloaded by url " + crlUrl + ": " + urlConnection.getResponseMessage());
            }
        } catch (IOException e) {
            // Could not download new crl
            throw new CheckCertificateRevokedUnexpectedException("IO error via access to url " + crlUrl, e, crlUrl);
        } catch (CRLException e) {
            // Something was wrong with crl
            throw new CheckCertificateRevokedUnexpectedException("CRL problem during access to url " + crlUrl, e, crlUrl);
        } catch (Exception e) {
            throw new CheckCertificateRevokedUnexpectedException("Unexpected exception during access to url " + crlUrl, e, crlUrl);
        }
    }

    /**
     * The NextUpdate value from the CRL
     *
     * @return The NextUpdate value from the CRL, if some data are available in
     *         this instance. Otherwise it is client responsibility to replace
     *         it with some another date.
     */
    /*
      * dlk, 2012-03-19
      *
      * Removed synchronized modifier. This object cannot be cleared if it was
      * initialized - protected in updateData method. Also data is marked as
      * volatile
      */
    public Date getNextUpdate() {
        if (data != null) {
            return data.getNextUpdate();
        }
        return null;
    }

    /**
     * Get thisUpdate value from the CRL.
     *
     * @return The ThisUpdate value from the CRL, if some data are available in
     *         this instance. Otherwise it is client responsibility to replace
     *         it with some another date.
     * @since OIORASP 1.3.0
     */
    public Date getCrlThisUpdate() {
        if (data != null) {
            return data.getThisUpdate();
        }
        return null;
    }

    /**
     * Local time when crl cache was successfully updated.
     *
     * @return The lastUpdate value is changed each time when CRL cache was
     *         successfully updated. It can be different to getCrlThisUpdate()
     *         result because returns local time value from the CRL, which is
     *         different to time when CRL data are generated on CRL server it
     *         with some another date.
     * @since OIORASP 1.3.0
     */
    public Date getLastUpdate() {
        return lastUpdate;
    }

    /**
     * Convenience method to see size of CRL cached data and its status
     *
     * @param localLastUpdate
     */
    protected void logCacheState(long elapsed, X509CRL crlData, Date localLastUpdate) {
        log.info("CrlInstance: it took " + elapsed + " ms to lookup into " + url);
        log.info("CrlInstance: CRL next update date: " + crlData.getNextUpdate());
        log.info("CrlInstance: CRL this update date: " + crlData.getThisUpdate());
        log.info("CrlInstance: Local last update date: " + localLastUpdate);

        /*
           * Note - if the set (crlData.getRevokedCertificates()) is empty, null
           * is returned (NOT a empty list)
           */
        Set<? extends X509CRLEntry> revokedCertificates = crlData.getRevokedCertificates();
        log.info("CrlInstance: Revoked certificates size: " + (revokedCertificates != null ? revokedCertificates.size() : 0));

        if (revokedCertificates != null) {
            Date maxRevocationDate = null;
            Map<Integer, Integer> groupByYear = new LinkedHashMap<Integer, Integer>();
            Calendar c = Calendar.getInstance();
            for (X509CRLEntry x509crlEntry : revokedCertificates) {
                Date revocationDate = x509crlEntry.getRevocationDate();
                if (maxRevocationDate == null || revocationDate.after(maxRevocationDate)) {
                    maxRevocationDate = revocationDate;
                }
                c.setTime(revocationDate);
                int year = c.get(Calendar.YEAR);
                Integer count = groupByYear.get(year);
                if (count == null) {
                    groupByYear.put(year, 1);
                } else {
                    groupByYear.put(year, 1 + count);
                }
            }

            log.info("CrlInstance: Revoked records count by year: " + groupByYear);
            log.info("CrlInstance: Latest revocation date: " + maxRevocationDate);
        } else {
            log.info("CrlInstance: None certificate is recorded as revoked");
        }
    }
}