package dk.gov.oiosi.security.revocation.crl;

import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.List;

import dk.gov.oiosi.security.revocation.CheckCertificateRevokedUnexpectedException;
import dk.gov.oiosi.security.revocation.RevocationException;
import dk.gov.oiosi.security.revocation.RevocationResponse;

/**
 * Simplified implementation of CrlLookup without OCSP2 certificates support.
 * Such certificates have 3 level structure, when additionally to root
 * certificate and functional certificate in between them there can be also
 * intermediate certificate, which should also be checked against revocation
 * server.
 * <p>
 * The goal of this class - provide possibility to test OIORASP 1.3.0 without
 * OCSP2 certificates, also as to start using it in production without OCSP2.
 * <p>
 * To use this class instead of CrlLookup, which supports OCSP2,
 * RaspConfiguration.xml should be updated:
 * <p>
 * 
 * <pre>
 * &lt;ConfigurationSection xsi:type="RevocationLookupFactoryConfig"&gt;
 * <p>
 * &lt;ImplementationNamespaceClass&gt;dk.gov.oiosi.security.revocation.crl.SimpleCrlLookup&lt;/ImplementationNamespaceClass&gt;
 * <p>
 * &lt;/ConfigurationSection&gt;
 * </pre>
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class SimpleCrlLookup extends CrlLookup {

	/**
	 * Checks a certificate status in a CRL.
	 * 
	 * @param certificate The certificate to check
	 * @return The RevocationResponse object that contains the result
	 * @throws RevocationException
	 */
	public RevocationResponse checkCertificate(X509Certificate certificate) throws RevocationException {
		/*
		 * Assumptions: - Certificate has an CRL Distribution Points extension
		 * value. - An HTTP distribution point is present.
		 */

		/*
		 * Fail fast!
		 */
		if (certificate == null) {
			throw new RevocationException("Fault happened at CrlLookup.checkCertificate: null certificate was passed");
		}

		RevocationResponse response = new RevocationResponse();
		RevocationException innerException = null;

		/*
		 * Retrieve URL distribution points
		 */
		List<URL> urlList = getURLs(certificate);

		for (URL url : urlList) {
			CrlInstance crl = cache.getCRL(url);
			try {
				if (!crl.isRevoked(certificate)) {
					response.setIsValid(true);
					response.setNextUpdate(crl.getNextUpdate());
					return response;
				} else {
					response.setIsValid(false);
					return response;
				}
			} catch (CheckCertificateRevokedUnexpectedException e) {
				/*
				 * There was an error in checking the certificate. Try the next
				 * url. Remember first error.
				 */
				if (innerException == null) {
					StringBuilder sb = new StringBuilder();
					sb.append("Fault happened at CrlLookup.checkCertificate: all extracted CRL URLs failed, at first for URL ");
					sb.append(url);
					sb.append(" for certificate ");
					sb.append(certificate.getSubjectX500Principal());
					sb.append(", all resolved URLs: ").append(urlList);
					innerException = new RevocationException(sb.toString(), e);
				}
			}
		}

		/*
		 * If all URLs failed to do CRL check, then abort the processing of the
		 * message.
		 */
		if (innerException != null) {
			throw innerException;
		} else {
			/*
			 * This can happen ONLY if getURLs returned empty list of
			 * certificates
			 */
			StringBuilder sb = new StringBuilder();
			sb.append("Fault happened at CrlLookup.checkCertificate: none URL to CRL server is retrieved from certificate ");
			sb.append(certificate.getSubjectX500Principal());
			throw new RevocationException(sb.toString());
		}
	}

}
