/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation.ocsp;

import dk.gov.oiosi.communication.Uri;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URISyntaxException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.CertificateHandlingException;
import dk.gov.oiosi.security.RootCertificateConfig;
import dk.gov.oiosi.security.lookup.CertificateLoader;

/**
 * OCSP dynamic configuration.
 */
public class OcspConfig {
     /**
     * url.
     */
    private Uri serverUrl = null;

     /**
     * timeout.
     */
    private long defaultTimeoutMsec = 10000;

    /**
     * Default constructor that initializes the OscpConfig with default values.
     */
    public OcspConfig() { }

    /**
     * Gets the URL of the OCSP server.
     * 
     * @return ServerUrl
     */
    public String getServerUrl() {
        if (serverUrl == null) {
            return null;
        }
        else {
            return serverUrl.toString();
        }    	
    }

    /**
     * Gets the URL of the OCSP server.
     * 
     * @param value
     *            ServerUrl
     * @throws URISyntaxException 
     */
    public void setServerUrl(String value) throws URISyntaxException {
        serverUrl = new Uri(value);
    }

    /**
     * Sets the URL of the OCSP server.
     * 
     * @param value
     *            ServerUrl
     */
    public void setServerUrl(Uri value) {
        serverUrl = value;
    }

    /**
     * The default timeout in milliseconds for the OCSP lookup operation.
     * 
     * @return DefaultTimeoutMsec
     */
    public long getDefaultTimeoutMsec() {
        return defaultTimeoutMsec;
    }

    /**
     * The default timeout in milliseconds for the OCSP lookup operation.
     * 
     * @param value
     *            DefaultTimeoutMsec
     */
    public void setDefaultTimeoutMsec(long value) {
        if (defaultTimeoutMsec < 0) {
            throw new IllegalArgumentException("Timeout value < 0 must be greater than -1.");
        }
        defaultTimeoutMsec = value;
    }

     public List<X509Certificate> getDefaultOcesRootCertificateCollectionFromStore() throws CertificateHandlingException {

        List<X509Certificate> x509List = new ArrayList<X509Certificate>();
        try {
            ConfigurationHandler confighandler = ConfigurationHandler.getInstance();
            List<RootCertificateConfig>	rootCertificateList	= confighandler.getRootCertificateLocationCollection();
            for (RootCertificateConfig rootCertificate : rootCertificateList)
            {
                x509List = CertificateLoader.GetCertificateListFromCertificateStoreInformation(rootCertificate);
            }
        } catch (KeyStoreException e) {
            throw new CertificateHandlingException(e);
        } catch (NoSuchAlgorithmException e) {
            throw new CertificateHandlingException(e);
        } catch (CertificateException e) {
            throw new CertificateHandlingException(e);
        } catch (FileNotFoundException e) {
            throw new CertificateHandlingException(e);
        } catch (IOException e) {
            throw new CertificateHandlingException(e);
        }

        return x509List;
    }
}
