/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */
package dk.gov.oiosi.uddi;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.exception.ArgumentNullException;

import java.util.List;

/**
 * The LookupParameters class is used as argument when doing an UDDI lookup.
 */
public class LookupParameters {

    public static final String RASPPROFILECONFORMANCECLAIM = "http://oio.dk/profiles/OIOSI/1.0/secureReliableAsyncProfile/1.0/";
    public static final String BASICPROFILECONFORMANCECLAIM = "http://oio.dk/profiles/BasicProfile1.1/";
    public static final String MODELTPROFILECONFORMANCECLAIM = "http://oio.dk/profiles/OWSA/modelT/1.0/";
    public static final String OTHERPROFILECONFORMANCECLAIM = "http://oio.dk/profiles/otherProfile/";

    /**
     * E.g. EAN 285829391
     */
    private Identifier identifier;

    /**
     * E.g. the Uddi ID corresponding to a service (Order, Invoice)
     */
    private UddiId serviceId;

    /**
     * E.g. the Uddi ID corresponding to a profile (Catalogue-CatAdv-1.0)
     */
    private List<UddiId> profileIds;

    /**
     * E.g. BuyerParty
     */
    private String profileRoleIdentifier;

    /**
     * E.g. http://oio.dk/profiles/OIOSI/1.0/secureReliableAsyncProfile/1.0/
     */
    private String profileConformanceClaim = "http://oio.dk/profiles/OIOSI/1.0/secureReliableAsyncProfile/1.0/";

    /**
     * E.g. http or email
     */
    private List<EndpointAddressTypeCode> acceptedTransportProtocols;

    /**
     * @param identifier
     * @param serviceId
     * @param profileIds
     * @param acceptedTransportProtocols
     * @param profileRoleIdentifier
     * @param profileConformanceClaim
     */
    public LookupParameters(
            Identifier identifier,
            UddiId serviceId,
            List<UddiId> profileIds,
            List<EndpointAddressTypeCode> acceptedTransportProtocols,
            String profileRoleIdentifier,
            String profileConformanceClaim) {

        if (identifier == null) {
            throw new ArgumentNullException("identifier");
        }
        if (serviceId == null) {
            throw new ArgumentNullException("serviceId");
        }
        if (profileIds == null) {
            throw new ArgumentNullException("profileIds");
        }
        if (acceptedTransportProtocols == null) {
            throw new ArgumentNullException("acceptedTransportProtocols");
        }
        if (profileRoleIdentifier == null) {
            throw new ArgumentNullException("profileRoleIdentifier");
        }
        if (profileIds.size() == 0) {
            throw new IllegalArgumentException("profileIds must contain at least one item");
        }
        if (profileConformanceClaim == null) {
            throw new ArgumentNullException("profileConformanceClaim");
        }

        this.identifier = identifier;
        this.serviceId = serviceId;
        this.profileIds = profileIds;
        this.acceptedTransportProtocols = acceptedTransportProtocols;
        this.profileRoleIdentifier = profileRoleIdentifier;
        this.profileConformanceClaim = profileConformanceClaim;
    }

    /**
     * @param identifier
     * @param serviceId
     * @param profileIds
     * @param acceptedTransportProtocols
     * @param profileRoleIdentifier
     */
    public LookupParameters(
            Identifier identifier,
            UddiId serviceId,
            List<UddiId> profileIds,
            List<EndpointAddressTypeCode> acceptedTransportProtocols,
            String profileRoleIdentifier) {

        if (identifier == null) {
            throw new ArgumentNullException("identifier");
        }
        if (serviceId == null) {
            throw new ArgumentNullException("serviceId");
        }
        if (profileIds == null) {
            throw new ArgumentNullException("profileIds");
        }
        if (acceptedTransportProtocols == null) {
            throw new ArgumentNullException("includedTransportProtocols");
        }
        if (profileRoleIdentifier == null) {
            throw new ArgumentNullException("profileRoleIdentifier");
        }
        if (profileIds.size() == 0) {
            throw new IllegalArgumentException("profileIds must contain at least one item");
        }

        this.identifier = identifier;
        this.serviceId = serviceId;
        this.profileIds = profileIds;
        this.acceptedTransportProtocols = acceptedTransportProtocols;
        this.profileRoleIdentifier = profileRoleIdentifier;
    }

    /**
     * @param identifier
     * @param serviceId
     * @param acceptedTransportProtocols
     * @param profileConformanceClaim
     */
    public LookupParameters(
            Identifier identifier,
            UddiId serviceId,
            List<EndpointAddressTypeCode> acceptedTransportProtocols,
            String profileConformanceClaim) {
        if (identifier == null) {
            throw new ArgumentNullException("identifier");
        }
        if (serviceId == null) {
            throw new ArgumentNullException("serviceId");
        }
        if (acceptedTransportProtocols == null) {
            throw new ArgumentNullException("acceptedTransportProtocols");
        }
        if (profileConformanceClaim == null) {
            throw new ArgumentNullException("profileConformanceClaim");
        }

        this.identifier = identifier;
        this.serviceId = serviceId;
        this.acceptedTransportProtocols = acceptedTransportProtocols;
        this.profileConformanceClaim = profileConformanceClaim;
    }

    /**
     * Constructor used to get endpoints that have a a specific identifier, service identifier,
     * some specific profile identifiers and some specific transport protocols.
     * <p/>
     * This can be used to get endpoints for a specific business, document and business process.
     *
     * @param identifier
     * @param serviceId
     * @param profileIds
     * @param acceptedTransportProtocols
     */
    public LookupParameters(
            Identifier identifier,
            UddiId serviceId,
            List<UddiId> profileIds,
            List<EndpointAddressTypeCode> acceptedTransportProtocols) {

        if (identifier == null) {
            throw new ArgumentNullException("identifier");
        }
        if (serviceId == null) {
            throw new ArgumentNullException("serviceId");
        }
        if (profileIds == null) {
            throw new ArgumentNullException("profileIds");
        }
        if (acceptedTransportProtocols == null) {
            throw new ArgumentNullException("includedTransportProtocols");
        }
        if (profileIds.size() == 0) {
            throw new IllegalArgumentException("profileIds must contain at least one item");
        }

        this.identifier = identifier;
        this.serviceId = serviceId;
        this.profileIds = profileIds;
        this.acceptedTransportProtocols = acceptedTransportProtocols;
    }

    /**
     * Constructor used to get endpoints that have a specific identifier, service identifier and
     * some specific transport protocols.
     * <p/>
     * This can be used to get the profiles that are supported by the receiver part.
     *
     * @param identifier
     * @param serviceId
     * @param acceptedTransportProtocols
     */
    public LookupParameters(Identifier identifier, UddiId serviceId, List<EndpointAddressTypeCode> acceptedTransportProtocols) {
        if (identifier == null) {
            throw new ArgumentNullException("identifier");
        }
        if (serviceId == null) {
            throw new ArgumentNullException("serviceId");
        }
        if (acceptedTransportProtocols == null) {
            throw new ArgumentNullException("acceptedTransportProtocols");
        }

        this.identifier = identifier;
        this.serviceId = serviceId;
        this.acceptedTransportProtocols = acceptedTransportProtocols;
    }

    public LookupParameters(Identifier identifier) {
        if (identifier == null) {
            throw new ArgumentNullException("identifier");
        }

        this.identifier = identifier;
    }

    /**
     * Constructor that is used to get all services for at specific identifier
     *
     * @param identifier                 The identifier.
     * @param acceptedTransportProtocols The protocols to accept.
     */
    public LookupParameters(
            Identifier identifier,
            List<EndpointAddressTypeCode> acceptedTransportProtocols) {
        if (identifier == null) {
            throw new ArgumentNullException("identifier");
        }
        if (acceptedTransportProtocols == null) {
            throw new ArgumentNullException("acceptedTransportProtocols");
        }

        this.identifier = identifier;
        this.acceptedTransportProtocols = acceptedTransportProtocols;
        this.profileConformanceClaim = RASPPROFILECONFORMANCECLAIM;
    }

    public Identifier getIdentifier() {
        return identifier;
    }

    public UddiId getServiceId() {
        return serviceId;
    }

    public List<UddiId> getProfileIds() {
        return profileIds;
    }

    public String getProfileRoleIdentifier() {
        return profileRoleIdentifier;
    }

    public String getProfileConformanceClaim() {
        return profileConformanceClaim;
    }

    public List<EndpointAddressTypeCode> getAcceptedTransportProtocols() {
        return acceptedTransportProtocols;
    }

    @Override
    public String toString() {
        return "LookupParameters{" +
                "identifier=" + identifier +
                ", serviceId=" + serviceId +
                ", profileIds=" + profileIds +
                ", profileRoleIdentifier='" + profileRoleIdentifier + '\'' +
                ", profileConformanceClaim='" + profileConformanceClaim + '\'' +
                ", acceptedTransportProtocols=" + acceptedTransportProtocols +
                '}';
    }
}