/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.itst.uddi.client.types.core.AccessPoint;
import dk.itst.uddi.client.types.core.BindingTemplate;
import dk.itst.uddi.client.types.core.TModel;

import java.util.LinkedList;
import java.util.List;

class UddiBinding {
    private final BindingTemplate template;
    private final List<TModel> tModels;

    public UddiBinding(BindingTemplate template, List<TModel> tModels) {
        if (template == null) {
            throw new ArgumentNullException("template");
        }
        if (tModels == null) {
            throw new ArgumentNullException("tModels");
        }

        this.template = template;
        this.tModels = tModels;
    }

    public EndpointAddress GetEndpointAddress() throws Exception {
        AccessPoint accessPointItem = template.getAccessPoint();
        if (accessPointItem == null) {
            throw new Exception("accessPoint type expected");
        }
        return IdentifierUtility.getEndpointAddressFromString(accessPointItem.getStringValue());
    }

    public List<ProcessRoleDefinition> GetProcesses() {
        List<ProcessRoleDefinition> processes = new LinkedList<ProcessRoleDefinition>();
        for (TModel model : tModels) {
            UddiTModel uddiTModel = new UddiTModel(model);
            if (!uddiTModel.IsProfileRole()) {
                continue;
            }
            String name = uddiTModel.getName();
            String description = uddiTModel.getDescription();
            String role = uddiTModel.getProfileRoleId();
            String roleType = uddiTModel.getProfileRoleTypeId();
            UddiId processDefinitionReferenceId = IdentifierUtility.getUddiIDFromString(uddiTModel.GetProcessDefinitionReferenceId());
            ProcessRoleDefinition roleDefinition = new ProcessRoleDefinition(name, description, role, roleType, processDefinitionReferenceId);
            processes.add(roleDefinition);
        }

        return processes;
    }

    /// <summary>
    ///
    /// </summary>
    /// <param name="profileUddiIds">A list of profiles of which only one needs to be found in order for the binding to support the profiles</param>
    /// <param name="roleIdentifier">If set to null non role check is performed and all roles are accepted.</param>
    /// <returns></returns>
    protected boolean SupportsOneOrMoreProfileAndRole(List<UddiId> profileUddiIds, String roleIdentifier) {
        for (TModel tModelItem : GetTModelProfiles()) {
            UddiTModel uddiTModel = new UddiTModel(tModelItem);
            String profile = uddiTModel.GetProcessDefinitionReferenceId();
            String role = uddiTModel.getProfileRoleId();
            boolean hasProfileAndRole = hasOneOrMoreProfileAndRole(profile, role, profileUddiIds, roleIdentifier);
            if (hasProfileAndRole) {
                return true;
            }
        }

        return false;
    }

    private boolean hasOneOrMoreProfileAndRole(String profile, String role, List<UddiId> profileUddiIds, String roleIdentifier) {
        if (profile == null) {
            return false;
        }
        if (role == null) {
            return false;
        }

        boolean hasProfile = false;
        for (UddiId profileUddiId : profileUddiIds) {
            hasProfile = profile.equalsIgnoreCase(profileUddiId.getID());
            if (hasProfile) {
                break;
            }
        }

        boolean hasRole;
        if (roleIdentifier == null) {
            hasRole = true;
        } else {
            hasRole = role.equalsIgnoreCase(roleIdentifier);
        }

        if (hasProfile && hasRole) {
            return true;
        }
        return false;
    }

    /// <summary>
    ///
    /// </summary>
    /// <returns>True if valid</returns>
    private Iterable<TModel> GetTModelProfiles() {
        List<TModel> modelProfiles = new LinkedList<TModel>();
        for (TModel tModelItem : tModels) {
            if (tModelItem == null) {
                continue;
            }
            if (tModelItem.getCategoryBag() == null) {
                continue;
            }
            if (tModelItem.getCategoryBag().getKeyedReferenceArray().length < 1) {
                continue;
            }

            UddiTModel uddiTModel = new UddiTModel(tModelItem);
            if (!uddiTModel.IsProfileRole()) {
                continue;
            }

            modelProfiles.add(tModelItem);
        }

        return modelProfiles;
    }
}