/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */
package dk.gov.oiosi.uddi;

import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.ICache;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.configuration.CacheFactory;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.exception.MainException;
import dk.itst.uddi.client.ConnectionInfo;
import dk.itst.uddi.client.InquiryClient;
import dk.itst.uddi.client.UDDIClient;
import dk.itst.uddi.client.query.FindServiceQuery;
import dk.itst.uddi.client.query.FindServiceResult;
import dk.itst.uddi.client.types.core.*;

import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * Class for resolving endpoints on the UDDI-based Address Resolution Service (ARS). Features of this class: * Resolves a
 * business level endpoint to a service endpoint, using the UDDI 2.0 inquiry API * Performs automatic caching of resolving
 * attempts * Supports caching policies based on call result feedback
 * <p/>
 * Extension points: * Extension mechanism for adding additional key types for resolving * Extendable configuration model
 */

public class UddiLookupClient implements IUddiLookupClient {

    public static final String RASPREGISTRATIONCONFORMANCECLAIM = "http://oio.dk/profiles/OIOSI/1.0/UDDI/registrationModel/1.1/";
    private static final ICache<UddiLookupKey, List<UddiService>> serviceCache = createServiceCache();
    private static final ICache<UddiId, UddiTModel> tModelCache = createTModelCache();

    private Uri endpointAddress;
    private UDDIClient _uddiProxy;

    /**
     * Default constructor. Attempts to load configuration from file.
     *
     * @param endpointAddress Endpoint address.
     * @throws dk.gov.oiosi.exception.MainException
     *          On error...
     */
    public UddiLookupClient(Uri endpointAddress) throws MainException {
        this.endpointAddress = endpointAddress;

        ConnectionInfo ci = new ConnectionInfo();
        ci.setInquiry(endpointAddress.toString() + "?platform=Java&raspVersion=2.0.0");
        try {
            _uddiProxy = new UDDIClient(ci);
        } catch (MalformedURLException e) {
            throw new MainException(e);
        }
    }

    public static ICache<UddiLookupKey, List<UddiService>> createServiceCache() {
        return CacheFactory.getInstance().getUddiServiceCache();
    }

    public static ICache<UddiId, UddiTModel> createTModelCache() {
        return CacheFactory.getInstance().getUddiTModelCache();
    }


    /**
     * Translates a business level key ("EndpointKey", e.g. an EAN number) to an endpoint address (e.g. an URL).
     *
     * @param lookupParameters The business level key of the endpoint, e.g. an EAN number as well as options of the
     *                         translation, i.e. address type filters
     * @return Returns a collection of matching addresses
     */
    public List<UddiLookupResponse> lookup(LookupParameters lookupParameters) throws MainException {
        if (lookupParameters == null) {
            throw new ArgumentNullException("lookupParameters");
        }

        List<UddiLookupResponse> uddiLookupResponses;
        try {
            uddiLookupResponses = GetUddiResponses(lookupParameters);
        } catch (Exception e) {
            throw new MainException(e);
        }

        if (lookupParameters.getAcceptedTransportProtocols() != null) {
            List<UddiLookupResponse> supportedResponses = new LinkedList<UddiLookupResponse>();
            for (UddiLookupResponse uddiLookupResponse : uddiLookupResponses) {
                boolean hasSupportedTransportProtocol = this.HasSupportedTransportProtocol(uddiLookupResponse, lookupParameters);
                if (hasSupportedTransportProtocol) {
                    supportedResponses.add(uddiLookupResponse);
                }
            }
            return supportedResponses;
        } else {
            // If null, then accept everything
            return uddiLookupResponses;
        }
    }

    public List<ProcessDefinition> GetProcessDefinitions(List<UddiId> processDefinitionIds) {
        List<UddiId> missingIds = new LinkedList<UddiId>();
        List<UddiTModel> foundTModels = new LinkedList<UddiTModel>();

        //Check the cache for any existing tmodels.
        for (UddiId processDefinitionId : processDefinitionIds) {
            OutVariable<UddiTModel> out = new OutVariable<UddiTModel>();
            if (tModelCache.tryGetValue(processDefinitionId, out)) {
                foundTModels.add(out.getVariable());
                continue;
            }
            missingIds.add(processDefinitionId);
        }

        //Get the tmodels not in the cache
        List<UddiTModel> tmodels = this.GetUddiTModels(missingIds);
        //Adds the tmodels to the cache
        for (UddiTModel tmodel : tmodels) {
            tModelCache.set(tmodel.getUddiId(), tmodel);
        }

        List<ProcessDefinition> processDefinitions = new LinkedList<ProcessDefinition>();
        foundTModels.addAll(tmodels);
        for (UddiTModel tmodel : foundTModels) {
            UddiId uddiId = tmodel.getUddiId();
            String name = tmodel.getName();
            String description = tmodel.getDescription();
            String profileId = tmodel.GetProfileId();
            String profileTypeId = tmodel.GetProfileTypeId();
            String registrationConformanceClaim = tmodel.GetRegistrationConformanceClaim();

            ProcessDefinition processDefinition = new ProcessDefinition(uddiId, name, description, profileId, profileTypeId, registrationConformanceClaim);
            processDefinitions.add(processDefinition);
        }
        return processDefinitions;
    }

    private boolean HasSupportedTransportProtocol(UddiLookupResponse uddiLookupResponse, LookupParameters lookupParameters) {
        EndpointAddress address = uddiLookupResponse.getEndpointAddress();
        return lookupParameters.getAcceptedTransportProtocols().contains(address.getEndpointAddressTypeCode());
    }

    private List<UddiLookupResponse> GetUddiResponses(LookupParameters lookupParameters) throws Exception {
        boolean filterResponseByProfile = lookupParameters.getProfileIds() != null;

        UddiLookupKey key = new UddiLookupKey(lookupParameters.getIdentifier(), lookupParameters.getServiceId(), endpointAddress);
        List<UddiService> uddiServices;
        OutVariable<List<UddiService>> uddiServicesOutVar = new OutVariable<List<UddiService>>();
        if (serviceCache.tryGetValue(key, uddiServicesOutVar)) {
            uddiServices = uddiServicesOutVar.getVariable();
        } else {
            uddiServices = GetUddiServices(lookupParameters.getIdentifier(), lookupParameters.getServiceId(), lookupParameters.getProfileConformanceClaim());

            // Do not cache negative results
            if (uddiServices.size() > 0) {
                serviceCache.set(key, uddiServices);
            }
        }

        List<UddiLookupResponse> lookupResponses = new LinkedList<UddiLookupResponse>();
        for (UddiService uddiService : uddiServices) {
            if (uddiService.isInactiveOrExpired()) {
                continue;
            }

            Iterable<UddiBinding> supportedBindings;
            if (filterResponseByProfile) {
                supportedBindings = uddiService.GetBindingsSupportingAnyProfileAndRole(lookupParameters.getProfileIds(), lookupParameters.getProfileRoleIdentifier());
            } else {
                supportedBindings = uddiService.getBindings();
            }

            for (UddiBinding uddiBinding : supportedBindings) {
                UddiLookupResponse lookupResponse = GetLookupResponse(lookupParameters, uddiService, uddiBinding);
                lookupResponses.add(lookupResponse);
            }
        }

        return lookupResponses;
    }

    private UddiLookupResponse GetLookupResponse(LookupParameters lookupParameters, UddiService uddiService, UddiBinding uddiBinding) throws Exception {
        return new UddiLookupResponse(
                lookupParameters.getIdentifier(),
                uddiBinding.GetEndpointAddress(),
                uddiService.GetActivationDateUtc(),
                uddiService.GetExpirationDateUtc(),
                uddiService.GetCertificateSubject(),
                uddiService.GetTermsOfUseUrl(),
                uddiService.GetContactMail(),
                uddiService.GetVersion(),
                uddiService.GetNewerVersion(),
                uddiBinding.GetProcesses()
        );
    }

    private List<UddiService> GetUddiServices(Identifier organizationIdentifier, UddiId serviceUddiId, String profileConformanceClaim) throws MalformedURLException {
        KeyedReference profileConformanceClaimReference = KeyedReference.Factory.newInstance();
        profileConformanceClaimReference.setTModelKey("uddi:cc5f1df6-ae0a-4781-b24a-f30315893af7");
        profileConformanceClaimReference.setKeyName("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Categories/profileConformanceClaim/");
        profileConformanceClaimReference.setKeyValue(profileConformanceClaim);

        KeyedReference registrationConformanceClaim = KeyedReference.Factory.newInstance();
        registrationConformanceClaim.setTModelKey("uddi:80496ef5-4d24-4788-a3f8-12fb54a75106");
        registrationConformanceClaim.setKeyName("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Categories/registrationConformanceClaim/");
        registrationConformanceClaim.setKeyValue(RASPREGISTRATIONCONFORMANCECLAIM);

        KeyedReference endpointKeyType = KeyedReference.Factory.newInstance();
        endpointKeyType.setTModelKey("uddi:182a4a2b-3717-4283-b97c-55cc3b684dae");
        endpointKeyType.setKeyName("http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/endpointKeyType/");
        endpointKeyType.setKeyValue(organizationIdentifier.getKeyTypeValue());

        KeyedReference endpointKey = KeyedReference.Factory.newInstance();
        endpointKey.setTModelKey("uddi:e733684d-9f40-40ff-8807-1d80abc7c665");
        endpointKey.setKeyName("http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/endpointKey/");
        endpointKey.setKeyValue(organizationIdentifier.getAsString());

        KeyedReference[] categories = new KeyedReference[]{profileConformanceClaimReference, registrationConformanceClaim, endpointKeyType, endpointKey};

        CategoryBag serviceCategories = CategoryBag.Factory.newInstance();
        serviceCategories.setKeyedReferenceArray(categories);

        FindServiceQuery q = _uddiProxy.getInquiry().findService();
        q.addQualifier("andAllKeys");

        if (serviceUddiId != null) {
            q.addTModel(serviceUddiId.getID());
        }

        for (KeyedReference reference : serviceCategories.getKeyedReferenceArray()) {
            q.addCategory(reference.getKeyName(), reference.getKeyValue(), reference.getTModelKey());
        }

        FindServiceResult serviceResult = q.execute();

        if (serviceResult.getServiceInfos().length == 0) {
            return new LinkedList<UddiService>();
        }

        List<String> endPointUddiIds = new LinkedList<String>();
        for (ServiceInfo serviceInfo : serviceResult.getServiceInfos()) {
            endPointUddiIds.add(serviceInfo.getServiceKey());
        }

        InquiryClient inquiry = _uddiProxy.getInquiry();

        List<UddiService> uddiServices = new LinkedList<UddiService>();
        for (BusinessService businessServiceItem : inquiry.getServices(endPointUddiIds)) {
            List<UddiBinding> uddiBindings = new LinkedList<UddiBinding>();
            for (BindingTemplate bindingTemplate : businessServiceItem.getBindingTemplates().getBindingTemplateArray()) {
                List<String> tModelKeys = new LinkedList<String>();
                for (TModelInstanceInfo tModelInfo : bindingTemplate.getTModelInstanceDetails().getTModelInstanceInfoArray()) {
                    tModelKeys.add(tModelInfo.getTModelKey());
                }

                TModel[] tModels = inquiry.getTModels(tModelKeys);

                UddiBinding uddiBinding = new UddiBinding(bindingTemplate, Arrays.asList(tModels));
                uddiBindings.add(uddiBinding);
            }

            UddiService uddiService = new UddiService(businessServiceItem, uddiBindings);
            uddiServices.add(uddiService);
        }

        return uddiServices;
    }

    private List<UddiTModel> GetUddiTModels(List<UddiId> uddiIds) {
        String[] tModelKeyArray = new String[uddiIds.size()];
        for (int i = 0; i < uddiIds.size(); i++) {
            tModelKeyArray[i] = uddiIds.get(i).getID();
        }

        TModel[] tmodelDetails = _uddiProxy.getInquiry().getTModels(tModelKeyArray);

        if (tmodelDetails == null) {
            return new LinkedList<UddiTModel>();
        }

        List<UddiTModel> uddiTmodels = new LinkedList<UddiTModel>();
        for (TModel tmodel : tmodelDetails) {
            UddiTModel uddiTmodel = new UddiTModel(tmodel);
            uddiTmodels.add(uddiTmodel);
        }
        return uddiTmodels;
    }
}