/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.MailAddress;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.security.CertificateSubject;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * Response from an UDDI lookup
 */
public class UddiLookupResponse {

    /**
     * NewerVersionReference for endpoint.
     */
    private UddiId newerVersionReference;

    /**
     * Gets or sets the actual endpoint identifier of the request. This may be
     * different from what the user/client specified only if that EAN number was
     * in the gateway range configuration, and if no individual UDDI
     * registration matching the client search criteria was found on the UDDI.
     * In this case this property holds the value of the gateway identifier.
     */
    private Identifier endpointIdentifierActual;

    /**
     * CertificateSubject == SubjectDN from endpoint certificate
     */
    private CertificateSubject certificateSubjectSerialNumber;

    /**
     * Activation date for endpoint.
     */
    private Date activationDate;

    /**
     * Expiration time for endpoint.
     */
    private Date expirationDate;

    /**
     * Terms of use for endpoint.
     */
    private Uri termsOfUseUrl;

    /**
     * Found endpoint address
     */
    private EndpointAddress endpointAddress;

    private MailAddress serviceContactEmail;

    /**
     * Version.
     */
    private Version version;

    private List<ProcessRoleDefinition> processRoles = new LinkedList<ProcessRoleDefinition>();


    /**
     * Default constructor.
     *
     * @deprecated
     */
    public UddiLookupResponse() {
    }

    /**
     * Constructor.
     *
     * @param endpointIdentifierActual       The actual endpoint identifier of the request. This may be
     *                                       different from what the user/client specified, e.g. it may be
     *                                       a gateway range EAN.
     * @param endpointAddress                Address of the endpoint
     * @param activationDate                 Activation date of the endpoint registration
     * @param expirationDate                 Expiration date of the endpoint registration
     * @param certificateSubjectSerialNumber Subject of the certificate
     * @param termsOfUseUrl                  URL to a file describing the terms of use
     * @param serviceContactEmail            Email of the service contact
     * @param version                        Version of the endpoint
     * @param newerVersionReference          Possible reference to a newer version
     * @deprecated
     */
    public UddiLookupResponse(Identifier endpointIdentifierActual, EndpointAddress endpointAddress, Date activationDate,
                              Date expirationDate, CertificateSubject certificateSubjectSerialNumber, Uri termsOfUseUrl,
                              MailAddress serviceContactEmail, Version version, UddiId newerVersionReference) {
        this.endpointIdentifierActual = endpointIdentifierActual;
        this.endpointAddress = endpointAddress;
        this.activationDate = activationDate;
        this.expirationDate = expirationDate;
        this.certificateSubjectSerialNumber = certificateSubjectSerialNumber;
        this.termsOfUseUrl = termsOfUseUrl;
        this.serviceContactEmail = serviceContactEmail;
        this.version = version;
        this.newerVersionReference = newerVersionReference;
    }

    public UddiLookupResponse(Identifier endpointIdentifierActual, EndpointAddress endpointAddress, Date activationDate,
                              Date expirationDate, CertificateSubject certificateSubjectSerialNumber, Uri termsOfUseUrl,
                              MailAddress serviceContactEmail, Version version, UddiId newerVersionReference, List<ProcessRoleDefinition> processes) {
        this.endpointIdentifierActual = endpointIdentifierActual;
        this.endpointAddress = endpointAddress;
        this.activationDate = activationDate;
        this.expirationDate = expirationDate;
        this.certificateSubjectSerialNumber = certificateSubjectSerialNumber;
        this.termsOfUseUrl = termsOfUseUrl;
        this.serviceContactEmail = serviceContactEmail;
        this.version = version;
        this.newerVersionReference = newerVersionReference;
        this.setProcessRoles(processes);
    }


    /**
     * Gets the EndpointIdentifierActual
     *
     * @return EndpointIdentifierActual
     */
    public Identifier getEndpointIdentifierActual() {
        return endpointIdentifierActual;
    }

    /**
     * EndpointIdentifierActual
     *
     * @param value EndpointIdentifierActual
     * @deprecated use setEndpointIdentifierActual(IIdentifier value)
     */
    public void getEndpointIdentifierActual(Identifier value) {
        endpointIdentifierActual = value;
    }

    /**
     * Sets the EndpointIdentifierActual
     *
     * @param value The identifier
     */
    public void setEndpointIdentifierActual(Identifier value) {
        endpointIdentifierActual = value;
    }

    /**
     * Gets found endpoint address
     *
     * @return EndpointAddress The address
     */
    public EndpointAddress getEndpointAddress() {
        return endpointAddress;
    }

    /**
     * Sets found endpoint address
     *
     * @param value The endpoint address
     */
    public void setEndpointAddress(EndpointAddress value) {
        endpointAddress = value;
    }

    /**
     * Gets expiration date of found endpoint registration
     *
     * @return Expiration date
     */
    public Date getExpirationDate() {
        return expirationDate;
    }

    /**
     * Sets expiration date of found endpoint registration.
     *
     * @param value Expiration date
     */
    public void setExpirationDate(Date value) {
        expirationDate = value;
    }

    /**
     * Gets activation date of found endpoint registration
     *
     * @return Activation date
     */
    public Date getActivationDate() {
        return activationDate;
    }

    /**
     * Sets activation date of found endpoint registration
     *
     * @param value Activation date
     */
    public void setActivationDate(Date value) {
        activationDate = value;
    }

    /**
     * Gets the certificate subject
     *
     * @return Certificate subject
     */
    public CertificateSubject getCertificateSubjectSerialNumber() {
        return certificateSubjectSerialNumber;
    }

    /**
     * Sets the certificate subject
     *
     * @param value Certificate subject
     */
    public void setCertificateSubjectSerialNumber(CertificateSubject value) {
        certificateSubjectSerialNumber = value;
    }


    /**
     * Gets the terms of use uri's of found endpoint address
     *
     * @return TermsOfUseUrl
     */
    public Uri getTermsOfUseUrl() {
        return termsOfUseUrl;
    }

    /**
     * Sets the terms of use uri's of found endpoint address
     *
     * @param value TermsOfUseUrl
     */
    public void setTermsOfUseUrl(Uri value) {
        termsOfUseUrl = value;
    }

    /**
     * Gets service contact e-mails of found endpoint address contacts
     *
     * @return ServiceContactEmail
     */
    public MailAddress getServiceContactEmail() {
        return serviceContactEmail;
    }

    /**
     * Set service contact emails of found endpoint address contacts
     *
     * @param value ServiceContactEmail
     */
    public void setServiceContactEmail(MailAddress value) {
        serviceContactEmail = value;
    }

    /**
     * Gets version of found endpoint address
     *
     * @return Version
     */
    public Version getVersion() {
        return version;
    }

    /**
     * Set version of found endpoint address
     *
     * @param value Version
     */
    public void setVersion(Version value) {
        version = value;
    }

    /**
     * Gets newer version reference on found endpoint address
     *
     * @return newerVersionReference
     */
    public UddiId getNewerVersionReference() {
        return newerVersionReference;
    }

    /**
     * Sets newer version reference on found endpoint address
     *
     * @param value newerVersionReference
     */
    public void setNewerVersionReference(UddiId value) {
        newerVersionReference = value;
    }

    /**
     * Gets a value indicating if there is a newer version of found
     * endpoints
     *
     * @return Returns true if there is a newer version
     */
    public boolean hasNewerVersion() {
        return newerVersionReference != null;
    }

    @Deprecated
    public void setProcesses(List<ProcessRoleDefinition> processes) {
        this.processRoles = processes;
    }

    @Deprecated
    public List<ProcessRoleDefinition> getProcesses() {
        return processRoles;
    }

    /**
     * Sets the process roles.
     *
     * @param processRoles List of ProcessRoleDefinition.
     */
    public void setProcessRoles(List<ProcessRoleDefinition> processRoles) {
        this.processRoles = processRoles;
    }

    /**
     * Gets the process roles.
     *
     * @return List of ProcessRoleDefinition.
     */
    public List<ProcessRoleDefinition> getProcessRoles() {
        return processRoles;
    }

    @Override
    public String toString() {
        return "UddiLookupResponse{" +
                "newerVersionReference=" + newerVersionReference +
                ", endpointIdentifierActual=" + endpointIdentifierActual +
                ", certificateSubjectSerialNumber=[" + certificateSubjectSerialNumber + "]" +
                ", activationDate=" + activationDate +
                ", expirationDate=" + expirationDate +
                ", termsOfUseUrl=" + termsOfUseUrl +
                ", endpointAddress=" + endpointAddress +
                ", serviceContactEmail=" + serviceContactEmail +
                ", version=" + version +
                ", processRoles=\n" + processRolesHelper(processRoles) +
                '}';
    }

    private String processRolesHelper(List<ProcessRoleDefinition> processRoleDefinitions) {
        StringBuilder out = new StringBuilder();
        for (ProcessRoleDefinition processRoleDefinition : processRoleDefinitions) {
            out.append(" : ").append(processRoleDefinition).append("\n");
        }

        return out.toString();
    }
}