/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */
package dk.gov.oiosi.uddi;

import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.ValUtil;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.MailAddress;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.itst.uddi.client.types.core.BusinessService;
import dk.itst.uddi.client.types.core.KeyedReference;

import java.net.URISyntaxException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

public class UddiService {
    private static final String activationDateId = "uddi:B5449299-B951-4266-9952-4C4470970782";
    private static final String certificateSubjectId = "uddi:e6feac92-0aee-4824-ae02-882760609e8a";
    private static final String expirationDateId = "uddi:2a8d2edc-cd1e-453f-9777-b623c80aaba5";
    private static final String newerVersionId = "uddi:23c1ed31-ee5c-4a13-983e-92a0815e6120";
    private static final String contactMailId = "uddi:5194201c-fc02-4d2e-8224-910939ac384d";
    private static final String termsOfUseUrlId = "uddi:8cc1e1c1-d0f4-4bbe-90e0-90cd1976d944";
    private static final String versionMajorId = "uddi:9c119cc4-cca5-4d98-bee2-5fe3999506e9";
    private static final String versionMinorId = "uddi:ba4afec4-f6c1-4205-8212-5eb0472000f9";
    private static final String versionRevisionId = "uddi:8e7ce808-0279-4042-8088-133667744c6f";

    private final BusinessService service;
    private final UddiCategoryBag categoryBag;
    private final List<UddiBinding> bindings;

    public UddiService(BusinessService service, List<UddiBinding> uddiBindings) {
        if (service == null) {
            throw new ArgumentNullException("service");
        }
        if (uddiBindings == null) {
            throw new ArgumentNullException("uddiBindings");
        }

        this.service = service;
        this.categoryBag = new UddiCategoryBag(service.getCategoryBag());
        bindings = uddiBindings;
    }

    public List<UddiBinding> getBindings() {
        return bindings;
    }

    public Date GetActivationDateUtc() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!this.categoryBag.tryGetKeyedReference(activationDateId, out)) {
            return new Date(Long.MIN_VALUE);
        }
        KeyedReference activationDate = out.getVariable();
        return getDatetimeFromLifetimeDates(activationDate.getKeyValue(), false);
    }

    public CertificateSubject GetCertificateSubject() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!this.categoryBag.tryGetKeyedReference(certificateSubjectId, out)) {
            return null;
        }
        KeyedReference certificateReference = out.getVariable();
        return new CertificateSubject(certificateReference.getKeyValue());
    }

    public Date GetExpirationDateUtc() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!this.categoryBag.tryGetKeyedReference(expirationDateId, out)) {
            return new Date(Long.MAX_VALUE);
        }
        KeyedReference expirationDate = out.getVariable();
        return getDatetimeFromLifetimeDates(expirationDate.getKeyValue(), false);
    }

    public UddiId GetNewerVersion() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!this.categoryBag.tryGetKeyedReference(newerVersionId, out)) {
            return null;
        }
        KeyedReference newVersKeyref = out.getVariable();

        if (newVersKeyref.getKeyValue() == null || newVersKeyref.getKeyValue().isEmpty()) {
            return null;
        }
        return IdentifierUtility.getUddiIDFromString(newVersKeyref.getKeyValue());
    }

    public Uri GetTermsOfUseUrl() throws URISyntaxException {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!this.categoryBag.tryGetKeyedReference(termsOfUseUrlId, out)) {
            return null;
        }
        KeyedReference termsOfUseUrl = out.getVariable();

        if (termsOfUseUrl.getKeyValue() == null || termsOfUseUrl.getKeyValue().isEmpty()) {
            return null;
        }
        return new Uri(termsOfUseUrl.getKeyValue());
    }

    public MailAddress GetContactMail() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        if (!this.categoryBag.tryGetKeyedReference(contactMailId, out)) {
            return null;
        }
        KeyedReference contactMail = out.getVariable();

        if (contactMail.getKeyValue() == null || contactMail.getKeyValue().isEmpty()) {
            return null;
        }
        return new MailAddress(contactMail.getKeyValue());
    }

    public Version GetVersion() {
        OutVariable<KeyedReference> out = new OutVariable<KeyedReference>();
        this.categoryBag.tryGetKeyedReference(versionMajorId, out);
        KeyedReference majorVersion = out.getVariable();

        out = new OutVariable<KeyedReference>();
        this.categoryBag.tryGetKeyedReference(versionMinorId, out);
        KeyedReference minorVersion = out.getVariable();

        out = new OutVariable<KeyedReference>();
        this.categoryBag.tryGetKeyedReference(versionRevisionId, out);
        KeyedReference revisionVersion = out.getVariable();

        int majorInt = 0;
        int minorInt = 0;
        int revisionInt = 0;

        if (majorVersion != null && majorVersion.getKeyValue() != null && !majorVersion.getKeyValue().isEmpty()) {
            majorInt = Integer.parseInt(majorVersion.getKeyValue());
        }
        if (minorVersion != null && minorVersion.getKeyValue() != null && !minorVersion.getKeyValue().isEmpty()) {
            minorInt = Integer.parseInt(minorVersion.getKeyValue());
        }
        if (revisionVersion != null && revisionVersion.getKeyValue() != null && !revisionVersion.getKeyValue().isEmpty()) {
            revisionInt = Integer.parseInt(revisionVersion.getKeyValue());
        }
        return new Version(majorInt, minorInt, revisionInt);
    }

    public Iterable<UddiBinding> GetBindingsSupportingAnyProfileAndRole(List<UddiId> profileUddiIds, String roleIdentifier) {
        List<UddiBinding> supportedBindings = new LinkedList<UddiBinding>();
        for (UddiBinding binding : bindings) {
            if (binding.SupportsOneOrMoreProfileAndRole(profileUddiIds, roleIdentifier)) {
                supportedBindings.add(binding);
            }
        }

        return supportedBindings;
    }

    /// <summary>
    /// Returns true if this service is inactive or expired, according to its UDDI registration.
    /// All registrations on the UDDI are assumed to follow danish time zone conventions
    /// </summary>
    /// <returns>Returns true if this service is inactive or expired, according to its UDDI registration.</returns>
    public boolean isInactiveOrExpired() {
        Date activationDateUTC = GetActivationDateUtc();
        Date expirationDateUTC = GetExpirationDateUtc();
        Date nowUTC = new Date();

        if (nowUTC.before(activationDateUTC)) {
            return true;
        }
        if (nowUTC.after(expirationDateUTC)) {
            return true;
        }
        return false;
    }

    /**
     * Returns a DateTime representation of either an endpoint expiration
     * or activation date, in UTC
     *
     * @param date              The date of an endpoint expiration or activation.
     * @param getActivationDate True if the date represents an activation date,
     *                          false if expiration date. This influences which default is returned, if no date
     *                          is present in the string (DateTime.MinValue for activation date,
     *                          DateTime.MaxValue for expiration time
     * @return Returns a DateTime representation of either an endpoint expiration
     *         or activation date, in UTC
     */
    private Date getDatetimeFromLifetimeDates(String date, boolean getActivationDate) {
        if (ValUtil.isNullOrEmpty(date)) {
            // Then set default date:
            if (getActivationDate) {
                return new Date(Long.MIN_VALUE);
            } else {
                return new Date(Long.MAX_VALUE);
            }
        }
        // 1 - try correct format : "2019-09-07T00:00:00.0000000Z"
        try {
            return new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSSSSS'Z'").parse(date);
        } catch (Exception ignore) {
        }
        // 2 - fall back to "yyyy-MM-dd"
        try {
            return new SimpleDateFormat("yyyy-MM-dd").parse(date);
        } catch (Exception ignore) {
        }
        return null;
    }

    @Override
    public String toString() {
        return service.getServiceKey();
    }
}