/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.util;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.WSSecurityException;
import org.apache.ws.security.components.crypto.CredentialException;
import org.apache.ws.security.components.crypto.Merlin;

import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.cert.X509Certificate;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * RaspMerlin handles error in Rampart - not being able to handle dynamically transferred certificates.
 */
public class RaspMerlin extends Merlin {

    private Log log = LogFactory.getLog(RaspMerlin.class);

    /**
     * certificate cache.
     */
    static ConcurrentMap<String, X509Certificate> certificates = new ConcurrentHashMap<String, X509Certificate>();

    /**
     * Constructor.
     *
     * @param properties properties
     * @throws CredentialException
     * @throws IOException
     */
    public RaspMerlin(Properties properties) throws CredentialException, IOException {
        super(properties);
        this.RemoveJVMCaCerts();
    }

    /**
     * Constructor.
     *
     * @param properties  properties
     * @param classLoader classLoader
     * @throws CredentialException
     * @throws IOException
     */
    public RaspMerlin(Properties properties, ClassLoader classLoader) throws CredentialException, IOException {
        super(properties, classLoader);
        this.RemoveJVMCaCerts();
    }


    private void RemoveJVMCaCerts() {
        try {
            // Remove JVM cacerts as trusted ca certificates
            cacerts = KeyStore.getInstance("JKS");
            cacerts.load(null, null);
        } catch (Exception e) {
            //e.printStackTrace();
            log.warn(e);
        }
    }


    /*
     * (non-Javadoc)
     *
     * @see org.apache.ws.security.components.crypto.CryptoBase#getAliasForX509Cert(byte[])
     */
    public String getAliasForX509Cert(byte[] certificateThumbprint) throws WSSecurityException {
        String alias = super.getAliasForX509Cert(certificateThumbprint);
        log.debug("getAliasForX509Cert, alias=" + alias);
        if (alias == null) {
            // alias is not found - check if certificate is saved for validation
            try {
                String b64 = new String(Base64.encodeBase64(certificateThumbprint));
                log.debug("getAliasForX509Cert, looking for certificate, key=" + b64);
                boolean hasAlias = getKeyStore().containsAlias(b64);
                if (hasAlias) {
                    // thumbprint alias exist
                    return b64;
                } else {
                    X509Certificate cert = (X509Certificate) certificates.get(b64);
                    if (cert == null) {
                        // not found ??
                    } else {
                        getKeyStore().setCertificateEntry(b64, cert);
                        log.debug("Certifiate succesfully retrieved from cache.");
                        return b64;
                    }
                }
            } catch (KeyStoreException ignore) {
                log.debug("Keystore error", ignore);
            }
        }
        return alias;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.ws.security.components.crypto.CryptoBase#getSKIBytesFromCert(java.security.cert.X509Certificate)
     */
    public byte[] getSKIBytesFromCert(X509Certificate certificate) throws WSSecurityException {
        byte[] bytes = super.getSKIBytesFromCert(certificate);
        try {
            String alias = getKeyStore().getCertificateAlias(certificate);
            log.debug("getSKIBytesFromCert, alias=" + alias);
            if (alias == null) {
                // if alias does not exist in keystore
                // - this is encryption certificate
                // - save certificate for validation on replies

                // use b64 thumbprint as key
                String b64 = new String(Base64.encodeBase64(bytes));

                certificates.put(b64, certificate);
                LogFactory.getLog(getClass()).debug("Certificate cached for later use, key=" + b64);
            }
        } catch (KeyStoreException ignore) {
            log.debug("Keystore error", ignore);
        }
        return bytes;
    }
}