/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml;

import dk.gov.oiosi.exception.MainException;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMXMLParserWrapper;
import org.apache.axiom.om.impl.jaxp.OMSource;
import org.apache.axiom.om.impl.llom.factory.OMXMLBuilderFactory;
import org.apache.axis2.util.XMLUtils;
import org.apache.commons.configuration.ConfigurationUtils;
import org.w3c.dom.Element;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.net.URL;

/**
 * Wraps the Java standard w3 and Axis Axiom XML-document representation.
 */
public class XmlDocument {

    protected String fileName = null;

    /**
     * org.w3c.dom representation of the document.
     */
    private Element w3Document;

    /**
     * Axiom representation of the document.
     */
    private OMElement omDocument;

    /**
     * Default constructor.
     */
    public XmlDocument() {
    }

    /**
     * Constructor that takes a w3c document.
     *
     * @param doc w3c document
     */
    public XmlDocument(Element doc) {
        setDocument(doc);
    }

    /**
     * Constructor that takes an Axiom document.
     *
     * @param doc Axiom document
     */
    public XmlDocument(OMElement doc) {
        setDocument(doc);
    }

    /**
     * Set document as an w3c document.
     *
     * @param doc w3c document
     */
    public void setDocument(Element doc) {
        w3Document = doc;
        omDocument = null;
    }

    /**
     * Set document as Axiom document.
     *
     * @param doc Axiom document
     */
    public void setDocument(OMElement doc) {
        w3Document = null;
        omDocument = doc;
    }

    /**
     * Returns the document as w3c document. Convert if needed.
     *
     * @return w3c document
     * @throws XmlDocumentException On error...
     */
    public Element getW3Document() throws XmlDocumentException {
        if (omDocument == null && w3Document == null) {
            return null;
        }
        if (w3Document != null) {
            return w3Document;
        }
        try {
            return XMLUtils.toDOM(omDocument);
        } catch (Exception e) {
            throw new XmlDocumentException(omDocument);
        }
    }

    /**
     * Returns the document as Axiom document. Convert if needed.
     *
     * @return Axiom document
     * @throws XmlDocumentException On error...
     */
    public OMElement getOMDocument() throws XmlDocumentException {
        if (omDocument == null && w3Document == null) {
            return null;
        }
        if (omDocument != null) {
            return omDocument;
        }
        try {
            return XMLUtils.toOM(w3Document);
        } catch (Exception e) {
            throw new XmlDocumentException(w3Document);
        }
    }

    /**
     * Loads the document from file system. Axiom format is used.
     *
     * @param file Name of document file
     * @throws MainException         On error...
     * @throws FileNotFoundException On error...
     */
    public void load(String file) throws MainException, FileNotFoundException {
        load(file, true);
    }

    /**
     * Loads the document from file system. Axiom format is used.
     *
     * @param file       Name of document file
     * @param buildAxiom set to false to avoid document building
     * @throws MainException         On error...
     * @throws FileNotFoundException On error...
     * @since OIORASP 1.3.0
     */
    public void load(String file, boolean buildAxiom) throws MainException, FileNotFoundException {
        XMLStreamReader parser;
        try {
            InputStream inputStream = locateInputStream(file);
            parser = XMLInputFactory.newInstance().createXMLStreamReader(inputStream);
            OMXMLParserWrapper builder = OMXMLBuilderFactory.createStAXOMBuilder(OMAbstractFactory.getOMFactory(), parser);
            OMElement doc = builder.getDocumentElement();

            /*
                * dlk, 2012-04-05
                *
                * In future versions in order to speead up big documents processing,
                * it is necessary to load documents without its building.
                *
                * In OIORASP 1.3.0 there is no optimization code for, so the method
                * load(String) should build the document by default.
                */
            if (buildAxiom) {
                doc.build();
            }
            /*
             * dlk, 2012-03-19
             *
             * Set file name with which this document was loaded for transformer caching purposes
             */
            fileName = file;
            omDocument = doc;
        } catch (FileNotFoundException e) {
            throw e;
        } catch (Exception e) {
            throw new XmlException("Failed to read XML : " + e.getMessage(), e);
        }
    }

    /**
     * Builds input stream object for given path
     *
     * @param file path to file. Uses
     *             {@link org.apache.commons.configuration.ConfigurationUtils}
     *             method locate(String) for resource search.
     * @return InputStream object for given path to file
     * @throws FileNotFoundException On I/O error...
     * @since OIORASP 1.3.0
     */
    public static InputStream locateInputStream(String file) throws FileNotFoundException {
        InputStream inputStream;
        try {
            URL url = ConfigurationUtils.locate(file);
            inputStream = url.openStream();
        } catch (Exception e) {
            inputStream = new BufferedInputStream(new FileInputStream(file), 4 * 1024);
        }
        return inputStream;
    }

    /**
     * @return true if this instance is based on AXIOM
     * @since OIORASP 1.3.0
     */
    public boolean isOMBased() {
        return omDocument != null;
    }

    /**
     * @return true if this instance is based on W3C DOM
     * @since OIORASP 1.3.0
     */
    public boolean isW3CBased() {
        return w3Document != null;
    }

    /**
     * @return depending on what this instance is based on, returns proper Source implementation
     * @since OIORASP 1.3.0
     */
    public Source getTransformationSource() {
        if (isOMBased()) {
            return new OMSource(omDocument);
        }
        return new DOMSource(w3Document);
    }

    @Override
    /**
     * @return String with information about state of the instance. Only for debug/logging purposes.
     */
    /*
      * E.g. this method is used in
      * dk.gov.oiosi.xml.schematron.SchematronValidator
      * .schematronValidateXmlDocument(XmlDocument, XmlDocument, boolean) error
      * message - it gives more information about problem than just exception
      * stack trace.
      */
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("XmlDocument [fileName=");
        builder.append(fileName);
        builder.append(", w3cBased=");
        builder.append(isW3CBased());
        builder.append(", omBased=");
        builder.append(isOMBased());
        builder.append("]");
        return builder.toString();
    }

    /**
     * @return fileName with which this document was loaded. If XmlDocument was
     *         created with ready DOM or AXIOM object, returns null.
     * @since OIORASP 1.3.0
     */
    public String getFileName() {
        return fileName;
    }
}