/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.documenttype;

import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * Standard way to search for document types.
 */
public class DocumentTypeConfigSearcher {

    private Log log = LogFactory.getLog(DocumentTypeConfigSearcher.class);

    /**
     * document types.
     */
    private static DocumentTypeCollectionConfig documentTypeConfig;


    /**
     * Default constructor
     *
     * @throws ConfigurationException On error...
     */
    public DocumentTypeConfigSearcher() throws ConfigurationException {
        synchronized (DocumentTypeConfigSearcher.class) {
            if (documentTypeConfig == null) {
                documentTypeConfig = ConfigurationHandler.getInstance().getRaspDocumentTypeCollectionConfig();
            }
        }
    }

    /**
     * Searches the configuration handler for a document type that have the same
     * name and namespace as the XML-document given with the parameter.
     *
     * @param document The document to examine.
     * @return Returns a matching DocumentTypeConfig, or throws an exception if no match was found
     * @throws NoDocumentTypeFoundFromXmlDocumentException
     *
     * @throws SearchForDocumentTypeFromXmlDocumentFailedException
     *
     */
    public DocumentTypeConfig findUniqueDocumentType(XmlDocument document) throws NoDocumentTypeFoundFromXmlDocumentException,
            SearchForDocumentTypeFromXmlDocumentFailedException {
        if (document == null) {
            throw new ArgumentNullException("XmlDocument");
        }

        try {
            DocumentTypeConfig documentType = tryFindUniqueDocumentType(document);
            if (documentType == null) {
                log.error("Document not registered in RaspConfiguration.xml : " + document.getOMDocument().getLocalName());
                throw new NoDocumentTypeFoundFromXmlDocumentException(document);
            }
            return documentType;
        } catch (NoDocumentTypeFoundFromXmlDocumentException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new SearchForDocumentTypeFromXmlDocumentFailedException(document, ex);
        }
    }

    /**
     * Tries to find the XML-document type from the XML-document. If none can be
     * found it will return null, else the found type is returned. If
     * more than one is found it throws an exception.
     *
     * @param document The XmlDocument to examine.
     * @return null if not found or the DocumentTypeConfig if unique match exist
     * @throws XmlException, AmbiguousDocumentTypeFoundFromXmlDocumentException
     */
    public DocumentTypeConfig tryFindUniqueDocumentType(XmlDocument document)
            throws XmlException, AmbiguousDocumentTypeFoundFromXmlDocumentException {
        if (document == null) {
            throw new ArgumentNullException("document");
        }

        DocumentTypeConfig[] configList = documentTypeConfig.getDocumentTypes();

        List<DocumentTypeConfig> currentDocumentTypes = new ArrayList<DocumentTypeConfig>();
        for (DocumentTypeConfig config : configList) {
            if (config.isDocumentOfType(document)) {
                currentDocumentTypes.add(config);
            }
        }

        if (currentDocumentTypes.size() > 1) {
            throw new AmbiguousDocumentTypeFoundFromXmlDocumentException(document.getOMDocument());
        }
        if (currentDocumentTypes.size() < 1) {
            return null;
        }
        return (DocumentTypeConfig) currentDocumentTypes.get(0);
    }
}