/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.schema;

import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.xml.XMLConstants;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import java.io.File;
import java.net.URL;

/**
 * Represents a validator that can validate whether an XML-document is valid for a given XML-schema.
 */
public class SchemaValidator {

    private static Log log = LogFactory.getLog(SchemaValidator.class);

    /**
     * Validates whether the XML-document is valid to the given XML-schema. Further it ensures that the schema and the document has the same
     * namespace. If either the document is invalid or the namespaces are incorrect then an exception is thrown.
     *
     * @param xmlDocument document to check
     * @param xmlSchema   schema for validating
     * @throws SchemaValidationFailedException On error...
     */
    public void schemaValidateXmlDocument(XmlDocument xmlDocument, XmlSchema xmlSchema) throws SchemaValidationException {
        try {
            long start = System.currentTimeMillis();
            // create and load Schema
            SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema;

            // Try to read schema file. Try reading as resource - fallback to reading as file.
            try {
                URL url = SchemaValidator.class.getClassLoader().getResource(xmlSchema.getFileName());
                schema = factory.newSchema(url);
            } catch (Exception e) {
                File file = new File(xmlSchema.getFileName());
                schema = factory.newSchema(file);
                //Source source = new StreamSource(xmlSchema.getFileName());
                //schema = factory.newSchema(source);
            }

            // Java Schema validator validates namespaces automatically
            // checkNamespace(xmlDocument, xmlSchema);

            // create a validator
            Validator validator = schema.newValidator();

            if (log.isTraceEnabled()) {
                log.trace("SchemaValidator: built new validator object in " + (System.currentTimeMillis() - start) + " ms, implemented by class " + validator.getClass());
            }

            // validate document
            /*
                * dlk 2012.03.27
                *
                * Use proper transformation source depending on contents of
                * XmlDocument. For AXIOM - OMSource, for DOM - DOMSource
                */
            validator.validate(xmlDocument.getTransformationSource());
        } catch (Exception e) {
            log.error("Schema check failed : " + e.getMessage(), e);
            throw new SchemaValidationFailedException(xmlDocument, e);
        }
    }
}