package dk.gov.oiosi.xml.schematron.sax;

import dk.gov.oiosi.xml.schematron.SchematronErrorException;

import java.io.Serializable;
import java.util.Arrays;

/**
 * Contains configuration for SchematronValidatorSAXHandler basing on given
 * errorXPath and errorMessageXPath values.<br>
 *
 * Parses and validates given pair of XPath to be compatible with
 * {@link SchematronValidatorSAXHandler} implementation.<br>
 *
 * Parsing of these XPath values is separated from
 * {@link SchematronValidatorSAXHandler} to be able to cache it and do
 * parsing/validation.<br>
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 *
 */
public class SchematronValidatorSAXHandlerConfig implements Serializable {

	private static final long serialVersionUID = -1556678452625798808L;

	private final String[] tagList;
	private final String errorMessageTag;

	public SchematronValidatorSAXHandlerConfig(String[] tagList, String errorMessageTag) {
		this.tagList = tagList;
		this.errorMessageTag = errorMessageTag;
	}

	/**
	 * Builds new entity basing on given XPath pair.
	 *
	 * @param errorXPath
	 *            Very simple XPath, which is actually just list of
	 * @param errorMessageXPath
	 * @return
	 * @throws SchematronErrorException
	 */
	public static SchematronValidatorSAXHandlerConfig build(String errorXPath, String errorMessageXPath) throws SchematronErrorException {
		if (errorXPath == null || errorMessageXPath == null) {
			throw new SchematronErrorException("One of errorXPath or errorMessageXPath is null: errorXPath=" + errorXPath + ", errorMessageXPath = " + errorMessageXPath);
		}
		String[] tagListLocal;
		String errorMessageTagLocal;
		if (errorXPath.equals(errorMessageXPath)) {
			/*
			 * Save path to description tag in tagList, last tag - as
			 * errorMessageTag
			 */
			tagListLocal = splitXPathToTags(errorXPath);
			errorMessageTagLocal = tagListLocal[tagListLocal.length - 1];
			tagListLocal = Arrays.copyOfRange(tagListLocal, 0, tagListLocal.length - 1);
		} else {
			if (errorMessageXPath.startsWith(errorXPath)) {
				/*
				 * Save path to error in tagList, last tag of errorMessageXPath
				 * - as errorMessageTag
				 */
				tagListLocal = splitXPathToTags(errorXPath);
				errorMessageTagLocal = errorMessageXPath.substring(errorXPath.length());
				if (errorMessageTagLocal.startsWith("/")) {
					errorMessageTagLocal = errorMessageTagLocal.substring(1);
				}
			} else {
				throw new SchematronErrorException("Current version of SchematronValidatorSAXHandler does not support this combination of errorXPath and errorMessageXPath: errorXPath=" + errorXPath + ", errorMessageXPath = " + errorMessageXPath);
			}
		}
		return new SchematronValidatorSAXHandlerConfig(tagListLocal, errorMessageTagLocal);
	}

	private static String[] splitXPathToTags(String simpleXPath) throws SchematronErrorException {
		String line = simpleXPath;
		if (line.startsWith("/")) {
			line = line.substring(1);
		}
		if (line.endsWith("/")) {
			line = line.substring(0, line.length() - 1);
		}
		String tagList[] = line.split("/");

		for (int i = 0; i < tagList.length; i++) {
			tagList[i] = tagList[i].trim();
			if (!isOneWordString(tagList[i])) {
				throw new SchematronErrorException("XPath for SchematronValidatorSAXHandler should contain only list of tags delimited with slash, no XPath functions or selectors are supported. But in XPath '" + simpleXPath + "' part '" + tagList[i] + "' was found");
			}
		}
		return tagList;
	}

	/*
	 * Marked as protected to be testable
	 */
	protected static boolean isOneWordString(String text) {
		if (text != null) {
			String t = text.trim();
			if (t.length() > 0) {
				/*
				 * From Pattern javadoc:
				 *
				 * \w A word character: [a-zA-Z_0-9]
				 */
				return t.matches("\\w+");
			}
		}
		return false;
	}

	/**
	 * @return List of tags, which are parents of error message tag
	 */
	public String[] getTagList() {
		return tagList;
	}

	/**
	 * @return tag name, which stops processing and when it is closed, result
	 *         its contents should be returned by
	 *         {@link SchematronValidatorSAXHandler} as a message in
	 *         {@link SchematronValidatorSAXException}
	 */
	public String getErrorMessageTag() {
		return errorMessageTag;
	}

	@Override
	public String toString() {
		final int maxLen = 10;
		StringBuilder builder = new StringBuilder();
		builder.append("SchematronValidatorSAXHandlerConfig [tagList=");
		builder.append(tagList != null ? Arrays.asList(tagList).subList(0, Math.min(tagList.length, maxLen)) : null);
		builder.append(", errorMessageTag=");
		builder.append(errorMessageTag);
		builder.append("]");
		return builder.toString();
	}

}
