/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.xml.xpath;

import java.util.Collections;
import java.util.List;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMText;
import org.apache.axiom.om.xpath.AXIOMXPath;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.exception.NullOrEmptyArgumentException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;

/**
 * Resolves xpaths.
 * 
 * Uses Apache Axiom to do the hard work.
 */
public class DocumentXPathResolver {

	private static final Log log = LogFactory.getLog(DocumentXPathResolver.class);

    /**
     * Returns whether the given xpath yields any results.
     * 
     * @param xmlDocument
     *            document
     * @param xpath
     *            xpath
     * @param prefixedNamespaces
     *            namespaces
     * @return has result
     * @throws XmlException
     */
    public static boolean hasAnyElementsByXpath(XmlDocument xmlDocument, String xpath, PrefixedNamespace[] prefixedNamespaces)
            throws XmlException {
        checkParameters(xmlDocument, xpath, prefixedNamespaces);
        int nodesCount = getNodes(xmlDocument, xpath, prefixedNamespaces).size();
        return nodesCount > 0;
    }

    /**
     * Gets the value from an xpath expression. If there is no or multiple results from the xpath expression an exception is thrown.
     * 
     * @param xmlDocument
     *            The xml document to search
     * @param xpath
     *            The xpath expression to apply
     * @param prefixedNamespaces
     *            A list of namespace-prefixes used in the xpath expression
     * @return Returns the value from the xpath
     * @throws XmlException
     */
    public static String getElementValueByXpath(XmlDocument xmlDocument, String xpath, PrefixedNamespace[] prefixedNamespaces)
            throws XmlException {
        checkParameters(xmlDocument, xpath, prefixedNamespaces);

        // Get the nodes from the xpath query
        List<?> nodes = getNodes(xmlDocument, xpath, prefixedNamespaces);

        // Check whether the contrains are followed
        int nodesCount = nodes.size();
        if (nodesCount < 1)
            throw new NoXPathResultsException(xpath);
        if (nodesCount > 1)
            throw new TooManyXpathResultsException(xpath, nodesCount);

        // Return result based on type
        return getValue(nodes.get(0));
    }


    /**
     * Gets the first value from an xpath expression. If there are no results from the xpath expression an exception is thrown.
     * 
     * @param xmlDocument
     *            The xml document to search
     * @param xpath
     *            The xpath expression to apply
     * @param prefixedNamespaces
     *            A list of namespace-prefixes used in the xpath expression
     * @return The first result value from the xpath expression
     * @throws NoXPathResultsException
     * @throws XmlException
     */
    public static String getFirstElementValueByXPath(XmlDocument xmlDocument, String xpath, PrefixedNamespace[] prefixedNamespaces)
            throws NoXPathResultsException, XmlException {

        checkParameters(xmlDocument, xpath, prefixedNamespaces);

        String value = tryGetFirstElementValueByXPath(xmlDocument, xpath, prefixedNamespaces);
        if( value == null) {
            throw new NoXPathResultsException(xpath);
        }
        return value;
    }


    /**
     * Tries get the first value from an XPath expression. It returns true or false whether there was any results from the xpath
     * expression.
     * 
     * @param xmlDocument
     *            The XML-document to search
     * @param xpath
     *            The XPath expression to apply
     * @param prefixedNamespaces
     *            A list of namespace-prefixes used in the XPath expression
     * @return the first element value. Can be null there was none
     */
    public static String tryGetFirstElementValueByXPath(XmlDocument xmlDocument, String xpath,
                PrefixedNamespace[] prefixedNamespaces) throws XmlException {

        checkParameters(xmlDocument, xpath, prefixedNamespaces);

        // Get the nodes from the xpath query
        List<?> nodes = getNodes(xmlDocument, xpath, prefixedNamespaces);
        int nodesCount = nodes.size();
        if (nodesCount < 1) {
            return null;
        }
        
        return getValue(nodes.get(0));
    }

    /**
     * Gets the values resulted from an XPath query on the XML-document.
     * 
     * @param xmlDocument The XML-document to search
     * @param xpath The XPath expression to apply
     * @param prefixedNamespaces A list of namespace-prefixes used in the XPath expression
     * @return A string array containing the results from the XPath expression.
     */
    public static String[] getElementValuesByXPath(XmlDocument xmlDocument, String xpath, PrefixedNamespace[] prefixedNamespaces)
            throws XmlException {

        checkParameters(xmlDocument, xpath, prefixedNamespaces);

        // Get the nodes from the xpath query
        List<?> nodes = getNodes(xmlDocument, xpath, prefixedNamespaces);

        int nodesCount = nodes.size();
        String[] values = new String[nodesCount];
        for (int i = 0; i < nodesCount; i++) {
            values[i] = getValue(nodes.get(i));
        }

        return values;
    }

    /**
     * util.
     * 
     * @param xmlDocument
     *            document
     * @param xpath
     *            xpath
     * @param prefixedNamespaces
     *            namespaces
     * @throws XPathSizeTooSmallException
     */
    private static void checkParameters(XmlDocument xmlDocument, String xpath, PrefixedNamespace[] prefixedNamespaces)
            throws XPathSizeTooSmallException {
        if (xpath == null) {
            throw new NullOrEmptyArgumentException("xpath");
        }
        if (xmlDocument == null) {
            throw new ArgumentNullException("xmlDocument");
        }
        if (prefixedNamespaces == null) {
            throw new ArgumentNullException("prefixedNamespaces");
        }
        if (xpath.trim().length() <= 1) {
            if( ".".equals(xpath) || "/".equals(xpath)) {
                ; // these are legal
            } else {
                throw new XPathSizeTooSmallException(xpath);
            }
        }
    }

    /**
     * Return nodes searched by xpath.
     * 
     * @param xmlDocument
     *            document
     * @param xpath
     *            xpath
     * @param prefixedNamespaces
     *            namespaces
     * @return list of nodes
     * @throws XmlException
     */
    private static List<?> getNodes(XmlDocument xmlDocument, String xpath, PrefixedNamespace[] prefixedNamespaces) throws XmlException {
        AXIOMXPath xpathExpression;
        try {
            xpathExpression = new AXIOMXPath(xpath);
            for (int i = 0; i < prefixedNamespaces.length; i++) {
                PrefixedNamespace rdn = prefixedNamespaces[i];

                xpathExpression.addNamespace(rdn.getPrefix(), rdn.getNamespace());
            }
            List<?> nList = xpathExpression.selectNodes(xmlDocument.getOMDocument());
            return nList;
        } catch (Exception e) {
            throw new XmlException("Failed to perform Xpath search " + xpath + " : " + e.getMessage(), e);
        }
    }

	/**
	 * It was found out, that in case if XPath is defined starting from root
	 * entity, the whole AXIOM document is built.
	 * 
	 * To avoid this, let's use relative XPath, which start from given node.
	 * 
	 * For this purpose, we need to delete first part of XPath expression,
	 * assuming that all passed expressions to this method are very simple *
	 * 
	 * @param xmlDocument
	 * @param xpath
	 * @param prefixedNamespaces
	 * @return list of nodes
	 * @throws XmlException
	 */
    /*
     * TODO DLK: Integrate it into getNodes
     */
    protected static List<?> getNodesRelative(XmlDocument xmlDocument, String xpath, PrefixedNamespace[] prefixedNamespaces) throws XmlException {
    	long start = System.currentTimeMillis();
    	if (log.isDebugEnabled()) {
    		log.debug("Start getNodes for "+xpath);
    	}
		/*
		 * If given XPath starts with '/', it means that it is not relative.
		 * Replace it (by removing first search predicate) with corresponding
		 * relative XPath
		 */
    	String xpathToUse = xpath.trim();
        int firstSlashIndex = xpathToUse.indexOf('/');
        if (firstSlashIndex >= 0 && firstSlashIndex + 1 < xpathToUse.length()) {
        	int secondSlashIndex = xpathToUse.indexOf('/', firstSlashIndex+1);
        	if (secondSlashIndex > 0) {
        		xpathToUse = xpathToUse.substring(secondSlashIndex + 1);
        		if (log.isDebugEnabled()) {
        			log.debug("Modified xpath (removed root entity): "+xpathToUse);
        		}
        	}
        }
        NamedAXIOMXPath xpathExpression;
        try {
            xpathExpression = new NamedAXIOMXPath(xpathToUse);
            for (int i = 0; i < prefixedNamespaces.length; i++) {
                PrefixedNamespace rdn = prefixedNamespaces[i];
                xpathExpression.addNamespace(rdn.getPrefix(), rdn.getNamespace());
            }
            List<?> nList = xpathExpression.selectNodes(xmlDocument.getOMDocument());
            /*
             * dlk, 2012.02.28
             * 
             * By some reasons, xpath selector can return list with null if case nothing is found
             * 
             * TODO DLK: Find out why it can happen
             */
            if (nList.size() == 1 && nList.get(0) == null) {
            	nList = Collections.emptyList();
            }
            if (log.isDebugEnabled()) {
            	log.debug("Done getNodes for "+xpath + " (replaced with " + xpathToUse + "), in "+(System.currentTimeMillis()-  start)+", found "+(nList.size())+" results");
            }
            return nList;
        } catch (Exception e) {
            throw new XmlException("Failed to perform Xpath search " + xpath + " (replaced with " + xpathToUse + "), : " + e.getMessage(), e);
        }
    }
    
    
    /**
     * Get value depending on type of XML element.
     * @param object
     * @return value
     */
    private static String getValue(Object object) {
		/*
		 * dlk, 2012-04-05
		 * 
		 * If null object is returned, do not throw an exception, as it can be
		 * normal situation for relative XPath search via NamedXPath
		 */
    	if (object == null) {
    		return null;
    	}
        if (object instanceof OMElement) {
            OMElement e = (OMElement) object;
            return e.getText();
        } else if (object instanceof OMAttribute) {
            OMAttribute a = (OMAttribute) object;
            return a.getAttributeValue();
        } else if (object instanceof OMText) {
        	return ((OMText) object).getText();
        } else if (object instanceof String) {
        	return (String) object;
        }

        // for now only elements and attributes are searched in Rasp
        throw new RaspLibraryException("Unknown Type : " + object.getClass());
    }

}
