package dk.gov.oiosi.common.cache.stat;

import dk.gov.oiosi.common.cache.stat.CacheStatisticsData;
import edu.emory.mathcs.backport.java.util.concurrent.ExecutorService;
import edu.emory.mathcs.backport.java.util.concurrent.Executors;
import edu.emory.mathcs.backport.java.util.concurrent.TimeUnit;
import junit.framework.TestCase;

public class CacheStatisticsDataTest extends TestCase {

	private static final int THREAD_COUNT = 20;
	private static final int THREAD_POOL_SIZE = 10;
	private static final int REPEAT_THREAD_COUNT = 1000;

	/**
	 * This test shows that {@link CacheStatisticsData} implementation is thread
	 * safe and guarantees atomica operations on its contents
	 */
	public void testThreadSafety() {
		final CacheStatisticsData csd = new CacheStatisticsData();
		ExecutorService pool = Executors.newFixedThreadPool(THREAD_POOL_SIZE);

		/*
		 * Let's synchronize all threads to start simultaniously
		 */
		final Object waitMonitor = new Object();

		for (int i = 0; i < THREAD_COUNT; i++) {
			pool.execute(new Runnable() {
				@Override
				public void run() {
					try {
						synchronized (waitMonitor) {
							waitMonitor.wait(10);
						}
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					for (int i = 0; i < REPEAT_THREAD_COUNT; i++) {
						csd.incrementAddTotalCount();
						csd.toCompactStateString();
						csd.incrementCheckExpiredTotalCount();
						csd.toCompactStateString();
						csd.incrementCheckExpiredTotalRemovedCount();
						csd.toCompactStateString();
						csd.incrementContainsMatchCount();
						csd.toCompactStateString();
						csd.incrementContainsMissCount();
						csd.toCompactStateString();
						csd.incrementRemoveTotalCount();
						csd.toCompactStateString();
						csd.incrementSetTotalCount();
						csd.toCompactStateString();
						csd.incrementTryGetMatchCount();
						csd.toCompactStateString();
						csd.incrementTryGetMissCount();
						csd.toCompactStateString();
					}
				}
			});
		}

		synchronized (waitMonitor) {
			waitMonitor.notifyAll();
		}

		pool.shutdown();
		try {
			pool.awaitTermination(1, TimeUnit.DAYS);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		int expectedCount = THREAD_COUNT * REPEAT_THREAD_COUNT;

		assertEquals(expectedCount, csd.getAddTotalCount());
		assertEquals(expectedCount, csd.getCheckExpiredTotalCount());
		assertEquals(expectedCount, csd.getCheckExpiredTotalRemovedCount());
		assertEquals(expectedCount, csd.getContainsMatchCount());
		assertEquals(expectedCount, csd.getContainsMissCount());
		assertEquals(expectedCount, csd.getRemoveTotalCount());
		assertEquals(expectedCount, csd.getSetTotalCount());
		assertEquals(expectedCount, csd.getTryGetMatchCount());
		assertEquals(expectedCount, csd.getTryGetMissCount());

		String expectedState = "tryGet(match/miss)=20000/20000 contains(match/miss)=20000/20000 remove/set/add=20000/20000/20000 checkExpired(total/removedCount)=20000/20000";
		assertEquals(expectedState, csd.toCompactStateString());
		assertEquals(expectedState, csd.toString());
	}

	public void testToCompactStateString() {
		CacheStatisticsData d = new CacheStatisticsData();
		assertEquals("empty", String.valueOf(d));
		d.incrementAddTotalCount();
		assertEquals("remove/set/add=0/0/1", String.valueOf(d));
		d.init();
		d.incrementCheckExpiredTotalCount();
		assertEquals("checkExpired(total/removedCount)=1/0", String.valueOf(d.toCompactStateString()));
		d.init();
		d.incrementTryGetMatchCount();
		assertEquals("tryGet(match/miss)=1/0", String.valueOf(d.toCompactStateString()));
		d.incrementCheckExpiredTotalRemovedCount();
		assertEquals("tryGet(match/miss)=1/0 checkExpired(total/removedCount)=0/1", String.valueOf(d.toCompactStateString()));
		d.incrementCheckExpiredTotalRemovedCount(100);
		assertEquals(101, d.getCheckExpiredTotalRemovedCount());
		d.incrementCheckExpiredTotalRemovedCount(100);
		assertEquals(201, d.getCheckExpiredTotalRemovedCount());
		assertEquals("tryGet(match/miss)=1/0 checkExpired(total/removedCount)=0/201", String.valueOf(d.toCompactStateString()));
	}
	
}
