/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 */

package dk.gov.oiosi.security.revocation.crl;

import dk.gov.oiosi.common.schedule.ScheduledExecutor;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.revocation.IRevocationLookup;
import dk.gov.oiosi.security.revocation.RevocationResponse;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * jUnit test class for testing certificate revocation against crl.
 */
public class CrlLookupTest extends TestCase {

    @Override
    public void setUp() throws Exception {
        super.setUp();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);

        /*
        * User log4j to modify logging properties for some classes
        */
        Logger.getLogger(CrlCache.class).setLevel(Level.ALL);
        Logger.getLogger(ScheduledExecutor.class).setLevel(Level.ALL);
        Logger.getLogger(CrlInstance.class).setLevel(Level.ALL);
    }


    /**
     * Test crl for revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc_revoked");
        assertNotNull("Certificate not found", cert);

        IRevocationLookup lookup = buildCrlLookup();
        RevocationResponse response = lookup.checkCertificate(cert);

        assertFalse(response.isValid());
    }

    /*
      * Extracted to be overwritten in SimpleCrlLookupTest
      */
    protected CrlLookup buildCrlLookup() {
        return new CrlLookup();
    }

    /**
     * Test crl for non-revoked certificate.
     *
     * @throws Exception On error...
     */
    public void testNonRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc");

        IRevocationLookup lookup = buildCrlLookup();
        RevocationResponse response = lookup.checkCertificate(cert);

        assertTrue(response.isValid());
    }

    public void testExpiredButNotRevoked() throws Exception {
        X509Certificate cert = getCertificate("server_tdc_expired");

        IRevocationLookup lookup = buildCrlLookup();
        RevocationResponse response = lookup.checkCertificate(cert);

        assertTrue(response.isValid());
    }

    public void testFoces2NotExpiredNotRevoked() throws Exception {
        X509Certificate cert = getCertificate("tu_generel_foces2");

        IRevocationLookup lookup = buildCrlLookup();
        RevocationResponse response = lookup.checkCertificate(cert);

        assertTrue(response.isValid());
    }

    public void testFoces2Revoked() throws Exception {
        X509Certificate cert = getCertificate("tu_generel_foces2_revoked");

        IRevocationLookup lookup = buildCrlLookup();
        RevocationResponse response = lookup.checkCertificate(cert);

        assertFalse(response.isValid());
    }

    public void testFoces2Expired() throws Exception {
        X509Certificate cert = getCertificate("tu_generel_foces2_expired");

        IRevocationLookup lookup = buildCrlLookup();
        RevocationResponse response = lookup.checkCertificate(cert);

        // NOTE - the certificate is not revoked, even though it is expired
        // So this test should show not revoked
        // TODO: Why is revoked cert. set på return isValid() ??
        assertTrue("Wrong test result", response.isValid());
    }

    public void testLookTestMultiThread() throws InterruptedException {
        List<Thread> threads = new ArrayList<Thread>();
        for (int i = 0; i < 32; i++) {
            Runnable runnable = new ThreadCertificateCheck();
            Thread thread = new Thread(runnable);
            thread.start();
            threads.add(thread);
        }

        boolean run = true;
        while (run) {
            boolean anyAlive = false;
            for (Thread thread : threads) {
                anyAlive = thread.isAlive();
                if (anyAlive) {
                    break;
                }
            }
            if (anyAlive) {
                Thread.sleep(1000);
                continue;
            }
            run = false;
        }
    }

    public void testRevocationSourceType() {
        assertEquals("Wrong type returned", IRevocationLookup.RevocationSourceType.CRL, buildCrlLookup().getRevocationSourceType());
    }

    class ThreadCertificateCheck implements Runnable {
        public void run() {
            try {
                CrlLookup crlLookup = new CrlLookup();
                for (int i = 0; i < 16; i++) {
                    Random random = new Random();
                    int select = random.nextInt(2);
                    if (select < 1) {
//                        System.out.println("{0} ThreadCertificateCheck number:{1} certificate:0");
                        X509Certificate cert = getCertificate("server_tdc");
                        RevocationResponse response = crlLookup.checkCertificate(cert);
                        assertTrue(response.isValid());
                    } else {
//                        System.out.println("{0} ThreadCertificateCheck number:{1} certificate:1");
                        X509Certificate cert = getCertificate("server_tdc_revoked");
                        RevocationResponse response = crlLookup.checkCertificate(cert);
                        assertFalse(response.isValid());
                    }
//                    System.out.println("{0} ThreadCertificateCheck number:{1} done");
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
     * Read certificate from keystore.
     *
     * @param keyLabel Key on the certificate to load from JKS.
     * @return X509Certificate
     * @throws Exception On error...
     */
    public static X509Certificate getCertificate(String keyLabel) throws Exception {
        // define receiver certificate
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream(TestConstants.PATH_KEYSTORE_TEST), TestConstants.PASSWORD_KEYSTORE_TEST.toCharArray());

        return (X509Certificate) ks.getCertificate(keyLabel);
    }
}