package dk.gov.oiosi.test.util;

import java.io.Serializable;
import java.util.*;

/**
 * Simple class for easier duration calculation - tick() is called to remember
 * start time, tock(text) to remember duration for given text.<br>
 *
 * If tock(text) is called several times for the same text, it calculates
 * average duration.<br>
 *
 * @author Dmitriy Lapko, TrueLink A/S
 * @since OIORASP 1.3.0
 */
public class TickTock implements Serializable {

	private static final long serialVersionUID = 3755719105124643546L;

	protected long start;
	protected long totalStart;

	protected Map<String, Long> averageMap;
	protected HashMap<String, Integer> attemptMap;
	protected Set<String> rowSet;

	public TickTock() {
		averageMap = new HashMap<String, Long>(10);
		attemptMap = new HashMap<String, Integer>(10);
		rowSet = new LinkedHashSet<String>(10);
		totalStart = System.currentTimeMillis();
		tick();
	}

	/**
	 * In case if the same object is reused across several runs, this method
	 * allows to get correct total time for last run
	 */
	public void restartTotal() {
		totalStart = System.currentTimeMillis();
	}

	/**
	 * Remember start time for next tock()
	 */
	public void tick() {
		start = System.currentTimeMillis();
	}

	/**
	 * Clear all remembered information except order of columns
	 */
	public void clearAverage() {
		averageMap.clear();
		attemptMap.clear();
		restartTotal();
		tick();
	}

	/**
	 * Combination of tock and tick
	 *
	 * @return current duration (not average)
	 */
	public long tockTick(String text) {
		long curDuration = tock(text);
		tick();
		return curDuration;
	}

	/**
	 * @return current duration (not average)
	 */
	public long tock(String text) {
		long duration = (System.currentTimeMillis() - start);
		updateAverage(text, duration);
		return duration;
	}

	protected void updateAverage(String text, long duration) {
		rowSet.add(text);
		Long average = averageMap.get(text);
		if (average == null) {
			average = duration;
			attemptMap.put(text, 1);
		} else {
			int attemptCount = attemptMap.get(text);
			average = Math.round((average.intValue() * attemptCount + duration) / (attemptCount + 1.0));
			attemptMap.put(text, attemptCount + 1);
		}
		averageMap.put(text, average);
	}

	/**
	 * Finish - calculate total duration
	 */
	public void finish() {
		updateAverage("total", (System.currentTimeMillis() - totalStart));
	}

	/**
	 * @return tab delimited list of data in order, with which data were added
	 */
	public String toStatusString() {
		StringBuilder sb = new StringBuilder();
		for (Iterator<String> iterator = rowSet.iterator(); iterator.hasNext();) {
			String title = iterator.next();
			Long value = averageMap.get(title);
			sb.append(value);
			sb.append("\t");
		}
		return sb.toString();
	}

	/**
	 * @return tab delimited list of titles in order, with which data were added
	 */
	public String toHeaderString() {
		StringBuilder sb = new StringBuilder();
		for (Iterator<String> iterator = rowSet.iterator(); iterator.hasNext();) {
			String title = iterator.next();
			sb.append(title);
			sb.append("\t");
		}
		return sb.toString();
	}

	public String toStatusTableString() {
		StringBuilder sb = new StringBuilder();
		for (Iterator<String> iterator = rowSet.iterator(); iterator.hasNext();) {
			String title = iterator.next();
			Long value = averageMap.get(title);
			sb.append(title);
			sb.append("\t");
			sb.append(value);
			sb.append("\n");
		}
		return sb.toString();
	}

}
