package dk.gov.oiosi.uddi;

import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.addressing.IdentifierSe;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.configuration.ConfigurationHandlerCleaner;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import junit.framework.TestCase;

import java.util.*;

/**
 * Verify that we can do real UDDI lookups, based on RASP Lice conf.
 * <p/>
 * Tests might fail if registrations in UDDI/NHR change over time!
 */
public class UddiLookupClientTest extends TestCase {

    private Uri uddiServerUri;
    private UddiLookupClient lookupClient;

    private final Identifier eanIdentifier = new IdentifierEan("5798009811578");
//    private final Identifier dunsIdentifier = new IdentifierDuns("1234567890");

    private final UddiId documentId_nonExisting = new UddiGuidId("uddi:xyz-0123456789", true);
    private final UddiId serviceId_order = new UddiGuidId("uddi:b138dc71-d301-42d1-8c2e-2c3a26faf56a", true);
    private final UddiId documentId_invoice = new UddiGuidId("uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47", true);
//    private final UddiId documentId_applicationResponse = new UddiGuidId("uddi:42F92342-C3ED-46ff-8A8A-6518F55D5CD5", true);
//    private final UddiId documentId_creditNote = new UddiGuidId("uddi:e4ec9613-4830-4bab-afee-c37ab1c67aec", true);

    private final UddiId profile_nonExistingProfileId = new UddiGuidId("uddi:88fbd6d5-6a25-4c08-91cc-5344c73c1111", true);
    private final UddiId profile_nesubl_5 = new UddiGuidId("uddi:AEE8B6DE-298F-4cbc-A96D-9AE8AED0AC31", true);
    private final UddiId profile_BilSim = new UddiGuidId("uddi:362229ac-b657-452a-b8f8-c93e62c670ff", true);
    private final UddiId profile_OrdAdvBilSim = new UddiGuidId("uddi:88fbd6d5-6a25-4c08-91cc-5344c73c4d69", true);

    private final List<EndpointAddressTypeCode> acceptHttpProtocol = new LinkedList<EndpointAddressTypeCode>();
    private final List<EndpointAddressTypeCode> acceptSmtpProtocol = new LinkedList<EndpointAddressTypeCode>();

    public void setUp() throws Exception {
        super.setUp();

        acceptHttpProtocol.add(EndpointAddressTypeCode.http);
        acceptSmtpProtocol.add(EndpointAddressTypeCode.email);

        ConfigurationHandlerCleaner.clear();
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_PRODUCTION);

        UddiConfig config = ConfigurationHandler.getInstance().getUddiConfig();
        uddiServerUri = config.getLookupRegistryFallbackConfig().getRegistries().iterator().next().getEndpoints().iterator().next();
        lookupClient = new UddiLookupClient(uddiServerUri);
    }

    public void testLookingUpExistingServiceMustReturnResponseWithValidProperties() throws Exception {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, serviceId_order, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertNotNull("Response object must exist", lookupResponses);
        assertEquals("Exactly 1 endpoint expected -", 1, lookupResponses.size());

        UddiLookupResponse response = lookupResponses.get(0);
        assertResponsePropertiesAreSetCorrectly(response);
    }

    public void testLookingUpExistingServiceMustReturnCertificateSubjectString() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, serviceId_order, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);

        assertTrue(lookupResponses.size() > 0);

        // Notice, CertificateSubject class changes SERIALNUMBER to OID.2.5.4.5, which is why this is tested even though its not what is in UDDI
        String expectedCertificateSubjectString = "OID.2.5.4.5=CVR:26769388-FID:1200406941690 + CN=NemHandel test service (funktionscertifikat), O=IT- og Telestyrelsen // CVR:26769388, C=DK";
        String actualCertificateSubjectString = lookupResponses.get(0).getCertificateSubjectSerialNumber().getSubjectString();
        assertEquals(expectedCertificateSubjectString, actualCertificateSubjectString);
    }

    public void testLookingUpNonExistingServiceShouldReturnEmptyResponse() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, documentId_nonExisting, profileIds, acceptHttpProtocol);

        List<UddiLookupResponse> lookupResponses = lookupClient.lookup(lookupParameters);
        assertEquals("Wrong number of returned responses", 0, lookupResponses.size());
    }

    public void testLookingUpExistingServiceProvidingNonExistingRoleMustReturnEmptyResponseList() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_nonExistingProfileId);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, serviceId_order, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(0, lookupResponses.size());
    }

    public void testLookingUpServiceProvidingExistingRoleMustReturnResponse() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);

        String sellerPartyRoleIdentifier = "SellerParty";
        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, serviceId_order, profileIds, acceptHttpProtocol, sellerPartyRoleIdentifier);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(1, lookupResponses.size());
    }

    public void testLookingUpExistingServiceWithoutRoleMustReturnEmptyResponseList() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);

        String nonExistingRoleIdentifier = "NonExistingSellerParty";
        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, serviceId_order, profileIds, acceptHttpProtocol, nonExistingRoleIdentifier);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(0, lookupResponses.size());
    }

    public void testLookingUpExistingServiceTwoProfilesBothExistingMustReturnResponse() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);
        profileIds.add(profile_nesubl_5);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, documentId_invoice, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(1, lookupResponses.size());
    }

    public void testLookingUpServiceProvidingNoProfilesWhereServiceHasProfileMustReturnResponse() throws MainException {
        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, documentId_invoice, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(1, lookupResponses.size());
    }

    public void testLookingUpExistingServiceTwoProfilesOneExistingMustReturnResponse() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_nonExistingProfileId);
        profileIds.add(profile_nesubl_5);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, documentId_invoice, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(1, lookupResponses.size());
    }

    /*
    public void LookingUpExistingServiceWithDunsIdentifierShouldReturnResponse() {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);

        LookupParameters lookupParameters = new LookupParameters(dunsIdentifier, documentId_invoice, profileIds, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(1, lookupResponses.size());
    }   */

    public void testLookingUpHttpOnlyServiceWithAcceptedProtocolTypeSetToSmtpShouldReturnEmptyResponse() throws MainException {
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_OrdAdvBilSim);

        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, documentId_invoice, profileIds, acceptSmtpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertEquals(0, lookupResponses.size());
    }

    public void testLookingUpAllRegistrationsOnEanNumber() throws MainException {
        LookupParameters lookupParameters = new LookupParameters(eanIdentifier, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertTrue(lookupResponses.size() > 0);
    }

    public void testLookingUpAllRegistrationsOnEanNumber_2() throws MainException {
        UddiId uddiDocumentTypeId = new UddiGuidId("uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47", true);
        LookupParameters lookupParameters = new LookupParameters(new IdentifierEan("5798002567915"), uddiDocumentTypeId, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertTrue(lookupResponses.size() > 0);
    }

    public void testDK_SE_Lookup() throws Exception {
//        List<UddiId> profileIds = new LinkedList<UddiId>();
//        profileIds.add(profile_nesubl_5);

        Identifier ean = new IdentifierSe("30101189"); // Logstor: https://registration.nemhandel.dk/NemHandelRegisterWeb/public/participant/info?keytype=DK:SE&key=21330248
//        LookupParameters lookupParameters = new LookupParameters(ean, documentId_invoice, profileIds, acceptHttpProtocol);
        LookupParameters lookupParameters = new LookupParameters(ean, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertNotNull("Response object must exist", lookupResponses);
        assertEquals("Exactly 1 endpoint expected -", 6, lookupResponses.size());
    }

    /**
     * Reference to lookup page: https://registration.nemhandel.dk/NemHandelRegisterWeb/public/participant/info?keytype=GLN&key=5798002570595
     *
     * @throws Exception On error...
     */
    public void testNHRUddiInquiryInterface() throws Exception {
        String url = "http://discoverypublic.uddi.ehandel.gov.dk/registry/uddi/inquiry";
        if (!uddiServerUri.getAbsoluteUri().equalsIgnoreCase(url)) {
            uddiServerUri = new Uri(url);
            lookupClient = new UddiLookupClient(uddiServerUri);
        }
        List<UddiId> profileIds = new LinkedList<UddiId>();
        profileIds.add(profile_BilSim);

//        Identifier ean = new IdentifierEan("5790001659905"); // DR
//        Identifier ean = new IdentifierEan("5798009811578"); // ITST Test EAN
        Identifier ean = new IdentifierEan("5798009812957");
        LookupParameters lookupParameters = new LookupParameters(ean, documentId_invoice, profileIds, acceptHttpProtocol);
//        LookupParameters lookupParameters = new LookupParameters(ean, acceptHttpProtocol);
//        LookupParameters lookupParameters = new LookupParameters(ean);
//        long start = new Date().getTime();
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
//        printUddiLookupResponses(lookupResponses);
        assertNotNull("Response object must exist", lookupResponses);
        assertEquals("Exactly 1 endpoint expected -", 1, lookupResponses.size());
    }

    /**
     * NHR Test Case 01.04.06: Verify logging from UDDI Inquiry adapter in Splunk.
     *
     * @throws Exception On error...
     */
    public void testNHR_TestCase_01_04_06() throws Exception {
        uddiServerUri = new Uri("http://lookup-test.nemhandel.dk/uddi/services/inquiry");
        lookupClient = new UddiLookupClient(uddiServerUri);
        List<UddiId> profileIdList = new LinkedList<UddiId>();
        profileIdList.add(profile_BilSim);

        // Using an existing EAN (random number)
        Identifier ean = new IdentifierEan("5790001659905"); // DR
        LookupParameters lookupParameters = new LookupParameters(ean, documentId_invoice, profileIdList, acceptHttpProtocol);
        List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
        assertNotNull("Response object must exist", lookupResponses);
        assertEquals("Exactly 1 endpoint expected -", 1, lookupResponses.size());
//        System.err.println(lookupResponses.get(0));

        // Using non existing EAN
        ean = new IdentifierEan("5790011223344");
        lookupParameters = new LookupParameters(ean, documentId_invoice, profileIdList, acceptHttpProtocol);
        lookupResponses = performLookup(lookupParameters);
        assertNotNull("Response object must exist", lookupResponses);
        assertEquals("Exactly 1 endpoint expected -", 0, lookupResponses.size());
    }

//    public void testNHRUddiInquiryInterfacePerformance() throws Exception {
//        uddiServerUri = new Uri("http://lookup-test.nemhandel.dk/UddiInquiryAdapter/inquiry");
//        lookupClient = new UddiLookupClient(uddiServerUri);
//        List<UddiId> profileIds = new LinkedList<UddiId>();
//        profileIds.add(profile_BilSim);
//        String[] eanArray = new String[]{"5798009693549"
//                , "5798009694393"
//                , "5790002052897"
//                , "5790002052880"
//                , "5798009692283"
//                , "5798009172594"
//                , "5798009690036"
//                , "5798009690562"
//                , "5798009693402"
//                , "5798009692276"
//                , "5798009694782"
//                , "5798005485551"
//                , "5798008611186"
//                , "5798009690029"
//                , "5798008504938"
//                , "5798008504969"
//                , "5798008504952"
//                , "5798009693365"
//                , "5798009690128"
//                , "5798009690579"
//                , "5798009389367"
//                , "5798009389190"
//                , "5790000398966"
//                , "5798007934392"
//                , "5790001719814"
//                , "5798009696304"
//                , "5798008306983"
//                , "5798008210464"
//                , "5790001902025"
//                , "5790002052903"
//                , "5798009691385"
//                , "5798009693358"
//                , "5798009695239"
//                , "5798009692177"
//                , "5798009692368"
//                , "5798009691477"
//                , "5798009694386"
//                , "5790001902117"
//                , "5798009691774"
//                , "5798009691873"
//                , "5790001980900"
//                , "5790001980917"
//                , "5790001980955"
//                , "5790001980962"
//                , "5790001980948"
//                , "5790001980986"
//                , "5790001981020"
//                , "5798008359576"
//                , "5790001980993"
//                , "5790001981006"};
//
//        long start = new Date().getTime();
//        for (String eanNumber : eanArray) {
//            Identifier ean = new IdentifierEan(eanNumber);
//            LookupParameters lookupParameters = new LookupParameters(ean, documentId_creditNote, profileIds, acceptHttpProtocol);
////        LookupParameters lookupParameters = new LookupParameters(ean);
//            List<UddiLookupResponse> lookupResponses = performLookup(lookupParameters);
//
//        }
//        System.err.println("Lookup time: "+(new Date().getTime()-start)+" ms.");
////        assertNotNull("Response object must exist", lookupResponses);
////        assertEquals("Exactly 1 endpoint expected -", 1, lookupResponses.size());
//    }


    private void assertResponsePropertiesAreSetCorrectly(UddiLookupResponse response) {
        Date expectedActivationDate = new GregorianCalendar(2000, Calendar.JANUARY, 1).getTime();
        assertEquals(expectedActivationDate, response.getActivationDate());

        String expectedEndpoint = "http://raspservice.nemhandel.gov.dk/RASP/1.2/reciever.svc";
        assertEquals(expectedEndpoint, response.getEndpointAddress().getAsUri().getAbsoluteUri());

        String expectedIdentifierActual = "5798009811578";
        assertEquals(expectedIdentifierActual, response.getEndpointIdentifierActual().getAsString());

        Date expectedExpirationDate = new GregorianCalendar(2039, Calendar.DECEMBER, 31).getTime();
        assertEquals(expectedExpirationDate, response.getExpirationDate());

        assertEquals(false, response.hasNewerVersion());
        assertEquals(null, response.getNewerVersionReference());

        // Verify first entry in list
        assertEquals("Procurement-OrdAdv-BilSim-1.0 SellerParty", response.getProcessRoles().get(0).getName());
//        for (ProcessRoleDefinition processRoleDefinition : response.getProcessRoles()) {
//            assertEquals("Procurement-OrdAdv-BilSim-1.0 SellerParty", processRoleDefinition.getName());
//            break;
//        }

        assertEquals("Wrong email on registration", "support@nemhandel.dk", response.getServiceContactEmail().toString());
        assertEquals(null, response.getTermsOfUseUrl());

        Version expectedVersion = new Version(1, 0, 0);
        assertEquals(expectedVersion.toString(), response.getVersion().toString());
    }

    private List<UddiLookupResponse> performLookup(LookupParameters lookupParameters) throws MainException {
        return lookupClient.lookup(lookupParameters);
    }

    /**
     * Helper method to print all UddiLookupResponse in List.
     *
     * @param uddiLookupResponses List of UddiLookupResponses.
     */
//    private void printUddiLookupResponses(List<UddiLookupResponse> uddiLookupResponses) {
//        for (UddiLookupResponse uddiLookupResponse : uddiLookupResponses) {
//            System.out.println(uddiLookupResponse);
//        }
//    }
}