package dk.gov.oiosi.xml.schematron.research.bulk;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;
import java.util.zip.GZIPInputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;

import dk.gov.oiosi.xml.schematron.research.SchematronComparatorBulkResearchTest;


/**
 * Helper class for {@link SchematronComparatorBulkResearchTest}
 * 
 * @author Dmitriy Lapko, TrueLink A/S
 */
public class FileSystemCrowler {

	public static final Logger log = Logger.getLogger(FileSystemCrowler.class);

	private static final int BUFFER_SIZE = 50 * 1024;

	private final List<IFileListener> fileListenerList;
	private final List<IFolderListener> folderListenerList;

	public FileSystemCrowler(List<IFileListener> fileListenerList, List<IFolderListener> folderListenerList) {
		this.fileListenerList = fileListenerList;
		this.folderListenerList = folderListenerList;
	}

	public void processFolder(File folder) throws Exception {
		for (IFolderListener l : folderListenerList) {
			l.apply(folder.getName());
		}
		log.info("Folder " + folder.getAbsolutePath());
		File[] listFiles = folder.listFiles();
		for (int i = 0; i < listFiles.length; i++) {
			File file = listFiles[i];
			if (file.isDirectory()) {
				processFolder(file);
				continue;
			}
			String fileName = file.getName();

			boolean atLeastOneMatch = false;
			for (IFileListener l : fileListenerList) {
				if (l.isMatch(fileName)) {
					atLeastOneMatch = true;
					break;
				}
			}
			if (atLeastOneMatch) {
				byte[] data = readFile(file);
				for (IFileListener l : fileListenerList) {
					if (l.isMatch(fileName)) {
						l.apply(fileName, data);
					}
				}
			}
			
			if (fileName.endsWith(".zip.gz")) {
				processZipGz(file);
				continue;
			}
		}
	}

	private void processZipGz(File file) throws Exception {
		log.info("Archive " + file.getAbsolutePath());
		InputStream is = new GZIPInputStream(new FileInputStream(file), BUFFER_SIZE);
		File zipFile = File.createTempFile(this.getClass().getName(), "testArchivedProdDocuments.zip");
		log.info("Archive unpack to " + zipFile.getAbsolutePath() + "...");
		OutputStream os = null;
		try {
			os = new BufferedOutputStream(new FileOutputStream(zipFile));
			IOUtils.copy(is, os);
			os.flush();
		} catch (Exception e) {
			throw e;
		} finally {
			try {
				is.close();
			} catch (Exception e) {
			}
			if (os != null) {
				try {
					os.close();
				} catch (Exception e) {
				}
			}
		}

		log.info("Archive unpacked, size " + zipFile.length());

		ZipInputStream zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(zipFile), BUFFER_SIZE));
		try {
			ZipEntry entry = null;
			while ((entry = zis.getNextEntry()) != null) {
				String name = entry.getName();
				if (!entry.isDirectory()) {

					boolean atLeastOneMatch = false;
					for (IFileListener l : fileListenerList) {
						if (l.isMatch(name)) {
							atLeastOneMatch = true;
							break;
						}
					}
					if (atLeastOneMatch) {
						byte[] data = readZipEntry(zis, entry);
						for (IFileListener l : fileListenerList) {
							if (l.isMatch(name)) {
								l.apply(name, data);
							}
						}
					}
				}
			}
		} finally {
			try {
				zis.close();
			} catch (Exception e) {
			}
		}
		if (!zipFile.delete()) {
			zipFile.deleteOnExit();
		}
	}

	private static byte[] readZipEntry(ZipInputStream zis, ZipEntry entry) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream((int) entry.getSize());
		IOUtils.copy(zis, baos);
		byte[] xmlData = baos.toByteArray();
		return xmlData;
	}

	private static byte[] readFile(File file) throws FileNotFoundException, IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(BUFFER_SIZE);
		FileInputStream fis = new FileInputStream(file);
		IOUtils.copy(fis, baos);
		fis.close();
		byte[] data = baos.toByteArray();
		return data;
	}

}
