package dk.gov.oiosi.xml.schematron.sax;

import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schematron.*;
import junit.framework.TestCase;

import javax.xml.transform.Transformer;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.List;

/**
 * To be sure, that SchematronValidatorSAXUtil works correctly, it is enough to
 * check one OIOUBL document type and OIOXML - in current RaspConfiguration.xml
 * there is only 2 combinations of errorXPath and errorXPathMessage.
 * <p/>
 * Also it is good to check all available document examples to provide equal
 * results for both kinds of schematron validation.
 *
 * @author Dmitriy Lapko, TrueLink A/S
 */
public class SchematronValidatorSAXUtilTest extends TestCase {

    protected void setUp() throws Exception {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_RASPCONFIGURATION_TEST);
        ConfigurationHandler.getInstance();
    }

    /**
     * Test valid and invalid OIOUBL Invoice
     */
    public void testOIOUBLInvoice() throws Exception {
        assertEquals("[F-INV009] Invalid ID. Must contain a value", checkDocument(TestConstants.PATH_INVOICE_NO_ID));
        assertNull(checkDocument(TestConstants.PATH_INVOICE));
    }

    /**
     * Test valid and invalid OIOXML Invoice
     */
    public void testOIOXMLInvoice() throws Exception {
        assertEquals(":\n   WARNING: EAN numbers are 13 digits in length\n", checkDocument(TestConstants.PATH_INVOICE_INVALID_EAN_NUMBER));
        assertNull(checkDocument(TestConstants.PATH_INVOICE_PIE));
    }

    /**
     * Test all available test documents, defined in TestConstants class, whose
     * path ends with .xml
     */
    public void testCompareValidationResultsOnAllTestDocuments() throws Exception {
        List<String> excludedFieldList = Arrays.asList("PATH_RASPCONFIGURATION_PRODUCTION", "PATH_RASPCONFIGURATION_TEST", "PATH_NONAMESPACE_XML", "PATH_INVOICE_WRONG_NAME_SPACE", "PATH_UNKNOWNTYPE_XML");
        Field[] fields = TestConstants.class.getDeclaredFields();
        for (Field field : fields) {
            if (Modifier.isPublic(field.getModifiers()) && !excludedFieldList.contains(field.getName())) {
                Object value = field.get(null);
                if (value instanceof String) {
                    String s = (String) value;
                    if (s.endsWith(".xml")) {
                        checkDocument(s);
                    }
                }
            }
        }
    }

    private String checkDocument(String path) throws Exception {
        XmlDocument xd = new XmlDocument();
        xd.load(path);

        return checkDocument(xd);
    }

    private String checkDocument(XmlDocument xd) throws Exception {
        DocumentTypeConfigSearcher dts = new DocumentTypeConfigSearcher();
        DocumentTypeConfig dtc = dts.findUniqueDocumentType(xd);
        assertNotNull(dtc);

        SchematronValidationConfig svc = dtc.getSchematronValidationConfig();
        assertNotNull(svc);

        SchematronStore ss = SchematronStoreFactory.getSchematronStore();
        Transformer transformer = ss.getCompiledSchematron(svc.getSchematronDocumentPath());
        assertNotNull(transformer);

        String saxError = null;
        try {
            SchematronValidatorSAXUtil.schematronValidateXmlDocument(xd.getTransformationSource(), transformer, svc.getSaxHandlerConfig());
        } catch (SchematronErrorException sve) {
            saxError = sve.getMessage();
        }

        String oldError = null;
        try {
            SchematronValidator sv = new SchematronValidator(svc);
            sv.schematronValidateXmlDocument(xd, transformer, false);
        } catch (SchematronErrorException sve) {
            oldError = sve.getMessage();
        }
        assertEquals("Error string found by new impl. is not the same as old impl.", oldError, saxError);

        return saxError;
    }
}